@@ -1,6 +1,6 @@
 #ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
-#include <Windows.h>
+#include "Common/CommonWindows.h"
 #include <sys/stat.h>
 #else
 #include <dirent.h>
@@ -22,6 +22,7 @@
 #include "Common/File/Path.h"
 #include "Common/File/AndroidStorage.h"
 #include "Common/Data/Encoding/Utf8.h"
+#include "Common/TimeUtil.h"
 
 #if PPSSPP_PLATFORM(UWP)
 #include "UWP/UWPHelpers/StorageManager.h"
@@ -33,8 +34,7 @@ bool free_disk_space(const Path &path, int64_t &space) {
 #if PPSSPP_PLATFORM(UWP)
 	if (GetDriveFreeSpace(path, space)) {
 		return true;
-	}
-	else
+	} else
 #endif
 	if (GetDiskFreeSpaceExW(path.ToWString().c_str(), &free, nullptr, nullptr)) {
 		space = free.QuadPart;
@@ -43,7 +43,7 @@ bool free_disk_space(const Path &path, int64_t &space) {
 #else
 	if (path.Type() == PathType::CONTENT_URI) {
 		space = Android_GetFreeSpaceByContentUri(path.ToString());
-		INFO_LOG(COMMON, "Free space at '%s': %" PRIu64, path.c_str(), space);
+		INFO_LOG(Log::Common, "Free space at '%s': %" PRIu64, path.c_str(), space);
 		return space >= 0;
 	}
 
