@@ -122,9 +122,7 @@ public:
 		if (program) {
 			glDeleteProgram(program);
 		}
-		if (locData_) {
-			delete locData_;
-		}
+		delete locData_;
 	}
 	struct Semantic {
 		int location;
@@ -378,14 +376,6 @@ public:
 		deleter_.pushBuffers.push_back(pushbuffer);
 	}
 
-	void BeginPushBuffer(GLPushBuffer *pushbuffer) {
-		pushbuffer->Begin();
-	}
-
-	void EndPushBuffer(GLPushBuffer *pushbuffer) {
-		pushbuffer->End();
-	}
-
 	bool IsInRenderPass() const {
 		return curRenderStep_ && curRenderStep_->stepType == GLRStepType::RENDER;
 	}
@@ -421,7 +411,6 @@ public:
 		// an init command, that's for sure.
 		GLRInitStep &step = initSteps_.push_uninitialized();
 		step.stepType = GLRInitStepType::BUFFER_SUBDATA;
-		_dbg_assert_(offset >= 0);
 		_dbg_assert_(offset <= buffer->size_ - size);
 		step.buffer_subdata.buffer = buffer;
 		step.buffer_subdata.offset = (int)offset;
@@ -835,6 +824,7 @@ public:
 		}
 	}
 
+	void StartThread();  // Currently only used on iOS, since we fully recreate the context on Android
 	void StopThread();
 
 	bool SawOutOfMemory() {
@@ -852,6 +842,10 @@ public:
 		skipGLCalls_ = true;
 	}
 
+	int GetNumSteps() const {
+		return (int)steps_.size();
+	}
+
 private:
 	bool Run(GLRRenderThreadTask &task);
 
@@ -874,6 +868,7 @@ private:
 	FastVec<GLRInitStep> initSteps_;
 
 	// Execution time state
+	// TODO: Rename this, as we don't actually use a compile thread on OpenGL.
 	bool runCompileThread_ = true;
 
 	// Thread is managed elsewhere, and should call ThreadFrame.
