@@ -1,11 +1,14 @@
 #pragma once
 
+#include <cstdint>
 #include <cstring>
 #include <string>
+#include <string_view>
 #include <vector>
 #include <utility>
 #include <functional>
 
+#include "Common/Common.h"
 #include "Common/Log.h"
 #include "Common/GPU/Vulkan/VulkanLoader.h"
 #include "Common/GPU/Vulkan/VulkanDebug.h"
@@ -20,13 +23,15 @@
 #define VK_PROFILE_BEGIN(vulkan, cmd, stage, ...) vulkan->GetProfiler()->Begin(cmd, stage, __VA_ARGS__);
 #define VK_PROFILE_END(vulkan, cmd, stage) vulkan->GetProfiler()->End(cmd, stage);
 
-enum {
-	VULKAN_FLAG_VALIDATE = 1,
-	VULKAN_FLAG_PRESENT_MAILBOX = 2,
-	VULKAN_FLAG_PRESENT_IMMEDIATE = 4,
-	VULKAN_FLAG_PRESENT_FIFO_RELAXED = 8,
-	VULKAN_FLAG_PRESENT_FIFO = 16,
+enum class VulkanInitFlags : uint32_t {
+	VALIDATE = (1 << 0),
+	PRESENT_MAILBOX = (1 << 1),
+	PRESENT_IMMEDIATE = (1 << 2),
+	PRESENT_FIFO_RELAXED = (1 << 3),
+	PRESENT_FIFO = (1 << 4),
+	DISABLE_IMPLICIT_LAYERS = (1 << 5),
 };
+ENUM_CLASS_BITOPS(VulkanInitFlags);
 
 enum {
 	VULKAN_VENDOR_NVIDIA = 0x000010de,
@@ -44,6 +49,11 @@ VK_DEFINE_HANDLE(VmaAllocation);
 
 std::string VulkanVendorString(uint32_t vendorId);
 
+template<class R, class T> inline void ChainStruct(R &root, T *newStruct) {
+	newStruct->pNext = root.pNext;
+	root.pNext = newStruct;
+}
+
 // Not all will be usable on all platforms, of course...
 enum WindowSystem {
 #ifdef _WIN32
@@ -173,25 +183,32 @@ public:
 	struct CreateInfo {
 		const char *app_name;
 		int app_ver;
-		uint32_t flags;
+		VulkanInitFlags flags;
 	};
 
 	VkResult CreateInstance(const CreateInfo &info);
 	void DestroyInstance();
 
-	int GetBestPhysicalDevice();
-	int GetPhysicalDeviceByName(const std::string &name);
-	void ChooseDevice(int physical_device);
-	bool EnableInstanceExtension(const char *extension);
-	bool EnableDeviceExtension(const char *extension);
-	VkResult CreateDevice();
+	int GetBestPhysicalDevice() const;
+	int GetPhysicalDeviceByName(std::string_view name) const;
+
+	// Convenience method to avoid code duplication.
+	// If it returns false, delete the context.
+	bool CreateInstanceAndDevice(const CreateInfo &info);
+
+	// The coreVersion is to avoid enabling extensions that are merged into core Vulkan from a certain version.
+	bool EnableInstanceExtension(const char *extension, uint32_t coreVersion);
+	bool EnableDeviceExtension(const char *extension, uint32_t coreVersion);
+
+	// Was previously two functions, ChooseDevice and CreateDevice.
+	VkResult CreateDevice(int physical_device);
 
 	const std::string &InitError() const { return init_error_; }
 
 	VkDevice GetDevice() const { return device_; }
 	VkInstance GetInstance() const { return instance_; }
-	uint32_t GetFlags() const { return flags_; }
-	void UpdateFlags(uint32_t flags) { flags_ = flags; }
+	VulkanInitFlags GetInitFlags() const { return flags_; }
+	void UpdateInitFlags(VulkanInitFlags flags) { flags_ = flags; }
 
 	VulkanDeleteList &Delete() { return globalDeleteList_; }
 
@@ -215,9 +232,6 @@ public:
 	VkFence CreateFence(bool presignalled);
 	bool CreateShaderModule(const std::vector<uint32_t> &spirv, VkShaderModule *shaderModule, const char *tag);
 
-	int GetBackbufferWidth() { return (int)swapChainExtent_.width; }
-	int GetBackbufferHeight() { return (int)swapChainExtent_.height; }
-
 	void BeginFrame(VkCommandBuffer firstCommandBuffer);
 	void EndFrame();
 
@@ -272,6 +286,7 @@ public:
 		VkPhysicalDeviceMultiviewFeatures multiview;
 		VkPhysicalDevicePresentWaitFeaturesKHR presentWait;
 		VkPhysicalDevicePresentIdFeaturesKHR presentId;
+		VkPhysicalDeviceProvokingVertexFeaturesEXT provokingVertex;
 	};
 
 	const PhysicalDeviceProps &GetPhysicalDeviceProperties(int i = -1) const {
@@ -347,6 +362,7 @@ public:
 		// out of MAX_INFLIGHT_FRAMES.
 		return inflightFrames_;
 	}
+
 	// Don't call while a frame is in progress.
 	void UpdateInflightFrames(int n);
 
@@ -354,12 +370,13 @@ public:
 		return curFrame_;
 	}
 
-	VkSwapchainKHR GetSwapchain() const {
-		return swapchain_;
-	}
-	VkFormat GetSwapchainFormat() const {
-		return swapchainFormat_;
-	}
+	VkSwapchainKHR GetSwapchain() const { return swapchain_; }
+	VkFormat GetSwapchainFormat() const { return swapchainFormat_; }
+	bool IsSwapchainInited() const { return swapchainInited_; }
+	bool HasRealSwapchain() const { return swapChainExtent_.width > 0; }
+
+	int GetBackbufferWidth() { return (int)swapChainExtent_.width; }
+	int GetBackbufferHeight() { return (int)swapChainExtent_.height; }
 
 	void SetProfilerEnabledPtr(bool *enabled) {
 		for (auto &frame : frame_) {
@@ -401,6 +418,14 @@ public:
 		return frame_[curFrame_].deleteList.GetLastDeleteCount();
 	}
 
+	u32 InstanceApiVersion() const {
+		return vulkanInstanceApiVersion_;
+	}
+
+	u32 DeviceApiVersion() const {
+		return vulkanDeviceApiVersion_;
+	}
+
 private:
 	bool ChooseQueue();
 
@@ -416,7 +441,7 @@ private:
 
 	bool CheckLayers(const std::vector<LayerProperties> &layer_props, const std::vector<const char *> &layer_names) const;
 
-	WindowSystem winsys_;
+	WindowSystem winsys_{};
 
 	// Don't use the real types here to avoid having to include platform-specific stuff
 	// that we really don't want in everything that uses VulkanContext.
@@ -428,6 +453,8 @@ private:
 	VkDevice device_ = VK_NULL_HANDLE;
 	VkQueue gfx_queue_ = VK_NULL_HANDLE;
 	VkSurfaceKHR surface_ = VK_NULL_HANDLE;
+	u32 vulkanInstanceApiVersion_ = 0;
+	u32 vulkanDeviceApiVersion_ = 0;
 
 	std::string init_error_;
 	std::vector<const char *> instance_layer_names_;
@@ -459,7 +486,7 @@ private:
 	// Swap chain extent
 	VkExtent2D swapChainExtent_{};
 
-	int flags_ = 0;
+	VulkanInitFlags flags_{};
 	PerfClass devicePerfClass_ = PerfClass::SLOW;
 
 	int inflightFrames_ = MAX_INFLIGHT_FRAMES;
@@ -479,16 +506,17 @@ private:
 	std::vector<VkDebugUtilsMessengerEXT> utils_callbacks;
 
 	VkSwapchainKHR swapchain_ = VK_NULL_HANDLE;
-	VkFormat swapchainFormat_;
+	VkFormat swapchainFormat_ = VK_FORMAT_UNDEFINED;
 
 	uint32_t queue_count = 0;
+	bool swapchainInited_ = false;
 
 	PhysicalDeviceFeatures deviceFeatures_;
 
 	VkSurfaceCapabilitiesKHR surfCapabilities_{};
 	std::vector<VkSurfaceFormatKHR> surfFormats_{};
 
-	VkPresentModeKHR presentMode_;
+	VkPresentModeKHR presentMode_ = VK_PRESENT_MODE_FIFO_KHR;
 	std::vector<VkPresentModeKHR> availablePresentModes_;
 
 	std::vector<VkCommandBuffer> cmdQueue_;
@@ -496,12 +524,6 @@ private:
 	VmaAllocator allocator_ = VK_NULL_HANDLE;
 };
 
-// Detailed control.
-void TransitionImageLayout2(VkCommandBuffer cmd, VkImage image, int baseMip, int mipLevels, int numLayers, VkImageAspectFlags aspectMask,
-	VkImageLayout oldImageLayout, VkImageLayout newImageLayout,
-	VkPipelineStageFlags srcStageMask, VkPipelineStageFlags dstStageMask,
-	VkAccessFlags srcAccessMask, VkAccessFlags dstAccessMask);
-
 // GLSL compiler
 void init_glslang();
 void finalize_glslang();
@@ -517,8 +539,10 @@ bool GLSLtoSPV(const VkShaderStageFlagBits shader_type, const char *sourceCode,
 const char *VulkanColorSpaceToString(VkColorSpaceKHR colorSpace);
 const char *VulkanFormatToString(VkFormat format);
 const char *VulkanPresentModeToString(VkPresentModeKHR presentMode);
+const char *VulkanImageLayoutToString(VkImageLayout imageLayout);
 
 std::string FormatDriverVersion(const VkPhysicalDeviceProperties &props);
+std::string FormatAPIVersion(u32 version);
 
 // Simple heuristic.
 bool IsHashMaliDriverVersion(const VkPhysicalDeviceProperties &props);
