@@ -4,7 +4,6 @@
 // Only draws and binds are handled here, resource creation and allocations are handled as normal -
 // that's the nice thing with Vulkan.
 
-#include <algorithm>
 #include <atomic>
 #include <condition_variable>
 #include <cstdint>
@@ -90,6 +89,7 @@ public:
 	VkDynamicState dynamicStates[6]{};
 	VkPipelineDynamicStateCreateInfo ds{ VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO };
 	VkPipelineRasterizationStateCreateInfo rs{ VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO };
+	VkPipelineRasterizationProvokingVertexStateCreateInfoEXT rs_provoking{ VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_PROVOKING_VERTEX_STATE_CREATE_INFO_EXT };
 
 	// Replaced the ShaderStageInfo with promises here so we can wait for compiles to finish.
 	Promise<VkShaderModule> *vertexShader = nullptr;
@@ -102,7 +102,7 @@ public:
 	std::string fragmentShaderSource;
 	std::string geometryShaderSource;
 
-	VkPrimitiveTopology topology;
+	VkPrimitiveTopology topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST;
 	VkVertexInputAttributeDescription attrs[8]{};
 	VkVertexInputBindingDescription ibd{};
 	VkPipelineVertexInputStateCreateInfo vis{ VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO };
@@ -136,11 +136,11 @@ struct VKRGraphicsPipeline {
 
 	VKRGraphicsPipelineDesc *desc = nullptr;
 	Promise<VkPipeline> *pipeline[(size_t)RenderPassType::TYPE_COUNT]{};
+	std::mutex mutex_;  // protects the pipeline array
 
 	VkSampleCountFlagBits SampleCount() const { return sampleCount_; }
 
 	const char *Tag() const { return tag_.c_str(); }
-
 private:
 	void DestroyVariantsInstant(VkDevice device);
 
@@ -158,7 +158,7 @@ struct CompileQueueEntry {
 	Type type;
 	VkRenderPass compatibleRenderPass;
 	RenderPassType renderPassType;
-	VKRGraphicsPipeline *graphics = nullptr;
+	VKRGraphicsPipeline* graphics = nullptr;
 	VkSampleCountFlagBits sampleCount;
 };
 
@@ -283,10 +283,16 @@ public:
 
 	void ReportBadStateForDraw();
 
+	int WaitForPipelines();
+
 	void NudgeCompilerThread() {
-		compileMutex_.lock();
+		compileQueueMutex_.lock();
 		compileCond_.notify_one();
-		compileMutex_.unlock();
+		compileQueueMutex_.unlock();
+	}
+
+	void AssertInRenderPass() const {
+		_dbg_assert_(curRenderStep_ && curRenderStep_->stepType == VKRStepType::RENDER);
 	}
 
 	// This is the first call in a draw operation. Instead of asserting like we used to, you can now check the
@@ -389,7 +395,7 @@ public:
 		data.blendColor.color = color;
 	}
 
-	void PushConstants(VkPipelineLayout pipelineLayout, VkShaderStageFlags stages, int offset, int size, void *constants) {
+	void PushConstants(VkShaderStageFlags stages, int offset, int size, void *constants) {
 		_dbg_assert_(curRenderStep_ && curRenderStep_->stepType == VKRStepType::RENDER);
 		_dbg_assert_(size + offset < 40);
 		VkRenderData &data = curRenderStep_->commands.push_uninitialized();
@@ -533,6 +539,10 @@ public:
 	void StartThreads();
 	void StopThreads();
 
+	size_t GetNumSteps() const {
+		return steps_.size();
+	}
+
 private:
 	void EndCurRenderStep();
 
@@ -550,6 +560,9 @@ private:
 	void ResetDescriptorLists(int frame);
 	void FlushDescriptors(int frame);
 
+	void SanityCheckPassesOnAdd();
+	bool CreateSwapchainViewsAndDepth(VkCommandBuffer cmdInit, VulkanBarrierBatch *barriers, FrameDataShared &frameDataShared);
+
 	FrameDataShared frameDataShared_;
 
 	FrameData frameData_[VulkanContext::MAX_INFLIGHT_FRAMES];
@@ -570,7 +583,7 @@ private:
 
 	bool insideFrame_ = false;
 	// probably doesn't need to be atomic.
-	std::atomic<bool> runCompileThread_;
+	std::atomic<bool> runCompileThread_{};
 
 	bool useRenderThread_ = true;
 	bool measurePresentTime_ = false;
@@ -604,7 +617,7 @@ private:
 	std::thread compileThread_;
 	// Sync
 	std::condition_variable compileCond_;
-	std::mutex compileMutex_;
+	std::mutex compileQueueMutex_;
 	std::vector<CompileQueueEntry> compileQueue_;
 
 	// Thread for measuring presentation delay.
