@@ -148,7 +148,7 @@ public:
 
 class VKRasterState : public RasterState {
 public:
-	VKRasterState(VulkanContext *vulkan, const RasterStateDesc &desc) {
+	VKRasterState(const RasterStateDesc &desc) {
 		cullFace = desc.cull;
 		frontFace = desc.frontFace;
 	}
@@ -187,7 +187,7 @@ public:
 	VKShaderModule(ShaderStage stage, const std::string &tag) : stage_(stage), tag_(tag) {
 		vkstage_ = StageToVulkan(stage);
 	}
-	bool Compile(VulkanContext *vulkan, ShaderLanguage language, const uint8_t *data, size_t size);
+	bool Compile(VulkanContext *vulkan, const uint8_t *data, size_t size);
 	const std::string &GetSource() const { return source_; }
 	~VKShaderModule() {
 		if (module_) {
@@ -214,14 +214,14 @@ private:
 	std::string tag_;
 };
 
-bool VKShaderModule::Compile(VulkanContext *vulkan, ShaderLanguage language, const uint8_t *data, size_t size) {
+bool VKShaderModule::Compile(VulkanContext *vulkan, const uint8_t *data, size_t size) {
 	// We'll need this to free it later.
 	vulkan_ = vulkan;
 	source_ = (const char *)data;
 	std::vector<uint32_t> spirv;
 	std::string errorMessage;
 	if (!GLSLtoSPV(vkstage_, source_.c_str(), GLSLVariant::VULKAN, spirv, &errorMessage)) {
-		WARN_LOG(G3D, "Shader compile to module failed (%s): %s", tag_.c_str(), errorMessage.c_str());
+		WARN_LOG(Log::G3D, "Shader compile to module failed (%s): %s", tag_.c_str(), errorMessage.c_str());
 		return false;
 	}
 
@@ -239,7 +239,7 @@ bool VKShaderModule::Compile(VulkanContext *vulkan, ShaderLanguage language, con
 		module_ = Promise<VkShaderModule>::AlreadyDone(shaderModule);
 		ok_ = true;
 	} else {
-		WARN_LOG(G3D, "vkCreateShaderModule failed (%s)", tag_.c_str());
+		WARN_LOG(Log::G3D, "vkCreateShaderModule failed (%s)", tag_.c_str());
 		ok_ = false;
 	}
 	return ok_;
@@ -271,7 +271,7 @@ public:
 	}
 
 	void SetDynamicUniformData(const void *data, size_t size) {
-		_dbg_assert_(size <= uboSize_);
+		_dbg_assert_((int)size <= uboSize_);
 		memcpy(ubo_, data, size);
 	}
 
@@ -327,7 +327,7 @@ struct DescriptorSetKey {
 
 class VKTexture : public Texture {
 public:
-	VKTexture(VulkanContext *vulkan, VkCommandBuffer cmd, VulkanPushPool *pushBuffer, const TextureDesc &desc)
+	VKTexture(VulkanContext *vulkan, const TextureDesc &desc)
 		: vulkan_(vulkan), mipLevels_(desc.mipLevels) {
 		format_ = desc.format;
 	}
@@ -380,6 +380,13 @@ public:
 	VKContext(VulkanContext *vulkan, bool useRenderThread);
 	~VKContext();
 
+	BackendState GetCurrentBackendState() const override {
+		return BackendState{
+			(u32)renderManager_.GetNumSteps(),
+			true,  // Means that the other value is meaningful.
+		};
+	}
+
 	void DebugAnnotate(const char *annotation) override;
 	void Wait() override {
 		vkDeviceWaitIdle(vulkan_->GetDevice());
@@ -391,7 +398,7 @@ public:
 	std::vector<std::string> GetDeviceList() const override {
 		std::vector<std::string> list;
 		for (int i = 0; i < vulkan_->GetNumPhysicalDevices(); i++) {
-			list.push_back(vulkan_->GetPhysicalDeviceProperties(i).properties.deviceName);
+			list.emplace_back(vulkan_->GetPhysicalDeviceProperties(i).properties.deviceName);
 		}
 		return list;
 	}
@@ -445,14 +452,14 @@ public:
 	void UpdateBuffer(Buffer *buffer, const uint8_t *data, size_t offset, size_t size, UpdateBufferFlags flags) override;
 	void UpdateTextureLevels(Texture *texture, const uint8_t **data, TextureCallback initDataCallback, int numLevels) override;
 
-	void CopyFramebufferImage(Framebuffer *src, int level, int x, int y, int z, Framebuffer *dst, int dstLevel, int dstX, int dstY, int dstZ, int width, int height, int depth, int channelBits, const char *tag) override;
-	bool BlitFramebuffer(Framebuffer *src, int srcX1, int srcY1, int srcX2, int srcY2, Framebuffer *dst, int dstX1, int dstY1, int dstX2, int dstY2, int channelBits, FBBlitFilter filter, const char *tag) override;
-	bool CopyFramebufferToMemory(Framebuffer *src, int channelBits, int x, int y, int w, int h, Draw::DataFormat format, void *pixels, int pixelStride, ReadbackMode mode, const char *tag) override;
+	void CopyFramebufferImage(Framebuffer *src, int level, int x, int y, int z, Framebuffer *dst, int dstLevel, int dstX, int dstY, int dstZ, int width, int height, int depth, Aspect aspects, const char *tag) override;
+	bool BlitFramebuffer(Framebuffer *src, int srcX1, int srcY1, int srcX2, int srcY2, Framebuffer *dst, int dstX1, int dstY1, int dstX2, int dstY2, Aspect aspects, FBBlitFilter filter, const char *tag) override;
+	bool CopyFramebufferToMemory(Framebuffer *src, Aspect aspects, int x, int y, int w, int h, Draw::DataFormat format, void *pixels, int pixelStride, ReadbackMode mode, const char *tag) override;
 	DataFormat PreferredFramebufferReadbackFormat(Framebuffer *src) override;
 
 	// These functions should be self explanatory.
 	void BindFramebufferAsRenderTarget(Framebuffer *fbo, const RenderPassInfo &rp, const char *tag) override;
-	void BindFramebufferAsTexture(Framebuffer *fbo, int binding, FBChannel channelBit, int layer) override;
+	void BindFramebufferAsTexture(Framebuffer *fbo, int binding, Aspect channelBit, int layer) override;
 
 	void GetFramebufferDimensions(Framebuffer *fbo, int *w, int *h) override;
 
@@ -484,11 +491,14 @@ public:
 	void Draw(int vertexCount, int offset) override;
 	void DrawIndexed(int vertexCount, int offset) override;
 	void DrawUP(const void *vdata, int vertexCount) override;
+	void DrawIndexedUP(const void *vdata, int vertexCount, const void *idata, int indexCount) override;
+	// Specialized for quick IMGUI drawing.
+	void DrawIndexedClippedBatchUP(const void *vdata, int vertexCount, const void *idata, int indexCount, Slice<ClippedDraw>, const void *dynUniforms, size_t size) override;
 
 	void BindCurrentPipeline();
 	void ApplyDynamicState();
 
-	void Clear(int mask, uint32_t colorval, float depthVal, int stencilVal) override;
+	void Clear(Aspect aspects, uint32_t colorval, float depthVal, int stencilVal) override;
 
 	void BeginFrame(DebugFlags debugFlags) override;
 	void EndFrame() override;
@@ -515,16 +525,13 @@ public:
 	std::string GetInfoString(InfoField info) const override {
 		// TODO: Make these actually query the right information
 		switch (info) {
-		case APINAME: return "Vulkan";
-		case VENDORSTRING: return vulkan_->GetPhysicalDeviceProperties().properties.deviceName;
-		case VENDOR: return VulkanVendorString(vulkan_->GetPhysicalDeviceProperties().properties.vendorID);
-		case DRIVER: return FormatDriverVersion(vulkan_->GetPhysicalDeviceProperties().properties);
-		case SHADELANGVERSION: return "N/A";;
-		case APIVERSION: 
-		{
-			uint32_t ver = vulkan_->GetPhysicalDeviceProperties().properties.apiVersion;
-			return StringFromFormat("%d.%d.%d", ver >> 22, (ver >> 12) & 0x3ff, ver & 0xfff);
-		}
+		case InfoField::APINAME: return "Vulkan";
+		case InfoField::VENDORSTRING: return vulkan_->GetPhysicalDeviceProperties().properties.deviceName;
+		case InfoField::VENDOR: return VulkanVendorString(vulkan_->GetPhysicalDeviceProperties().properties.vendorID);
+		case InfoField::DRIVER: return FormatDriverVersion(vulkan_->GetPhysicalDeviceProperties().properties);
+		case InfoField::SHADELANGVERSION: return "N/A";;
+		case InfoField::APIVERSION: return FormatAPIVersion(vulkan_->InstanceApiVersion());
+		case InfoField::DEVICE_API_VERSION: return FormatAPIVersion(vulkan_->DeviceApiVersion());
 		default: return "?";
 		}
 	}
@@ -540,7 +547,7 @@ public:
 
 	void Invalidate(InvalidationFlags flags) override;
 
-	void InvalidateFramebuffer(FBInvalidationStage stage, uint32_t channels) override;
+	void InvalidateFramebuffer(FBInvalidationStage stage, Aspect aspects) override;
 
 	void SetInvalidationCallback(InvalidationCallback callback) override {
 		renderManager_.SetInvalidationCallback(callback);
@@ -589,14 +596,18 @@ private:
 };
 
 // Bits per pixel, not bytes.
+// VERY incomplete!
 static int GetBpp(VkFormat format) {
 	switch (format) {
+	case VK_FORMAT_R32_SFLOAT:
 	case VK_FORMAT_R8G8B8A8_UNORM:
 	case VK_FORMAT_B8G8R8A8_UNORM:
 		return 32;
 	case VK_FORMAT_R8_UNORM:
+	case VK_FORMAT_S8_UINT:
 		return 8;
 	case VK_FORMAT_R8G8_UNORM:
+	case VK_FORMAT_R16_SFLOAT:
 	case VK_FORMAT_R16_UNORM:
 		return 16;
 	case VK_FORMAT_R4G4B4A4_UNORM_PACK16:
@@ -699,8 +710,10 @@ VulkanTexture *VKContext::GetNullTexture() {
 		nullTexture_ = new VulkanTexture(vulkan_, "Null");
 		int w = 8;
 		int h = 8;
-		nullTexture_->CreateDirect(cmdInit, w, h, 1, 1, VK_FORMAT_A8B8G8R8_UNORM_PACK32, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
-			VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT);
+		VulkanBarrierBatch barrier;
+		nullTexture_->CreateDirect(w, h, 1, 1, VK_FORMAT_A8B8G8R8_UNORM_PACK32, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
+			VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT, &barrier);
+		barrier.Flush(cmdInit);
 		uint32_t bindOffset;
 		VkBuffer bindBuf;
 		uint32_t *data = (uint32_t *)push_->Allocate(w * h * 4, 4, &bindBuf, &bindOffset);
@@ -751,7 +764,7 @@ SamplerState *VKContext::CreateSamplerState(const SamplerStateDesc &desc) {
 }
 
 RasterState *VKContext::CreateRasterState(const RasterStateDesc &desc) {
-	return new VKRasterState(vulkan_, desc);
+	return new VKRasterState(desc);
 }
 
 void VKContext::BindSamplerStates(int start, int count, SamplerState **state) {
@@ -772,7 +785,7 @@ bool VKTexture::Create(VkCommandBuffer cmd, VulkanBarrierBatch *postBarriers, Vu
 	// Zero-sized textures not allowed.
 	_assert_(desc.width * desc.height * desc.depth > 0);  // remember to set depth to 1!
 	if (desc.width * desc.height * desc.depth <= 0) {
-		ERROR_LOG(G3D,  "Bad texture dimensions %dx%dx%d", desc.width, desc.height, desc.depth);
+		ERROR_LOG(Log::G3D,  "Bad texture dimensions %dx%dx%d", desc.width, desc.height, desc.depth);
 		return false;
 	}
 	_dbg_assert_(pushBuffer);
@@ -789,17 +802,27 @@ bool VKTexture::Create(VkCommandBuffer cmd, VulkanBarrierBatch *postBarriers, Vu
 		usageBits |= VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
 	}
 
-	VkComponentMapping r8AsAlpha[4] = { VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_R };
+	VkComponentMapping r8AsAlpha[4] = { {VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_ONE, VK_COMPONENT_SWIZZLE_R} };
+	VkComponentMapping r8AsColor[4] = { {VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_ONE} };
 
-	if (!vkTex_->CreateDirect(cmd, width_, height_, 1, mipLevels_, vulkanFormat, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, usageBits, desc.swizzle == TextureSwizzle::R8_AS_ALPHA ? r8AsAlpha : nullptr)) {
-		ERROR_LOG(G3D,  "Failed to create VulkanTexture: %dx%dx%d fmt %d, %d levels", width_, height_, depth_, (int)vulkanFormat, mipLevels_);
+	VkComponentMapping *swizzle = nullptr;
+	switch (desc.swizzle) {
+	case TextureSwizzle::R8_AS_ALPHA: swizzle = r8AsAlpha; break;
+	case TextureSwizzle::R8_AS_GRAYSCALE: swizzle = r8AsColor; break;
+	case TextureSwizzle::DEFAULT:
+		break;
+	}
+	VulkanBarrierBatch barrier;
+	if (!vkTex_->CreateDirect(width_, height_, 1, mipLevels_, vulkanFormat, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, usageBits, &barrier, swizzle)) {
+		ERROR_LOG(Log::G3D,  "Failed to create VulkanTexture: %dx%dx%d fmt %d, %d levels", width_, height_, depth_, (int)vulkanFormat, mipLevels_);
 		return false;
 	}
+	barrier.Flush(cmd);
 	VkImageLayout layout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
 	if (desc.initData.size()) {
 		UpdateInternal(cmd, pushBuffer, desc.initData.data(), desc.initDataCallback, (int)desc.initData.size());
 		// Generate the rest of the mips automatically.
-		if (desc.initData.size() < mipLevels_) {
+		if ((int)desc.initData.size() < mipLevels_) {
 			vkTex_->GenerateMips(cmd, (int)desc.initData.size(), false);
 			layout = VK_IMAGE_LAYOUT_GENERAL;
 		}
@@ -822,6 +845,7 @@ void VKTexture::UpdateInternal(VkCommandBuffer cmd, VulkanPushPool *pushBuffer,
 	int d = depth_;
 	VkFormat vulkanFormat = DataFormatToVulkan(format_);
 	int bpp = GetBpp(vulkanFormat);
+	_dbg_assert_(bpp != 0);
 	int bytesPerPixel = bpp / 8;
 	TextureCopyBatch batch;
 	batch.reserve(numLevels);
@@ -869,8 +893,7 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 	: vulkan_(vulkan), renderManager_(vulkan, useRenderThread, frameTimeHistory_) {
 	shaderLanguageDesc_.Init(GLSL_VULKAN);
 
-	VkFormat depthStencilFormat = vulkan->GetDeviceInfo().preferredDepthStencilFormat;
-
+	caps_.coordConvention = CoordConvention::Vulkan;
 	caps_.setMaxFrameLatencySupported = true;
 	caps_.anisoSupported = vulkan->GetDeviceFeatures().enabled.standard.samplerAnisotropy != 0;
 	caps_.geometryShaderSupported = vulkan->GetDeviceFeatures().enabled.standard.geometryShader != 0;
@@ -902,6 +925,9 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 	caps_.sampleRateShadingSupported = vulkan->GetDeviceFeatures().enabled.standard.sampleRateShading != 0;
 	caps_.textureSwizzleSupported = true;
 
+	// Note that it must also be enabled on the pipelines (which we do).
+	caps_.provokingVertexLast = vulkan->GetDeviceFeatures().enabled.provokingVertex.provokingVertexLast;
+
 	// Present mode stuff
 	caps_.presentMaxInterval = 1;
 	caps_.presentInstantModeChange = false;  // TODO: Fix this with some work in VulkanContext
@@ -929,7 +955,7 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 	case VULKAN_VENDOR_APPLE: caps_.vendor = GPUVendor::VENDOR_APPLE; break;
 	case VULKAN_VENDOR_MESA: caps_.vendor = GPUVendor::VENDOR_MESA; break;
 	default:
-		WARN_LOG(G3D, "Unknown vendor ID %08x", deviceProps.vendorID);
+		WARN_LOG(Log::G3D, "Unknown vendor ID %08x", deviceProps.vendorID);
 		caps_.vendor = GPUVendor::VENDOR_UNKNOWN;
 		break;
 	}
@@ -950,14 +976,6 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 		caps_.verySlowShaderCompiler = true;
 	}
 
-	// Hide D3D9 when we know it likely won't work well.
-#if PPSSPP_PLATFORM(WINDOWS)
-	caps_.supportsD3D9 = true;
-	if (!strcmp(deviceProps.deviceName, "Intel(R) Iris(R) Xe Graphics")) {
-		caps_.supportsD3D9 = false;
-	}
-#endif
-
 	// VkSampleCountFlagBits is arranged correctly for our purposes.
 	// Only support MSAA levels that have support for all three of color, depth, stencil.
 
@@ -966,8 +984,10 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
     caps_.deviceID = deviceProps.deviceID;
 
     if (caps_.vendor == GPUVendor::VENDOR_QUALCOMM) {
-		// if (caps_.deviceID < 0x6000000)  // On sub 6xx series GPUs, disallow multisample.
-		multisampleAllowed = false;  // Actually, let's disable it on them all for now. See issue #18818.
+		if (caps_.deviceID < 0x6000000) { // On sub 6xx series GPUs, disallow multisample.
+			INFO_LOG(Log::G3D, "Multisampling was disabled due to old driver version (Adreno)");
+			multisampleAllowed = false;
+		}
 
 		// Adreno 5xx devices, all known driver versions, fail to discard stencil when depth write is off.
 		// See: https://github.com/hrydgard/ppsspp/pull/11684
@@ -1040,10 +1060,16 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 	}
 
 	if (!vulkan->Extensions().KHR_depth_stencil_resolve) {
-		INFO_LOG(G3D, "KHR_depth_stencil_resolve not supported, disabling multisampling");
+		INFO_LOG(Log::G3D, "KHR_depth_stencil_resolve not supported, disabling multisampling");
+		multisampleAllowed = false;
 	}
 
-	bugs_.Infest(Draw::Bugs::NO_DEPTH_CANNOT_DISCARD_STENCIL_MALI);
+	if (!vulkan->Extensions().KHR_create_renderpass2) {
+		WARN_LOG(Log::G3D, "KHR_create_renderpass2 not supported, disabling multisampling");
+		multisampleAllowed = false;
+	} else {
+		_dbg_assert_(vkCreateRenderPass2 != nullptr);
+	}
 
 	// We limit multisampling functionality to reasonably recent and known-good tiling GPUs.
 	if (multisampleAllowed) {
@@ -1053,9 +1079,10 @@ VKContext::VKContext(VulkanContext *vulkan, bool useRenderThread)
 		const auto &resolveProperties = vulkan->GetPhysicalDeviceProperties().depthStencilResolve;
 		if (((resolveProperties.supportedDepthResolveModes & resolveProperties.supportedStencilResolveModes) & VK_RESOLVE_MODE_SAMPLE_ZERO_BIT) != 0) {
 			caps_.multiSampleLevelsMask = (limits.framebufferColorSampleCounts & limits.framebufferDepthSampleCounts & limits.framebufferStencilSampleCounts);
-			INFO_LOG(G3D, "Multisample levels mask: %d", caps_.multiSampleLevelsMask);
+			INFO_LOG(Log::G3D, "Multisample levels mask: %d", caps_.multiSampleLevelsMask);
 		} else {
-			INFO_LOG(G3D, "Not enough depth/stencil resolve modes supported, disabling multisampling.");
+			INFO_LOG(Log::G3D, "Not enough depth/stencil resolve modes supported, disabling multisampling. Color: %d Depth: %d Stencil: %d",
+				limits.framebufferColorSampleCounts, limits.framebufferDepthSampleCounts, limits.framebufferStencilSampleCounts);
 			caps_.multiSampleLevelsMask = 1;
 		}
 	} else {
@@ -1138,7 +1165,6 @@ void VKContext::BindDescriptors(VkBuffer buf, PackedDescriptor descriptors[4]) {
 	descriptors[0].buffer.offset = 0;  // dynamic
 	descriptors[0].buffer.range = curPipeline_->GetUBOSize();
 
-	int numDescs = 1;
 	for (int i = 0; i < MAX_BOUND_TEXTURES; ++i) {
 		VkImageView view;
 		VkSampler sampler;
@@ -1187,7 +1213,7 @@ Pipeline *VKContext::CreateGraphicsPipeline(const PipelineDesc &desc, const char
 		} else if (vkshader->GetStage() == ShaderStage::Fragment) {
 			gDesc.fragmentShader = vkshader->Get();
 		} else {
-			ERROR_LOG(G3D, "Bad stage");
+			ERROR_LOG(Log::G3D, "Bad stage");
 			delete pipeline;
 			return nullptr;
 		}
@@ -1212,8 +1238,6 @@ Pipeline *VKContext::CreateGraphicsPipeline(const PipelineDesc &desc, const char
 
 	gDesc.dss = depth->info;
 
-	raster->ToVulkan(&gDesc.rs);
-
 	// Copy bindings from input layout.
 	gDesc.topology = primToVK[(int)desc.prim];
 
@@ -1232,9 +1256,13 @@ Pipeline *VKContext::CreateGraphicsPipeline(const PipelineDesc &desc, const char
 
 	gDesc.pipelineLayout = pipelineLayout_;
 
-	VkPipelineRasterizationStateCreateInfo rs{ VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO };
 	raster->ToVulkan(&gDesc.rs);
 
+	if (renderManager_.GetVulkanContext()->GetDeviceFeatures().enabled.provokingVertex.provokingVertexLast) {
+		ChainStruct(gDesc.rs, &gDesc.rs_provoking);
+		gDesc.rs_provoking.provokingVertexMode = VK_PROVOKING_VERTEX_MODE_LAST_VERTEX_EXT;
+	}
+
 	pipeline->pipeline = renderManager_.CreateGraphicsPipeline(&gDesc, pipelineFlags, 1 << (size_t)RenderPassType::BACKBUFFER, VK_SAMPLE_COUNT_1_BIT, false, tag ? tag : "thin3d");
 
 	if (desc.uniformDesc) {
@@ -1300,14 +1328,14 @@ Texture *VKContext::CreateTexture(const TextureDesc &desc) {
 	VkCommandBuffer initCmd = renderManager_.GetInitCmd();
 	if (!push_ || !initCmd) {
 		// Too early! Fail.
-		ERROR_LOG(G3D,  "Can't create textures before the first frame has started.");
+		ERROR_LOG(Log::G3D,  "Can't create textures before the first frame has started.");
 		return nullptr;
 	}
-	VKTexture *tex = new VKTexture(vulkan_, initCmd, push_, desc);
+	VKTexture *tex = new VKTexture(vulkan_, desc);
 	if (tex->Create(initCmd, &renderManager_.PostInitBarrier(), push_, desc)) {
 		return tex;
 	} else {
-		ERROR_LOG(G3D,  "Failed to create texture");
+		ERROR_LOG(Log::G3D,  "Failed to create texture");
 		tex->Release();
 		return nullptr;
 	}
@@ -1317,7 +1345,7 @@ void VKContext::UpdateTextureLevels(Texture *texture, const uint8_t **data, Text
 	VkCommandBuffer initCmd = renderManager_.GetInitCmd();
 	if (!push_ || !initCmd) {
 		// Too early! Fail.
-		ERROR_LOG(G3D, "Can't create textures before the first frame has started.");
+		ERROR_LOG(Log::G3D, "Can't create textures before the first frame has started.");
 		return;
 	}
 
@@ -1371,7 +1399,7 @@ BlendState *VKContext::CreateBlendState(const BlendStateDesc &desc) {
 // to avoid synchronization issues.
 class VKBuffer : public Buffer {
 public:
-	VKBuffer(size_t size, uint32_t flags) : dataSize_(size) {
+	VKBuffer(size_t size) : dataSize_(size) {
 		data_ = new uint8_t[size];
 	}
 	~VKBuffer() {
@@ -1386,7 +1414,7 @@ public:
 };
 
 Buffer *VKContext::CreateBuffer(size_t size, uint32_t usageFlags) {
-	return new VKBuffer(size, usageFlags);
+	return new VKBuffer(size);
 }
 
 void VKContext::UpdateBuffer(Buffer *buffer, const uint8_t *data, size_t offset, size_t size, UpdateBufferFlags flags) {
@@ -1426,10 +1454,10 @@ void VKContext::BindNativeTexture(int sampler, void *nativeTexture) {
 
 ShaderModule *VKContext::CreateShaderModule(ShaderStage stage, ShaderLanguage language, const uint8_t *data, size_t size, const char *tag) {
 	VKShaderModule *shader = new VKShaderModule(stage, tag);
-	if (shader->Compile(vulkan_, language, data, size)) {
+	if (shader->Compile(vulkan_, data, size)) {
 		return shader;
 	} else {
-		ERROR_LOG(G3D, "Failed to compile shader %s:\n%s", tag, (const char *)LineNumberString((const char *)data).c_str());
+		ERROR_LOG(Log::G3D, "Failed to compile shader %s:\n%s", tag, (const char *)LineNumberString((const char *)data).c_str());
 		shader->Release();
 		return nullptr;
 	}
@@ -1499,20 +1527,107 @@ void VKContext::DrawUP(const void *vdata, int vertexCount) {
 	int descSetIndex;
 	PackedDescriptor *descriptors = renderManager_.PushDescriptorSet(4, &descSetIndex);
 	BindDescriptors(vulkanUBObuf, descriptors);
-	renderManager_.Draw(descSetIndex, 1, &ubo_offset, vulkanVbuf, (int)vbBindOffset + curVBufferOffset_, vertexCount);
+	renderManager_.Draw(descSetIndex, 1, &ubo_offset, vulkanVbuf, (int)vbBindOffset, vertexCount);
+}
+
+void VKContext::DrawIndexedUP(const void *vdata, int vertexCount, const void *idata, int indexCount) {
+	_dbg_assert_(vertexCount >= 0);
+	_dbg_assert_(indexCount >= 0);
+	if (vertexCount <= 0 || indexCount <= 0) {
+		return;
+	}
+
+	VkBuffer vulkanVbuf, vulkanIbuf, vulkanUBObuf;
+	size_t vdataSize = vertexCount * curPipeline_->stride;
+	uint32_t vbBindOffset;
+	uint8_t *vdataPtr = push_->Allocate(vdataSize, 4, &vulkanVbuf, &vbBindOffset);
+	_assert_(vdataPtr != nullptr);
+	memcpy(vdataPtr, vdata, vdataSize);
+
+	size_t idataSize = indexCount * sizeof(u16);
+	uint32_t ibBindOffset;
+	uint8_t *idataPtr = push_->Allocate(idataSize, 4, &vulkanIbuf, &ibBindOffset);
+	_assert_(idataPtr != nullptr);
+	memcpy(idataPtr, vdata, idataSize);
+
+	uint32_t ubo_offset = (uint32_t)curPipeline_->PushUBO(push_, vulkan_, &vulkanUBObuf);
+
+	BindCurrentPipeline();
+	ApplyDynamicState();
+	int descSetIndex;
+	PackedDescriptor *descriptors = renderManager_.PushDescriptorSet(4, &descSetIndex);
+	BindDescriptors(vulkanUBObuf, descriptors);
+	renderManager_.DrawIndexed(descSetIndex, 1, &ubo_offset, vulkanVbuf, (int)vbBindOffset, vulkanIbuf, (int)ibBindOffset, indexCount, 1);
+}
+
+void VKContext::DrawIndexedClippedBatchUP(const void *vdata, int vertexCount, const void *idata, int indexCount, Slice<ClippedDraw> draws, const void *ub, size_t ubSize) {
+	_dbg_assert_(vertexCount >= 0);
+	_dbg_assert_(indexCount >= 0);
+	if (vertexCount <= 0 || indexCount <= 0 || draws.is_empty()) {
+		return;
+	}
+
+	curPipeline_ = (VKPipeline *)draws[0].pipeline;
+
+	VkBuffer vulkanVbuf, vulkanIbuf, vulkanUBObuf;
+	size_t vdataSize = vertexCount * curPipeline_->stride;
+	uint32_t vbBindOffset;
+	uint8_t *vdataPtr = push_->Allocate(vdataSize, 4, &vulkanVbuf, &vbBindOffset);
+	_assert_(vdataPtr != nullptr);
+	memcpy(vdataPtr, vdata, vdataSize);
+
+	constexpr int indexSize = sizeof(u16);
+
+	size_t idataSize = indexCount * indexSize;
+	uint32_t ibBindOffset;
+	uint8_t *idataPtr = push_->Allocate(idataSize, 4, &vulkanIbuf, &ibBindOffset);
+	_assert_(idataPtr != nullptr);
+	memcpy(idataPtr, idata, idataSize);
+
+	curPipeline_->SetDynamicUniformData(ub, ubSize);
+
+	uint32_t ubo_offset = (uint32_t)curPipeline_->PushUBO(push_, vulkan_, &vulkanUBObuf);
+
+	BindCurrentPipeline();
+	ApplyDynamicState();
+
+	for (auto &draw : draws) {
+		if (draw.pipeline != curPipeline_) {
+			VKPipeline *vkPipe = (VKPipeline *)draw.pipeline;
+			renderManager_.BindPipeline(vkPipe->pipeline, vkPipe->flags, pipelineLayout_);
+			curPipeline_ = (VKPipeline *)draw.pipeline;
+			curPipeline_->SetDynamicUniformData(ub, ubSize);
+		}
+		// TODO: Dirty-check these.
+		if (draw.bindTexture) {
+			BindTexture(0, draw.bindTexture);
+		} else if (draw.bindFramebufferAsTex) {
+			BindFramebufferAsTexture(draw.bindFramebufferAsTex, 0, draw.aspect, 0);
+		} else if (draw.bindNativeTexture) {
+			BindNativeTexture(0, draw.bindNativeTexture);
+		}
+		Draw::SamplerState *sstate = draw.samplerState;
+		BindSamplerStates(0, 1, &sstate);
+		int descSetIndex;
+		PackedDescriptor *descriptors = renderManager_.PushDescriptorSet(4, &descSetIndex);
+		BindDescriptors(vulkanUBObuf, descriptors);
+		renderManager_.SetScissor(draw.clipx, draw.clipy, draw.clipw, draw.cliph);
+		renderManager_.DrawIndexed(descSetIndex, 1, &ubo_offset, vulkanVbuf, (int)vbBindOffset, vulkanIbuf,
+			(int)ibBindOffset + draw.indexOffset * indexSize, draw.indexCount, 1);
+	}
 }
 
 void VKContext::BindCurrentPipeline() {
 	renderManager_.BindPipeline(curPipeline_->pipeline, curPipeline_->flags, pipelineLayout_);
 }
 
-void VKContext::Clear(int clearMask, uint32_t colorval, float depthVal, int stencilVal) {
+void VKContext::Clear(Aspect aspects, uint32_t colorval, float depthVal, int stencilVal) {
 	int mask = 0;
-	if (clearMask & FBChannel::FB_COLOR_BIT)
+	if (aspects & Aspect::COLOR_BIT)
 		mask |= VK_IMAGE_ASPECT_COLOR_BIT;
-	if (clearMask & FBChannel::FB_DEPTH_BIT)
+	if (aspects & Aspect::DEPTH_BIT)
 		mask |= VK_IMAGE_ASPECT_DEPTH_BIT;
-	if (clearMask & FBChannel::FB_STENCIL_BIT)
+	if (aspects & Aspect::STENCIL_BIT)
 		mask |= VK_IMAGE_ASPECT_STENCIL_BIT;
 	renderManager_.Clear(colorval, depthVal, stencilVal, mask);
 }
@@ -1555,6 +1670,7 @@ std::vector<std::string> VKContext::GetFeatureList() const {
 	AddFeature(features, "multiviewGeometryShader", vulkan_->GetDeviceFeatures().available.multiview.multiviewGeometryShader, vulkan_->GetDeviceFeatures().enabled.multiview.multiviewGeometryShader);
 	AddFeature(features, "presentId", vulkan_->GetDeviceFeatures().available.presentId.presentId, vulkan_->GetDeviceFeatures().enabled.presentId.presentId);
 	AddFeature(features, "presentWait", vulkan_->GetDeviceFeatures().available.presentWait.presentWait, vulkan_->GetDeviceFeatures().enabled.presentWait.presentWait);
+	AddFeature(features, "provokingVertexLast", vulkan_->GetDeviceFeatures().available.provokingVertex.provokingVertexLast, vulkan_->GetDeviceFeatures().enabled.provokingVertex.provokingVertexLast);
 
 	features.emplace_back(std::string("Preferred depth buffer format: ") + VulkanFormatToString(vulkan_->GetDeviceInfo().preferredDepthStencilFormat));
 
@@ -1630,6 +1746,9 @@ public:
 	void UpdateTag(const char *newTag) override {
 		buf_->UpdateTag(newTag);
 	}
+	const char *Tag() const override {
+		return buf_->Tag();
+	}
 private:
 	VKRFramebuffer *buf_;
 };
@@ -1640,43 +1759,42 @@ Framebuffer *VKContext::CreateFramebuffer(const FramebufferDesc &desc) {
 	_assert_(desc.width > 0);
 	_assert_(desc.height > 0);
 
-	VkCommandBuffer cmd = renderManager_.GetInitCmd();
-	VKRFramebuffer *vkrfb = new VKRFramebuffer(vulkan_, &renderManager_.PostInitBarrier(), cmd, renderManager_.GetQueueRunner()->GetCompatibleRenderPass(), desc.width, desc.height, desc.numLayers, desc.multiSampleLevel, desc.z_stencil, desc.tag);
+	VKRFramebuffer *vkrfb = new VKRFramebuffer(vulkan_, &renderManager_.PostInitBarrier(), desc.width, desc.height, desc.numLayers, desc.multiSampleLevel, desc.z_stencil, desc.tag);
 	return new VKFramebuffer(vkrfb, desc.multiSampleLevel);
 }
 
-void VKContext::CopyFramebufferImage(Framebuffer *srcfb, int level, int x, int y, int z, Framebuffer *dstfb, int dstLevel, int dstX, int dstY, int dstZ, int width, int height, int depth, int channelBits, const char *tag) {
+void VKContext::CopyFramebufferImage(Framebuffer *srcfb, int level, int x, int y, int z, Framebuffer *dstfb, int dstLevel, int dstX, int dstY, int dstZ, int width, int height, int depth, Aspect aspects, const char *tag) {
 	VKFramebuffer *src = (VKFramebuffer *)srcfb;
 	VKFramebuffer *dst = (VKFramebuffer *)dstfb;
 
 	int aspectMask = 0;
-	if (channelBits & FBChannel::FB_COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
-	if (channelBits & FBChannel::FB_DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
-	if (channelBits & FBChannel::FB_STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
+	if (aspects & Aspect::COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
+	if (aspects & Aspect::DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
+	if (aspects & Aspect::STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
 
 	renderManager_.CopyFramebuffer(src->GetFB(), VkRect2D{ {x, y}, {(uint32_t)width, (uint32_t)height } }, dst->GetFB(), VkOffset2D{ dstX, dstY }, aspectMask, tag);
 }
 
-bool VKContext::BlitFramebuffer(Framebuffer *srcfb, int srcX1, int srcY1, int srcX2, int srcY2, Framebuffer *dstfb, int dstX1, int dstY1, int dstX2, int dstY2, int channelBits, FBBlitFilter filter, const char *tag) {
+bool VKContext::BlitFramebuffer(Framebuffer *srcfb, int srcX1, int srcY1, int srcX2, int srcY2, Framebuffer *dstfb, int dstX1, int dstY1, int dstX2, int dstY2, Aspect aspects, FBBlitFilter filter, const char *tag) {
 	VKFramebuffer *src = (VKFramebuffer *)srcfb;
 	VKFramebuffer *dst = (VKFramebuffer *)dstfb;
 
 	int aspectMask = 0;
-	if (channelBits & FBChannel::FB_COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
-	if (channelBits & FBChannel::FB_DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
-	if (channelBits & FBChannel::FB_STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
+	if (aspects & Aspect::COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
+	if (aspects & Aspect::DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
+	if (aspects & Aspect::STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
 
 	renderManager_.BlitFramebuffer(src->GetFB(), VkRect2D{ {srcX1, srcY1}, {(uint32_t)(srcX2 - srcX1), (uint32_t)(srcY2 - srcY1) } }, dst->GetFB(), VkRect2D{ {dstX1, dstY1}, {(uint32_t)(dstX2 - dstX1), (uint32_t)(dstY2 - dstY1) } }, aspectMask, filter == FB_BLIT_LINEAR ? VK_FILTER_LINEAR : VK_FILTER_NEAREST, tag);
 	return true;
 }
 
-bool VKContext::CopyFramebufferToMemory(Framebuffer *srcfb, int channelBits, int x, int y, int w, int h, Draw::DataFormat format, void *pixels, int pixelStride, ReadbackMode mode, const char *tag) {
+bool VKContext::CopyFramebufferToMemory(Framebuffer *srcfb, Aspect aspects, int x, int y, int w, int h, Draw::DataFormat format, void *pixels, int pixelStride, ReadbackMode mode, const char *tag) {
 	VKFramebuffer *src = (VKFramebuffer *)srcfb;
 
 	int aspectMask = 0;
-	if (channelBits & FBChannel::FB_COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
-	if (channelBits & FBChannel::FB_DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
-	if (channelBits & FBChannel::FB_STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
+	if (aspects & Aspect::COLOR_BIT) aspectMask |= VK_IMAGE_ASPECT_COLOR_BIT;
+	if (aspects & Aspect::DEPTH_BIT) aspectMask |= VK_IMAGE_ASPECT_DEPTH_BIT;
+	if (aspects & Aspect::STENCIL_BIT) aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
 
 	return renderManager_.CopyFramebufferToMemory(src ? src->GetFB() : nullptr, aspectMask, x, y, w, h, format, (uint8_t *)pixels, pixelStride, mode, tag);
 }
@@ -1702,7 +1820,7 @@ void VKContext::BindFramebufferAsRenderTarget(Framebuffer *fbo, const RenderPass
 	curFramebuffer_ = fb;
 }
 
-void VKContext::BindFramebufferAsTexture(Framebuffer *fbo, int binding, FBChannel channelBit, int layer) {
+void VKContext::BindFramebufferAsTexture(Framebuffer *fbo, int binding, Aspect channelBit, int layer) {
 	VKFramebuffer *fb = (VKFramebuffer *)fbo;
 	_assert_(binding >= 0 && binding < MAX_BOUND_TEXTURES);
 
@@ -1712,13 +1830,14 @@ void VKContext::BindFramebufferAsTexture(Framebuffer *fbo, int binding, FBChanne
 
 	int aspect = 0;
 	switch (channelBit) {
-	case FBChannel::FB_COLOR_BIT:
+	case Aspect::COLOR_BIT:
 		aspect = VK_IMAGE_ASPECT_COLOR_BIT;
 		break;
-	case FBChannel::FB_DEPTH_BIT:
+	case Aspect::DEPTH_BIT:
 		aspect = VK_IMAGE_ASPECT_DEPTH_BIT;
 		break;
 	default:
+		// Hm, can we texture from stencil?
 		_assert_(false);
 		break;
 	}
@@ -1752,13 +1871,13 @@ void VKContext::HandleEvent(Event ev, int width, int height, void *param1, void
 	}
 }
 
-void VKContext::InvalidateFramebuffer(FBInvalidationStage stage, uint32_t channels) {
+void VKContext::InvalidateFramebuffer(FBInvalidationStage stage, Aspect aspects) {
 	VkImageAspectFlags flags = 0;
-	if (channels & FBChannel::FB_COLOR_BIT)
+	if (aspects & Aspect::COLOR_BIT)
 		flags |= VK_IMAGE_ASPECT_COLOR_BIT;
-	if (channels & FBChannel::FB_DEPTH_BIT)
+	if (aspects & Aspect::DEPTH_BIT)
 		flags |= VK_IMAGE_ASPECT_DEPTH_BIT;
-	if (channels & FBChannel::FB_STENCIL_BIT)
+	if (aspects & Aspect::STENCIL_BIT)
 		flags |= VK_IMAGE_ASPECT_STENCIL_BIT;
 	if (stage == FB_INVALIDATION_LOAD) {
 		renderManager_.SetLoadDontCare(flags);
