@@ -118,6 +118,25 @@ bool DataFormatIsBlockCompressed(DataFormat fmt, int *blockSize) {
 	}
 }
 
+int DataFormatNumChannels(DataFormat fmt) {
+	switch (fmt) {
+	case DataFormat::D16:
+	case DataFormat::D32F:
+	case DataFormat::R8_UNORM:
+	case DataFormat::R16_UNORM:
+	case DataFormat::R16_FLOAT:
+	case DataFormat::R32_FLOAT:
+		return 1;
+	case DataFormat::R8G8B8A8_UNORM:
+	case DataFormat::R8G8B8A8_UNORM_SRGB:
+	case DataFormat::B8G8R8A8_UNORM:
+	case DataFormat::B8G8R8A8_UNORM_SRGB:
+		return 4;
+	default:
+		return 0;
+	}
+}
+
 RefCountedObject::~RefCountedObject() {
 	const int rc = refcount_.load();
 	_dbg_assert_msg_(rc == 0xDEDEDE, "Unexpected refcount %d in object of type '%s'", rc, name_);
@@ -164,13 +183,6 @@ static const std::vector<ShaderSource> fsTexCol = {
 	"uniform sampler2D Sampler0;\n"
 	"void main() { gl_FragColor = texture2D(Sampler0, oTexCoord0) * oColor0; }\n"
 	},
-	{ShaderLanguage::HLSL_D3D9,
-	"struct PS_INPUT { float4 color : COLOR0; float2 uv : TEXCOORD0; };\n"
-	"sampler2D Sampler0 : register(s0);\n"
-	"float4 main(PS_INPUT input) : COLOR0 {\n"
-	"  return input.color * tex2D(Sampler0, input.uv);\n"
-	"}\n"
-	},
 	{ShaderLanguage::HLSL_D3D11,
 	"struct PS_INPUT { float4 color : COLOR0; float2 uv : TEXCOORD0; };\n"
 	"SamplerState samp : register(s0);\n"
@@ -208,13 +220,6 @@ static const std::vector<ShaderSource> fsTexColRBSwizzle = {
 	"uniform sampler2D Sampler0;\n"
 	"void main() { gl_FragColor = texture2D(Sampler0, oTexCoord0).zyxw * oColor0; }\n"
 	},
-	{ShaderLanguage::HLSL_D3D9,
-	"struct PS_INPUT { float4 color : COLOR0; float2 uv : TEXCOORD0; };\n"
-	"sampler2D Sampler0 : register(s0);\n"
-	"float4 main(PS_INPUT input) : COLOR0 {\n"
-	"  return input.color * tex2D(Sampler0, input.uv).zyxw;\n"
-	"}\n"
-	},
 	{ShaderLanguage::HLSL_D3D11,
 	"struct PS_INPUT { float4 color : COLOR0; float2 uv : TEXCOORD0; };\n"
 	"SamplerState samp : register(s0);\n"
@@ -249,12 +254,6 @@ static const std::vector<ShaderSource> fsCol = {
 	"varying vec4 oColor0;\n"
 	"void main() { gl_FragColor = oColor0; }\n"
 	},
-	{ ShaderLanguage::HLSL_D3D9,
-	"struct PS_INPUT { float4 color : COLOR0; };\n"
-	"float4 main(PS_INPUT input) : COLOR0 {\n"
-	"  return input.color;\n"
-	"}\n"
-	},
 	{ ShaderLanguage::HLSL_D3D11,
 	"struct PS_INPUT { float4 color : COLOR0; };\n"
 	"float4 main(PS_INPUT input) : SV_Target {\n"
@@ -290,18 +289,6 @@ static const std::vector<ShaderSource> vsCol = {
 	"  oColor0 = Color0;\n"
 	"}"
 	},
-	{ ShaderLanguage::HLSL_D3D9,
-	"struct VS_INPUT { float3 Position : POSITION; float4 Color0 : COLOR0; };\n"
-	"struct VS_OUTPUT { float4 Position : POSITION; float4 Color0 : COLOR0; };\n"
-	"float4x4 WorldViewProj : register(c0);\n"
-	"float2 TintSaturation : register(c4);\n"
-	"VS_OUTPUT main(VS_INPUT input) {\n"
-	"  VS_OUTPUT output;\n"
-	"  output.Position = mul(float4(input.Position, 1.0), WorldViewProj);\n"
-	"  output.Color0 = input.Color0;\n"
-	"  return output;\n"
-	"}\n"
-	},
 	{ ShaderLanguage::HLSL_D3D11,
 	"struct VS_INPUT { float3 Position : POSITION; float4 Color0 : COLOR0; };\n"
 	"struct VS_OUTPUT { float4 Color0 : COLOR0; float4 Position : SV_Position; };\n"
@@ -397,37 +384,6 @@ void main() {
     oColor0 = vec4(hsv2rgb(hsv), Color0.w);
 	oTexCoord0 = TexCoord0;
 })",
-	},
-	{ ShaderLanguage::HLSL_D3D9,
-	R"(
-struct VS_INPUT { float3 Position : POSITION; float2 Texcoord0 : TEXCOORD0; float4 Color0 : COLOR0; };
-struct VS_OUTPUT { float4 Position : POSITION; float2 Texcoord0 : TEXCOORD0; float4 Color0 : COLOR0; };
-float4x4 WorldViewProj : register(c0);
-float2 TintSaturation : register(c4);
-float3 rgb2hsv(float3 c) {
-	float4 K = float4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
-	float4 p = lerp(float4(c.bg, K.wz), float4(c.gb, K.xy), step(c.b, c.g));
-	float4 q = lerp(float4(p.xyw, c.r), float4(c.r, p.yzx), step(p.x, c.r));
-	float d = q.x - min(q.w, q.y);
-	float e = 1.0e-10;
-	return float3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
-}
-float3 hsv2rgb(float3 c) {
-	float4 K = float4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
-	float3 p = abs(frac(c.xxx + K.xyz) * 6.0 - K.www);
-	return c.z * lerp(K.xxx, saturate(p - K.xxx), c.y);
-}
-VS_OUTPUT main(VS_INPUT input) {
-	VS_OUTPUT output;
-	float3 hsv = rgb2hsv(input.Color0.xyz);
-	hsv.x += TintSaturation.x;
-	hsv.y *= TintSaturation.y;
-    output.Color0 = float4(hsv2rgb(hsv), input.Color0.w);
-	output.Position = mul(float4(input.Position, 1.0), WorldViewProj);
-	output.Texcoord0 = input.Texcoord0;
-	return output;
-}
-)"
 	},
 	{ ShaderLanguage::HLSL_D3D11,
 R"(
@@ -509,7 +465,7 @@ const UniformBufferDesc vsTexColBufDesc{ sizeof(VsTexColUB),{
 
 ShaderModule *CreateShader(DrawContext *draw, ShaderStage stage, const std::vector<ShaderSource> &sources) {
 	uint32_t supported = draw->GetSupportedShaderLanguages();
-	for (auto iter : sources) {
+	for (const auto &iter : sources) {
 		if ((uint32_t)iter.lang & supported) {
 			return draw->CreateShaderModule(stage, iter.lang, (const uint8_t *)iter.src, strlen(iter.src));
 		}
@@ -548,10 +504,6 @@ void DrawContext::DestroyPresets() {
 	}
 }
 
-DrawContext::~DrawContext() {
-	// TODO: Can't call DestroyPresets here, too late.
-}
-
 void ConvertFromRGBA8888(uint8_t *dst, const uint8_t *src, uint32_t dstStride, uint32_t srcStride, uint32_t width, uint32_t height, DataFormat format) {
 	// Must skip stride in the cases below.  Some games pack data into the cracks, like MotoGP.
 	const uint32_t *src32 = (const uint32_t *)src;
@@ -601,7 +553,7 @@ void ConvertFromRGBA8888(uint8_t *dst, const uint8_t *src, uint32_t dstStride, u
 		case Draw::DataFormat::R8G8B8A8_UNORM:
 		case Draw::DataFormat::UNDEFINED:
 		default:
-			WARN_LOG(G3D, "Unable to convert from format: %d", (int)format);
+			WARN_LOG(Log::G3D, "Unable to convert from format: %d", (int)format);
 			break;
 		}
 	}
@@ -663,7 +615,7 @@ void ConvertFromBGRA8888(uint8_t *dst, const uint8_t *src, uint32_t dstStride, u
 		case Draw::DataFormat::R8G8B8A8_UNORM:
 		case Draw::DataFormat::UNDEFINED:
 		default:
-			WARN_LOG(G3D, "Unable to convert from format to BGRA: %d", (int)format);
+			WARN_LOG(Log::G3D, "Unable to convert from format to BGRA: %d", (int)format);
 			break;
 		}
 	}
