@@ -8,7 +8,7 @@
 // expected to be implemented by subclasses.
 class GraphicsContext {
 public:
-	virtual ~GraphicsContext() {}
+	virtual ~GraphicsContext() = default;
 
 	virtual bool InitFromRenderThread(std::string *errorMessage) { return true; }
 	virtual void ShutdownFromRenderThread() {}
@@ -21,6 +21,7 @@ public:
 	virtual void Resume() {}
 
 	virtual void Resize() = 0;
+	virtual void NotifyWindowRestored() {}
 
 	// Needs casting to the appropriate type, unfortunately. Should find a better solution..
 	virtual void *GetAPIContext() { return nullptr; }
