@@ -12,8 +12,6 @@
 #include "Common/Input/InputState.h"
 #include "Common/Input/KeyCodes.h"
 
-#include "Common/GPU/Vulkan/VulkanContext.h"
-
 #include "Common/Math/lin/matrix4x4.h"
 
 #include "Common/Input/InputState.h"
@@ -26,6 +24,13 @@
 #include "Core/KeyMap.h"
 #include "Core/System.h"
 
+enum VRMatrix {
+	VR_PROJECTION_MATRIX,
+	VR_VIEW_MATRIX_LEFT_EYE,
+	VR_VIEW_MATRIX_RIGHT_EYE,
+	VR_MATRIX_COUNT
+};
+
 enum VRMirroring {
 	VR_MIRRORING_AXIS_X,
 	VR_MIRRORING_AXIS_Y,
@@ -42,13 +47,11 @@ static std::map<int, bool> pspKeys;  // key can be virtual, so not using the enu
 
 static int vr3DGeometryCount = 0;
 static long vrCompat[VR_COMPAT_MAX];
-static bool vrEnabled = false;
 static bool vrFlatForced = false;
 static bool vrFlatGame = false;
-static double vrFov[2] = {};
+static float vrMatrix[VR_MATRIX_COUNT][16];
 static bool vrMirroring[VR_MIRRORING_COUNT];
 static int vrMirroringVariant = 0;
-static float vrViewMatrix[2][16];
 static XrView vrView[2];
 
 static void (*cbNativeAxis)(const AxisInput *axis, size_t count);
@@ -107,11 +110,6 @@ static std::vector<ButtonMapping> controllerMapping[2] = {
 		leftControllerMapping,
 		rightControllerMapping
 };
-static bool controllerMotion[2][5] = {};
-static bool hmdMotion[4] = {};
-static float hmdMotionLast[2] = {};
-static float hmdMotionDiff[2] = {};
-static float hmdMotionDiffLast[2] = {};
 static int mouseController = 1;
 static bool mousePressed = false;
 
@@ -130,8 +128,10 @@ VR app flow integration
 */
 
 bool IsVREnabled() {
+	// For now, let the OPENXR build flag control enablement.
+	// This will change.
 #ifdef OPENXR
-	return vrEnabled;
+	return true;
 #else
 	return false;
 #endif
@@ -153,13 +153,14 @@ void InitVROnAndroid(void* vm, void* activity, const char* system, int version,
 	if (strcmp(vendor, "PICO") == 0) {
 		VR_SetPlatformFLag(VR_PLATFORM_CONTROLLER_PICO, true);
 		VR_SetPlatformFLag(VR_PLATFORM_EXTENSION_INSTANCE, true);
-	} else if ((strcmp(vendor, "META") == 0) || (strcmp(vendor, "OCULUS") == 0)) {
+		VR_SetPlatformFLag(VR_PLATFORM_EXTENSION_PASSTHROUGH, true);
+		VR_SetConfigFloat(VR_CONFIG_VIEWPORT_SUPERSAMPLING, 1.0f);
+	} else {
 		VR_SetPlatformFLag(VR_PLATFORM_CONTROLLER_QUEST, true);
-		VR_SetPlatformFLag(VR_PLATFORM_EXTENSION_FOVEATION, true);
 		VR_SetPlatformFLag(VR_PLATFORM_EXTENSION_PASSTHROUGH, true);
 		VR_SetPlatformFLag(VR_PLATFORM_EXTENSION_PERFORMANCE, true);
+		VR_SetConfigFloat(VR_CONFIG_VIEWPORT_SUPERSAMPLING, 1.3f);
 	}
-	VR_SetPlatformFLag(VR_PLATFORM_RENDERER_VULKAN, (GPUBackend)g_Config.iGPUBackend == GPUBackend::VULKAN);
 
 	//Init VR
 	ovrJava java;
@@ -169,24 +170,10 @@ void InitVROnAndroid(void* vm, void* activity, const char* system, int version,
 }
 #endif
 
-void EnterVR(bool firstStart, void* vulkanContext) {
+void EnterVR(bool firstStart) {
 	if (firstStart) {
 		engine_t* engine = VR_GetEngine();
-		bool useVulkan = (GPUBackend)g_Config.iGPUBackend == GPUBackend::VULKAN;
-		if (useVulkan) {
-			auto* context = (VulkanContext*)vulkanContext;
-			engine->graphicsBindingVulkan = {};
-			engine->graphicsBindingVulkan.type = XR_TYPE_GRAPHICS_BINDING_VULKAN_KHR;
-			engine->graphicsBindingVulkan.next = NULL;
-			engine->graphicsBindingVulkan.device = context->GetDevice();
-			engine->graphicsBindingVulkan.instance = context->GetInstance();
-			engine->graphicsBindingVulkan.physicalDevice = context->GetCurrentPhysicalDevice();
-			engine->graphicsBindingVulkan.queueFamilyIndex = context->GetGraphicsQueueFamilyIndex();
-			engine->graphicsBindingVulkan.queueIndex = 0;
-			VR_EnterVR(engine, &engine->graphicsBindingVulkan);
-		} else {
-			VR_EnterVR(engine, nullptr);
-		}
+		VR_EnterVR(engine);
 		IN_VRInit(engine);
 	}
 	VR_SetConfig(VR_CONFIG_VIEWPORT_VALID, false);
@@ -204,10 +191,6 @@ void SetVRCallbacks(void (*axis)(const AxisInput *axis, size_t count), bool(*key
 	cbNativeTouch = touch;
 }
 
-void SetVREnabled(bool enabled) {
-    vrEnabled = enabled;
-}
-
 /*
 ================================================================================
 
@@ -271,103 +254,6 @@ void UpdateVRInput(bool haptics, float dp_xscale, float dp_yscale) {
 		}
 	}
 
-	//motion control
-	if (g_Config.bEnableMotions) {
-		for (int j = 0; j < 2; j++) {
-			bool activate;
-			float limit = g_Config.fMotionLength; //length of needed movement in meters
-			XrVector3f axis = {0, 1, 0};
-			float center = ToRadians(VR_GetConfigFloat(VR_CONFIG_MENU_YAW));
-			XrQuaternionf orientation = XrQuaternionf_CreateFromVectorAngle(axis, center);
-			XrVector3f position = XrQuaternionf_Rotate(orientation, IN_VRGetPose(j).position);
-
-			//up
-			activate = position.y > limit;
-			keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-			keyInput.keyCode = NKCODE_EXT_MOTION_UP;
-			keyInput.deviceId = controllerIds[j];
-			if (controllerMotion[j][0] != activate) cbNativeKey(keyInput);
-			controllerMotion[j][0] = activate;
-
-			//down
-			activate = position.y < -limit * 1.5f;
-			keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-			keyInput.keyCode = NKCODE_EXT_MOTION_DOWN;
-			keyInput.deviceId = controllerIds[j];
-			if (controllerMotion[j][1] != activate) cbNativeKey(keyInput);
-			controllerMotion[j][1] = activate;
-
-			//left
-			activate = position.x < -limit * (j == 0 ? 1.0f : 0.25f);
-			keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-			keyInput.keyCode = NKCODE_EXT_MOTION_LEFT;
-			keyInput.deviceId = controllerIds[j];
-			if (controllerMotion[j][2] != activate) cbNativeKey(keyInput);
-			controllerMotion[j][2] = activate;
-
-			//right
-			activate = position.x > limit * (j == 1 ? 1.0f : 0.25f);
-			keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-			keyInput.keyCode = NKCODE_EXT_MOTION_RIGHT;
-			keyInput.deviceId = controllerIds[j];
-			if (controllerMotion[j][3] != activate) cbNativeKey(keyInput);
-			controllerMotion[j][3] = activate;
-
-			//forward
-			activate = position.z < -limit;
-			keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-			keyInput.keyCode = NKCODE_EXT_MOTION_FORWARD;
-			keyInput.deviceId = controllerIds[j];
-			if (controllerMotion[j][4] != activate) cbNativeKey(keyInput);
-			controllerMotion[j][4] = activate;
-		}
-	}
-
-	// Head control
-	if (g_Config.bHeadRotationEnabled) {
-		float pitch = -VR_GetHMDAngles().x;
-		float yaw = -VR_GetHMDAngles().y;
-		bool disable = vrFlatForced || appMode == VR_MENU_MODE;
-		bool isVR = !IsFlatVRScene();
-
-		// calculate delta angles of the rotation
-		if (isVR) {
-			float f = g_Config.bHeadRotationSmoothing ? 0.5f : 1.0f;
-			float deltaPitch = pitch - hmdMotionLast[0];
-			float deltaYaw = yaw - hmdMotionLast[1];
-			while (deltaYaw >= 180) deltaYaw -= 360;
-			while (deltaYaw < -180) deltaYaw += 360;
-			hmdMotionLast[0] = pitch;
-			hmdMotionLast[1] = yaw;
-			hmdMotionDiffLast[0] = hmdMotionDiffLast[0] * (1-f) + hmdMotionDiff[0] * f;
-			hmdMotionDiffLast[1] = hmdMotionDiffLast[1] * (1-f) + hmdMotionDiff[1] * f;
-			hmdMotionDiff[0] += deltaPitch;
-			hmdMotionDiff[1] += deltaYaw;
-			pitch = hmdMotionDiff[0];
-			yaw = hmdMotionDiff[1];
-		}
-
-		bool activate;
-		float limit = isVR ? g_Config.fHeadRotationScale : 20;
-		keyInput.deviceId = DEVICE_ID_XR_HMD;
-
-		//left
-		activate = !disable && yaw < -limit;
-		keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-		keyInput.keyCode = NKCODE_EXT_ROTATION_LEFT;
-		if (hmdMotion[2] != activate) cbNativeKey(keyInput);
-		if (isVR && activate) hmdMotionDiff[1] += limit;
-		hmdMotion[2] = activate;
-
-		//right
-		activate = !disable && yaw > limit;
-		keyInput.flags = activate ? KEY_DOWN : KEY_UP;
-		keyInput.keyCode = NKCODE_EXT_ROTATION_RIGHT;
-		if (hmdMotion[3] != activate) cbNativeKey(keyInput);
-		if (isVR && activate) hmdMotionDiff[1] -= limit;
-		hmdMotion[3] = activate;
-	}
-
 	// Camera adjust
 	if (pspKeys[VIRTKEY_VR_CAMERA_ADJUST]) {
 		for (auto& device : pspAxis) {
@@ -384,15 +270,9 @@ void UpdateVRInput(bool haptics, float dp_xscale, float dp_yscale) {
 						g_Config.fCameraHeight = clampFloat(g_Config.fCameraHeight, -150.0f, 150.0f);
 						break;
 					case JOYSTICK_AXIS_Z:
-						if (g_Config.bEnableVR) {
-							if (axis.second < -0.75f) g_Config.fHeadUpDisplayScale -= 0.01f;
-							if (axis.second > 0.75f) g_Config.fHeadUpDisplayScale += 0.01f;
-							g_Config.fHeadUpDisplayScale = clampFloat(g_Config.fHeadUpDisplayScale, 0.0f, 1.5f);
-						} else {
-							if (axis.second < -0.75f) g_Config.fCanvas3DDistance += 0.1f;
-							if (axis.second > 0.75f) g_Config.fCanvas3DDistance -= 0.1f;
-							g_Config.fCanvas3DDistance = clampFloat(g_Config.fCanvas3DDistance, 1.0f, 15.0f);
-						}
+						if (axis.second < -0.75f) g_Config.fCameraPitch -= 0.5f;
+						if (axis.second > 0.75f) g_Config.fCameraPitch += 0.5f;
+						g_Config.fCameraPitch = clampFloat(g_Config.fCameraPitch, -90.0f, 90.0f);
 						break;
 					case JOYSTICK_AXIS_RZ:
 						if (axis.second > 0.75f) g_Config.fCameraDistance -= 0.1f;
@@ -430,6 +310,8 @@ void UpdateVRInput(bool haptics, float dp_xscale, float dp_yscale) {
 		float speed = (cx + cy) / 2;
 		float x = cx - tan(ToRadians(angles.y - VR_GetConfigFloat(VR_CONFIG_MENU_YAW))) * speed;
 		float y = cy - tan(ToRadians(angles.x)) * speed * VR_GetConfigFloat(VR_CONFIG_CANVAS_ASPECT);
+		x *= VR_GetConfigFloat(VR_CONFIG_VIEWPORT_SUPERSAMPLING);
+		y *= VR_GetConfigFloat(VR_CONFIG_VIEWPORT_SUPERSAMPLING);
 
 		//set renderer
 		VR_SetConfig(VR_CONFIG_MOUSE_X, (int)x);
@@ -438,10 +320,10 @@ void UpdateVRInput(bool haptics, float dp_xscale, float dp_yscale) {
 		VR_SetConfig(VR_CONFIG_CANVAS_6DOF, g_Config.bEnable6DoF);
 
 		//inform engine about the status
-		TouchInput touch;
+		TouchInput touch{};
 		touch.id = mouseController;
 		touch.x = x * dp_xscale;
-		touch.y = (height - y - 1) * dp_yscale / VR_GetConfigFloat(VR_CONFIG_CANVAS_ASPECT);
+		touch.y = (height - y - 1) * dp_yscale;
 		bool pressed = IN_VRGetButtonState(mouseController) & ovrButton_Trigger;
 		if (mousePressed != pressed) {
 			if (pressed) {
@@ -533,11 +415,10 @@ bool UpdateVRKeys(const KeyInput &key) {
 
 	// Reset camera adjust
 	if (pspKeys[VIRTKEY_VR_CAMERA_ADJUST] && pspKeys[VIRTKEY_VR_CAMERA_RESET]) {
-		g_Config.fCanvas3DDistance = 3.0f;
 		g_Config.fCameraHeight = 0;
 		g_Config.fCameraSide = 0;
 		g_Config.fCameraDistance = 0;
-		g_Config.fHeadUpDisplayScale = 0.3f;
+		g_Config.fCameraPitch = 0;
 	}
 
 	//block keys by camera adjust
@@ -622,44 +503,58 @@ void* BindVRFramebuffer() {
 
 bool StartVRRender() {
 	if (!VR_GetConfig(VR_CONFIG_VIEWPORT_VALID)) {
-		VR_InitRenderer(VR_GetEngine(), IsMultiviewSupported());
+		VR_InitRenderer(VR_GetEngine());
 		VR_SetConfig(VR_CONFIG_VIEWPORT_VALID, true);
 	}
 
 	if (VR_InitFrame(VR_GetEngine())) {
 
-		// VR flags
-		bool vrIncompatibleGame = PSP_CoreParameter().compat.vrCompat().ForceFlatScreen;
-		bool vrScene = !vrFlatForced && (g_Config.bManualForceVR || (vr3DGeometryCount > 15));
-		bool vrStereo = !PSP_CoreParameter().compat.vrCompat().ForceMono && g_Config.bEnableStereo;
-
 		// Get VR status
 		for (int eye = 0; eye < ovrMaxNumEyes; eye++) {
 			vrView[eye] = VR_GetView(eye);
 		}
 		UpdateVRViewMatrices();
 
-		// Calculate field of view
+		// Update projection matrix
 		XrFovf fov = {};
-		for (auto & eye : vrView) {
-			fov.angleLeft += eye.fov.angleLeft / 2.0f;
-			fov.angleRight += eye.fov.angleRight / 2.0f;
-			fov.angleUp += eye.fov.angleUp / 2.0f;
-			fov.angleDown += eye.fov.angleDown / 2.0f;
+		for (int eye = 0; eye < ovrMaxNumEyes; eye++) {
+			fov.angleLeft += vrView[eye].fov.angleLeft / 2.0f;
+			fov.angleRight += vrView[eye].fov.angleRight / 2.0f;
+			fov.angleUp += vrView[eye].fov.angleUp / 2.0f;
+			fov.angleDown += vrView[eye].fov.angleDown / 2.0f;
 		}
-		vrFov[0] = 2.0 / (tan(fov.angleRight) - tan(fov.angleLeft));
-		vrFov[1] = 2.0 / (tan(fov.angleUp) - tan(fov.angleDown));
+		float nearZ = 0.01f;
+		float tanAngleLeft = tanf(fov.angleLeft);
+		float tanAngleRight = tanf(fov.angleRight);
+		float tanAngleDown = tanf(fov.angleDown);
+		float tanAngleUp = tanf(fov.angleUp);
+		float M[16] = {};
+		M[0] = 2 / (tanAngleRight - tanAngleLeft);
+		M[5] = 2 / (tanAngleUp - tanAngleDown);
+		M[8] = (tanAngleRight + tanAngleLeft) / (tanAngleRight - tanAngleLeft);
+		M[9] = (tanAngleUp + tanAngleDown) / (tanAngleUp - tanAngleDown);
+		M[10] = -1;
+		M[11] = -1;
+		M[14] = -(nearZ + nearZ);
+		if (IsImmersiveVRMode()) {
+			M[0] /= 2.0f;
+		}
+		memcpy(vrMatrix[VR_PROJECTION_MATRIX], M, sizeof(float) * 16);
 
 		// Decide if the scene is 3D or not
 		VR_SetConfigFloat(VR_CONFIG_CANVAS_ASPECT, 480.0f / 272.0f);
-		if (g_Config.bEnableVR && !vrIncompatibleGame && (appMode == VR_GAME_MODE) && vrScene) {
+		bool vrStereo = !PSP_CoreParameter().compat.vrCompat().ForceMono && g_Config.bEnableStereo;
+		if (!IsBigScreenVRMode() && (appMode == VR_GAME_MODE)) {
 			VR_SetConfig(VR_CONFIG_MODE, vrStereo ? VR_MODE_STEREO_6DOF : VR_MODE_MONO_6DOF);
+			VR_SetConfig(VR_CONFIG_REPROJECTION, IsImmersiveVRMode() ? 0 : 1);
 			vrFlatGame = false;
-		} else {
+		} else if (appMode == VR_GAME_MODE) {
 			VR_SetConfig(VR_CONFIG_MODE, vrStereo ? VR_MODE_STEREO_SCREEN : VR_MODE_MONO_SCREEN);
 			if (IsGameVRScene()) {
 				vrFlatGame = true;
 			}
+		} else {
+			VR_SetConfig(VR_CONFIG_MODE, VR_MODE_MONO_SCREEN);
 		}
 		vr3DGeometryCount /= 2;
 
@@ -667,9 +562,8 @@ bool StartVRRender() {
 		vrCompat[VR_COMPAT_SKYPLANE] = PSP_CoreParameter().compat.vrCompat().Skyplane;
 
 		// Set customizations
-		__DisplaySetFramerate(g_Config.bForce72Hz ? 72 : 60);
-		VR_SetConfigFloat(VR_CONFIG_CANVAS_DISTANCE, vrScene && (appMode == VR_GAME_MODE) ? g_Config.fCanvas3DDistance : g_Config.fCanvasDistance);
-		VR_SetConfig(VR_CONFIG_PASSTHROUGH, g_Config.bPassthrough);
+		VR_SetConfigFloat(VR_CONFIG_CANVAS_DISTANCE, !IsBigScreenVRMode() && (appMode == VR_GAME_MODE) ? g_Config.fCanvas3DDistance : g_Config.fCanvasDistance);
+		VR_SetConfig(VR_CONFIG_PASSTHROUGH, g_Config.bPassthrough && IsPassthroughSupported());
 		return true;
 	}
 	return false;
@@ -694,26 +588,28 @@ int GetVRFBOIndex() {
 
 int GetVRPassesCount() {
 	bool vrStereo = !PSP_CoreParameter().compat.vrCompat().ForceMono && g_Config.bEnableStereo;
-	if (!IsMultiviewSupported() && vrStereo) {
-		return 2;
-	} else {
-		return 1;
-	}
-}
-
-bool IsMultiviewSupported() {
-	return false;
+	return vrStereo ? 2 : 1;
 }
 
 bool IsPassthroughSupported() {
 	return VR_GetPlatformFlag(VR_PLATFORM_EXTENSION_PASSTHROUGH);
 }
 
+bool IsBigScreenVRMode() {
+	bool vrIncompatibleGame = PSP_CoreParameter().compat.vrCompat().ForceFlatScreen;
+	bool vrMode = (g_Config.bEnableVR || IsImmersiveVRMode()) && !vrIncompatibleGame;
+	bool vrScene = !vrFlatForced && (g_Config.bManualForceVR || (vr3DGeometryCount > 15));
+	return !vrMode || !vrScene;
+}
+
 bool IsFlatVRGame() {
 	return vrFlatGame;
 }
 
 bool IsFlatVRScene() {
+	if (g_Config.bForceVR && (!vrFlatForced || !g_Config.bManualForceVR)) {
+		return false;
+	}
 	int vrMode = VR_GetConfig(VR_CONFIG_MODE);
 	return (vrMode == VR_MODE_MONO_SCREEN) || (vrMode == VR_MODE_STEREO_SCREEN);
 }
@@ -722,6 +618,10 @@ bool IsGameVRScene() {
 	return (appMode == VR_GAME_MODE) || (appMode == VR_DIALOG_MODE);
 }
 
+bool IsImmersiveVRMode() {
+	return g_Config.bEnableImmersiveVR && !PSP_CoreParameter().compat.vrCompat().IdentityViewHack;
+}
+
 bool Is2DVRObject(float* projMatrix, bool ortho) {
 
 	// Quick analyze if the object is in 2D
@@ -792,17 +692,27 @@ void UpdateVRParams(float* projMatrix) {
 	}
 }
 
-void UpdateVRProjection(float* projMatrix, float* leftEye, float* rightEye) {
-	float hmdProjection[16];
-	memcpy(hmdProjection, projMatrix, 16 * sizeof(float));
-	hmdProjection[0] = vrFov[0];
-	hmdProjection[5] = vrFov[1];
-	memcpy(leftEye, hmdProjection, 16 * sizeof(float));
-	memcpy(rightEye, hmdProjection, 16 * sizeof(float));
+void UpdateVRProjection(float* projMatrix, float* output) {
+	for (int i = 0; i < 16; i++) {
+		if (!IsVREnabled() || IsBigScreenVRMode()) {
+			output[i] = projMatrix[i];
+		} else if (PSP_CoreParameter().compat.vrCompat().ProjectionHack && ((i == 8) || (i == 9))) {
+			output[i] = 0;
+		} else if (fabs(projMatrix[i]) > 0) {
+			output[i] = vrMatrix[VR_PROJECTION_MATRIX][i];
+			if ((output[i] > 0) != (projMatrix[i] > 0)) {
+				output[i] *= -1.0f;
+			}
+		} else {
+			output[i] = 0;
+		}
+	}
+	output[11] *= g_Config.fFieldOfViewPercentage / 100.0f;
 }
 
 void UpdateVRView(float* leftEye, float* rightEye) {
 	float* dst[] = {leftEye, rightEye};
+	float* matrix[] = {vrMatrix[VR_VIEW_MATRIX_LEFT_EYE], vrMatrix[VR_VIEW_MATRIX_RIGHT_EYE]};
 	for (int index = 0; index < 2; index++) {
 
 		// Validate the view matrix
@@ -816,7 +726,7 @@ void UpdateVRView(float* leftEye, float* rightEye) {
 
 		// Get view matrix from the headset
 		Lin::Matrix4x4 hmdView = {};
-		memcpy(hmdView.m, vrViewMatrix[index], 16 * sizeof(float));
+		memcpy(hmdView.m, matrix[index], 16 * sizeof(float));
 
 		// Combine the matrices
 		Lin::Matrix4x4 renderView = hmdView * gameView;
@@ -833,8 +743,11 @@ void UpdateVRViewMatrices() {
 	}
 
 	// Get input
+	XrPosef invView = XrPosef_Identity();
+	if (IsVREnabled()) {
+		invView = vrView[0].pose;
+	}
 	bool flatScreen = false;
-	XrPosef invView = vrView[0].pose;
 	int vrMode = VR_GetConfig(VR_CONFIG_MODE);
 	if ((vrMode == VR_MODE_MONO_SCREEN) || (vrMode == VR_MODE_STEREO_SCREEN)) {
 		invView = XrPosef_Identity();
@@ -855,23 +768,13 @@ void UpdateVRViewMatrices() {
 		invView = XrPosef_Inverse(invView);
 	}
 
-	// apply camera pitch offset
+	// apply camera pitch
+	float s = sin(ToRadians(g_Config.fCameraPitch));
+	float c = cos(ToRadians(g_Config.fCameraPitch));
 	XrVector3f positionOffset = {g_Config.fCameraSide, g_Config.fCameraHeight, g_Config.fCameraDistance};
-	if (!flatScreen) {
-		float pitchOffset = 0;
-		switch (g_Config.iCameraPitch) {
-			case 1: //Top view -> First person
-				pitchOffset = 90;
-				positionOffset = {positionOffset.x, positionOffset.z, -positionOffset.y};
-				break;
-			case 2: //First person -> Top view
-				pitchOffset = -90;
-				positionOffset = {positionOffset.x, -positionOffset.z + 20, positionOffset.y};
-				break;
-		}
-		XrQuaternionf rotationOffset = XrQuaternionf_CreateFromVectorAngle({1, 0, 0}, ToRadians(pitchOffset));
-		invView.orientation = XrQuaternionf_Multiply(rotationOffset, invView.orientation);
-	}
+	positionOffset = {positionOffset.x, s * positionOffset.z + c * positionOffset.y, c * positionOffset.z - s * positionOffset.y};
+	XrQuaternionf rotationOffset = XrQuaternionf_CreateFromVectorAngle({1, 0, 0}, ToRadians(g_Config.fCameraPitch));
+	invView.orientation = XrQuaternionf_Multiply(rotationOffset, invView.orientation);
 
 	// decompose rotation
 	XrVector3f rotation = XrQuaternionf_ToEulerAngles(invView.orientation);
@@ -879,20 +782,21 @@ void UpdateVRViewMatrices() {
 	float mYaw = my * ToRadians(rotation.y);
 	float mRoll = mz * ToRadians(rotation.z);
 
-	// use in-game camera interpolated rotation
-	if (g_Config.bHeadRotationEnabled) mYaw = -my * ToRadians(hmdMotionDiffLast[1]); // horizontal
-
 	// create updated quaternion
 	XrQuaternionf pitch = XrQuaternionf_CreateFromVectorAngle({1, 0, 0}, mPitch);
 	XrQuaternionf yaw = XrQuaternionf_CreateFromVectorAngle({0, 1, 0}, mYaw);
 	XrQuaternionf roll = XrQuaternionf_CreateFromVectorAngle({0, 0, 1}, mRoll);
 	invView.orientation = XrQuaternionf_Multiply(roll, XrQuaternionf_Multiply(pitch, yaw));
+	if (!VR_GetConfig(VR_CONFIG_REPROJECTION)) {
+		float axis = vrMirroring[VR_MIRRORING_PITCH] ? -1.0f : 1.0f;
+		invView.orientation = XrQuaternionf_CreateFromVectorAngle({axis, 0, 0}, ToRadians(g_Config.fCameraPitch));
+	}
 
 	float M[16];
 	XrQuaternionf_ToMatrix4f(&invView.orientation, M);
 
 	// Apply 6Dof head movement
-	if (g_Config.bEnable6DoF && !g_Config.bHeadRotationEnabled && (g_Config.iCameraPitch == 0)) {
+	if (g_Config.bEnable6DoF && IsVREnabled()) {
 		M[3] -= vrView[0].pose.position.x * (vrMirroring[VR_MIRRORING_AXIS_X] ? -1.0f : 1.0f) * scale;
 		M[7] -= vrView[0].pose.position.y * (vrMirroring[VR_MIRRORING_AXIS_Y] ? -1.0f : 1.0f) * scale;
 		M[11] -= vrView[0].pose.position.z * (vrMirroring[VR_MIRRORING_AXIS_Z] ? -1.0f : 1.0f) * scale;
@@ -925,12 +829,12 @@ void UpdateVRViewMatrices() {
 		M[11] += side.z;
 	}
 
-	for (int eye = 0; eye < ovrMaxNumEyes; eye++) {
+	for (int matrix = VR_VIEW_MATRIX_LEFT_EYE; matrix <= VR_VIEW_MATRIX_RIGHT_EYE; matrix++) {
 
 		// Stereoscopy
 		bool vrStereo = !PSP_CoreParameter().compat.vrCompat().ForceMono && g_Config.bEnableStereo;
-		if (vrStereo) {
-			bool mirrored = vrMirroring[VR_MIRRORING_AXIS_Z] ^ (eye == 1);
+		if (vrStereo && IsVREnabled()) {
+			bool mirrored = vrMirroring[VR_MIRRORING_AXIS_Z] ^ (matrix == VR_VIEW_MATRIX_RIGHT_EYE);
 			float dx = fabs(vrView[1].pose.position.x - vrView[0].pose.position.x);
 			float dy = fabs(vrView[1].pose.position.y - vrView[0].pose.position.y);
 			float dz = fabs(vrView[1].pose.position.z - vrView[0].pose.position.z);
@@ -943,6 +847,6 @@ void UpdateVRViewMatrices() {
 			M[11] += separation.z;
 		}
 
-		memcpy(vrViewMatrix[eye], M, sizeof(float) * 16);
+		memcpy(vrMatrix[matrix], M, sizeof(float) * 16);
 	}
 }
