@@ -28,33 +28,8 @@
 #include <cstring>
 #include <signal.h>
 
-#if defined(HAVE_LIBNX) || PPSSPP_PLATFORM(SWITCH)
-#include <netdb.h>
-#include <switch.h>
-// Missing include, *shrugs*
-extern "C" struct hostent *gethostbyname(const char *name);
-#endif // defined(HAVE_LIBNX) || PPSSPP_PLATFORM(SWITCH)
-
 #include <sys/types.h>
-// Net stuff
-#if defined(_WIN32)
-#include <WinSock2.h>
-#include <WS2tcpip.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#endif
-
-#include <fcntl.h>
-#include <errno.h>
-//#include <sqlite3.h>
-
-#ifndef MSG_NOSIGNAL
-// Default value to 0x00 (do nothing) in systems where it's not supported.
-#define MSG_NOSIGNAL 0x00
-#endif
-
+#include "Common/Net/SocketCompat.h"
 #include "Common/Data/Text/I18n.h"
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/System/OSD.h"
@@ -66,6 +41,11 @@ extern "C" struct hostent *gethostbyname(const char *name);
 #include "Core/Core.h"
 #include "Core/HLE/proAdhocServer.h"
 
+#ifdef _WIN32
+#undef errno
+#define errno WSAGetLastError()
+#endif
+
 // User Count
 uint32_t _db_user_count = 0;
 
@@ -517,7 +497,7 @@ void login_user_stream(int fd, uint32_t ip)
 		while(u != NULL && u->resolver.ip != ip) u = u->next;
 
 		if (u != NULL) { // IP Already existed
-			WARN_LOG(SCENET, "AdhocServer: Already Existing IP: %s\n", ip2str(*(in_addr*)&u->resolver.ip).c_str());
+			WARN_LOG(Log::sceNet, "AdhocServer: Already Existing IP: %s\n", ip2str(*(in_addr*)&u->resolver.ip).c_str());
 		}
 
 		// Unique IP Address
@@ -547,7 +527,7 @@ void login_user_stream(int fd, uint32_t ip)
 				user->last_recv = time(NULL);
 
 				// Notify User
-				INFO_LOG(SCENET, "AdhocServer: New Connection from %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
+				INFO_LOG(Log::sceNet, "AdhocServer: New Connection from %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
 
 				// Fix User Counter
 				_db_user_count++;
@@ -590,7 +570,7 @@ void login_user_data(SceNetAdhocctlUserNode * user, SceNetAdhocctlLoginPacketC2S
 		while (u != NULL && !IsMatch(u->resolver.mac, data->mac)) u = u->next;
 
 		if (u != NULL) { // MAC Already existed
-			WARN_LOG(SCENET, "AdhocServer: Already Existing MAC: %s [%s]\n", mac2str(&data->mac).c_str(), ip2str(*(in_addr*)&u->resolver.ip).c_str());
+			WARN_LOG(Log::sceNet, "AdhocServer: Already Existing MAC: %s [%s]\n", mac2str(&data->mac).c_str(), ip2str(*(in_addr*)&u->resolver.ip).c_str());
 		}
 
 		// Game Product Override
@@ -641,7 +621,7 @@ void login_user_data(SceNetAdhocctlUserNode * user, SceNetAdhocctlLoginPacketC2S
 			char safegamestr[10];
 			memset(safegamestr, 0, sizeof(safegamestr));
 			strncpy(safegamestr, game->game.data, PRODUCT_CODE_LENGTH);
-			INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) started playing %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
+			INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) started playing %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
 
 			// Update Status Log
 			update_status();
@@ -655,7 +635,7 @@ void login_user_data(SceNetAdhocctlUserNode * user, SceNetAdhocctlLoginPacketC2S
 	else
 	{
 		// Notify User
-		WARN_LOG(SCENET, "AdhocServer: Invalid Login Packet Contents from %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
+		WARN_LOG(Log::sceNet, "AdhocServer: Invalid Login Packet Contents from %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
 	}
 
 	// Logout User - Out of Memory or Invalid Arguments
@@ -690,7 +670,7 @@ void logout_user(SceNetAdhocctlUserNode * user)
 		char safegamestr[10];
 		memset(safegamestr, 0, sizeof(safegamestr));
 		strncpy(safegamestr, user->game->game.data, PRODUCT_CODE_LENGTH);
-		INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) stopped playing %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
+		INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) stopped playing %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
 
 		// Fix Game Player Count
 		user->game->playercount--;
@@ -716,7 +696,7 @@ void logout_user(SceNetAdhocctlUserNode * user)
 	else
 	{
 		// Notify User
-		WARN_LOG(SCENET, "AdhocServer: Dropped Connection to %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
+		WARN_LOG(Log::sceNet, "AdhocServer: Dropped Connection to %s", ip2str(*(in_addr*)&user->resolver.ip).c_str());
 	}
 
 	// Free Memory
@@ -859,8 +839,8 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 					packet.ip = user->resolver.ip;
 
 					// Send Data
-					int iResult = send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-					if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: connect_user[send peer] (Socket error %d)", errno);
+					int iResult = (int)send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+					if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: connect_user[send peer] (Socket error %d)", socket_errno);
 
 					// Set Player Name
 					packet.name = peer->resolver.name;
@@ -872,8 +852,8 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 					packet.ip = peer->resolver.ip;
 
 					// Send Data
-					iResult = send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-					if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: connect_user[send user] (Socket error %d)", errno);
+					iResult = (int)send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+					if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: connect_user[send user] (Socket error %d)", socket_errno);
 
 					// Set BSSID
 					if(peer->group_next == NULL) bssid.mac = peer->resolver.mac;
@@ -894,8 +874,8 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 				g->playercount++;
 
 				// Send Network BSSID to User
-				int iResult = send(user->stream, (const char*)&bssid, sizeof(bssid), MSG_NOSIGNAL);
-				if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: connect_user[send user bssid] (Socket error %d)", errno);
+				int iResult = (int)send(user->stream, (const char*)&bssid, sizeof(bssid), MSG_NOSIGNAL);
+				if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: connect_user[send user bssid] (Socket error %d)", socket_errno);
 
 				// Notify User
 				char safegamestr[10];
@@ -904,7 +884,7 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 				char safegroupstr[9];
 				memset(safegroupstr, 0, sizeof(safegroupstr));
 				strncpy(safegroupstr, (char *)user->group->group.data, ADHOCCTL_GROUPNAME_LEN);
-				INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) joined %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
+				INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) joined %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
 
 				// Update Status Log
 				update_status();
@@ -927,7 +907,7 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 			char safegroupstr2[9];
 			memset(safegroupstr2, 0, sizeof(safegroupstr2));
 			strncpy(safegroupstr2, (char *)user->group->group.data, ADHOCCTL_GROUPNAME_LEN);
-			WARN_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) attempted to join %s group %s without disconnecting from %s first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr, safegroupstr2);
+			WARN_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) attempted to join %s group %s without disconnecting from %s first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr, safegroupstr2);
 		}
 	}
 
@@ -941,7 +921,7 @@ void connect_user(SceNetAdhocctlUserNode * user, SceNetAdhocctlGroupName * group
 		char safegroupstr[9];
 		memset(safegroupstr, 0, sizeof(safegroupstr));
 		strncpy(safegroupstr, (char *)group->data, ADHOCCTL_GROUPNAME_LEN);
-		WARN_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) attempted to join invalid %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
+		WARN_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) attempted to join invalid %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
 	}
 
 	// Invalid State, Out of Memory or Invalid Group Name
@@ -986,8 +966,8 @@ void disconnect_user(SceNetAdhocctlUserNode * user)
 			packet.ip = user->resolver.ip;
 
 			// Send Data
-			int iResult = send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-			if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: disconnect_user[send peer] (Socket error %d)", errno);
+			int iResult = (int)send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+			if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: disconnect_user[send peer] (Socket error %d)", socket_errno);
 
 			// Move Pointer
 			peer = peer->group_next;
@@ -1000,7 +980,7 @@ void disconnect_user(SceNetAdhocctlUserNode * user)
 		char safegroupstr[9];
 		memset(safegroupstr, 0, sizeof(safegroupstr));
 		strncpy(safegroupstr, (char *)user->group->group.data, ADHOCCTL_GROUPNAME_LEN);
-		INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) left %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
+		INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) left %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
 
 		// Empty Group
 		if(user->group->playercount == 0)
@@ -1040,7 +1020,7 @@ void disconnect_user(SceNetAdhocctlUserNode * user)
 		char safegamestr[10];
 		memset(safegamestr, 0, sizeof(safegamestr));
 		strncpy(safegamestr, user->game->game.data, PRODUCT_CODE_LENGTH);
-		WARN_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) attempted to leave %s group without joining one first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
+		WARN_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) attempted to leave %s group without joining one first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
 	}
 
 	// Delete User
@@ -1085,20 +1065,20 @@ void send_scan_results(SceNetAdhocctlUserNode * user)
 			}
 
 			// Send Group Packet
-			int iResult = send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-			if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: send_scan_result[send user] (Socket error %d)", errno);
+			int iResult = (int)send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+			if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: send_scan_result[send user] (Socket error %d)", socket_errno);
 		}
 
 		// Notify Player of End of Scan
 		uint8_t opcode = OPCODE_SCAN_COMPLETE;
-		int iResult = send(user->stream, (const char*)&opcode, 1, MSG_NOSIGNAL);
-		if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: send_scan_result[send peer complete] (Socket error %d)", errno);
+		int iResult = (int)send(user->stream, (const char*)&opcode, 1, MSG_NOSIGNAL);
+		if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: send_scan_result[send peer complete] (Socket error %d)", socket_errno);
 
 		// Notify User
 		char safegamestr[10];
 		memset(safegamestr, 0, sizeof(safegamestr));
 		strncpy(safegamestr, user->game->game.data, PRODUCT_CODE_LENGTH);
-		INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) requested information on %d %s groups", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), user->game->groupcount, safegamestr);
+		INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) requested information on %d %s groups", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), user->game->groupcount, safegamestr);
 
 		// Exit Function
 		return;
@@ -1114,7 +1094,7 @@ void send_scan_results(SceNetAdhocctlUserNode * user)
 		char safegroupstr[9];
 		memset(safegroupstr, 0, sizeof(safegroupstr));
 		strncpy(safegroupstr, (char *)user->group->group.data, ADHOCCTL_GROUPNAME_LEN);
-		WARN_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) attempted to scan for %s groups without disconnecting from %s first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
+		WARN_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) attempted to scan for %s groups without disconnecting from %s first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr, safegroupstr);
 	}
 
 	// Delete User
@@ -1150,8 +1130,8 @@ void spread_message(SceNetAdhocctlUserNode *user, const char *message)
 				strcpy(packet.base.message, message);
 
 				// Send Data
-				int iResult = send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-				if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: spread_message[send user chat] (Socket error %d)", errno);
+				int iResult = (int)send(user->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+				if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: spread_message[send user chat] (Socket error %d)", socket_errno);
 			}
 		}
 
@@ -1180,7 +1160,7 @@ void spread_message(SceNetAdhocctlUserNode *user, const char *message)
 			}
 
 			// Chat Packet
-			SceNetAdhocctlChatPacketS2C packet;
+			SceNetAdhocctlChatPacketS2C packet{};
 
 			// Set Chat Opcode
 			packet.base.base.opcode = OPCODE_CHAT;
@@ -1192,8 +1172,8 @@ void spread_message(SceNetAdhocctlUserNode *user, const char *message)
 			packet.name = user->resolver.name;
 
 			// Send Data
-			int iResult = send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
-			if (iResult < 0) ERROR_LOG(SCENET, "AdhocServer: spread_message[send peer chat] (Socket error %d)", errno);
+			int iResult = (int)send(peer->stream, (const char*)&packet, sizeof(packet), MSG_NOSIGNAL);
+			if (iResult < 0) ERROR_LOG(Log::sceNet, "AdhocServer: spread_message[send peer chat] (Socket error %d)", socket_errno);
 
 			// Move Pointer
 			peer = peer->group_next;
@@ -1212,7 +1192,7 @@ void spread_message(SceNetAdhocctlUserNode *user, const char *message)
 			char safegroupstr[9];
 			memset(safegroupstr, 0, sizeof(safegroupstr));
 			strncpy(safegroupstr, (char *)user->group->group.data, ADHOCCTL_GROUPNAME_LEN);
-			INFO_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) sent \"%s\" to %d players in %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), message, counter, safegamestr, safegroupstr);
+			INFO_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) sent \"%s\" to %d players in %s group %s", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), message, counter, safegamestr, safegroupstr);
 		}
 
 		// Exit Function
@@ -1226,7 +1206,7 @@ void spread_message(SceNetAdhocctlUserNode *user, const char *message)
 		char safegamestr[10];
 		memset(safegamestr, 0, sizeof(safegamestr));
 		strncpy(safegamestr, user->game->game.data, PRODUCT_CODE_LENGTH);
-		WARN_LOG(SCENET, "AdhocServer: %s (MAC: %s - IP: %s) attempted to send a text message without joining a %s group first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
+		WARN_LOG(Log::sceNet, "AdhocServer: %s (MAC: %s - IP: %s) attempted to send a text message without joining a %s group first", (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str(), safegamestr);
 	}
 
 	// Delete User
@@ -1327,7 +1307,7 @@ void game_product_override(SceNetAdhocctlProductCode * product)
 					strncpy(product->data, crosslink, PRODUCT_CODE_LENGTH);
 
 					// Log Crosslink
-					INFO_LOG(SCENET, "Crosslinked %s to %s", productid, crosslink);
+					INFO_LOG(Log::sceNet, "Crosslinked %s to %s", productid, crosslink);
 
 					// Set Crosslinked Flag
 					crosslinked = 1;
@@ -1337,18 +1317,18 @@ void game_product_override(SceNetAdhocctlProductCode * product)
 			// Destroy Prepared SQL Statement
 			sqlite3_finalize(statement);
 		}*/
-		for (std::vector<db_crosslink>::iterator it = crosslinks.begin(); it != crosslinks.end(); it++) {
-			if (IsMatch(it->id_from, productid)) {
+		for (const auto &link : crosslinks) {
+			if (IsMatch(link.id_from, productid)) {
 				// Grab Crosslink ID
 				char crosslink[PRODUCT_CODE_LENGTH + 1];
-				strncpy(crosslink, it->id_to, PRODUCT_CODE_LENGTH);
+				strncpy(crosslink, link.id_to, PRODUCT_CODE_LENGTH);
 				crosslink[PRODUCT_CODE_LENGTH] = 0; // null terminated
 
 				// Crosslink Product Code
-				strncpy(product->data, it->id_to, PRODUCT_CODE_LENGTH);
+				strncpy(product->data, link.id_to, PRODUCT_CODE_LENGTH);
 
 				// Log Crosslink
-				INFO_LOG(SCENET, "AdhocServer: Crosslinked %s to %s", productid, crosslink);
+				INFO_LOG(Log::sceNet, "AdhocServer: Crosslinked %s to %s", productid, crosslink);
 
 				// Set Crosslinked Flag
 				crosslinked = 1;
@@ -1376,8 +1356,8 @@ void game_product_override(SceNetAdhocctlProductCode * product)
 				// Destroy Prepare SQL Statement
 				sqlite3_finalize(statement);
 			}*/
-			for (std::vector<db_productid>::iterator it = productids.begin(); it != productids.end(); it++) {
-				if (IsMatch(it->id, productid)) {
+			for (const auto &product : productids) {
+				if (IsMatch(product.id, productid)) {
 					// Set Exists Flag
 					exists = 1;
 					break;
@@ -1397,7 +1377,7 @@ void game_product_override(SceNetAdhocctlProductCode * product)
 						if(sqlite3_step(statement) == SQLITE_DONE)
 						{
 							// Log Addition
-							INFO_LOG(SCENET, "Added Unknown Product ID %s to Database", productid);
+							INFO_LOG(Log::sceNet, "Added Unknown Product ID %s to Database", productid);
 						}
 					}
 
@@ -1406,10 +1386,10 @@ void game_product_override(SceNetAdhocctlProductCode * product)
 				}*/
 				db_productid unkproduct;
 				strncpy(unkproduct.id, productid, sizeof(unkproduct.id));
-				strncpy(unkproduct.name, productid, sizeof(productid));
+				strncpy(unkproduct.name, productid, sizeof(unkproduct.name));
 				productids.push_back(unkproduct); //productids[productids.size()] = unkproduct;
 				// Log Addition
-				INFO_LOG(SCENET, "AdhocServer: Added Unknown Product ID %s to Database", productid);
+				INFO_LOG(Log::sceNet, "AdhocServer: Added Unknown Product ID %s to Database", productid);
 			}
 		}
 
@@ -1491,10 +1471,10 @@ void update_status()
 				}*/
 				//db_productid *foundid = NULL;
 				bool found = false;
-				for (std::vector<db_productid>::iterator it = productids.begin(); it != productids.end(); it++) {
-					if (IsMatch(it->id, productid)) {
+				for (const auto &product : productids) {
+					if (IsMatch(product.id, productid)) {
 						// Copy Game Name
-						strcpyxml(displayname, it->name, sizeof(displayname));
+						strcpyxml(displayname, product.name, sizeof(displayname));
 						found = true;
 						break;
 					}
@@ -1680,7 +1660,7 @@ int proAdhocServerThread(int port) // (int argc, char * argv[])
 	// Result
 	int result = 0;
 
-	INFO_LOG(SCENET, "AdhocServer: Begin of AdhocServer Thread");
+	INFO_LOG(Log::sceNet, "AdhocServer: Begin of AdhocServer Thread");
 
 	// Create Signal Receiver for CTRL + C
 	//signal(SIGINT, interrupt);
@@ -1695,7 +1675,7 @@ int proAdhocServerThread(int port) // (int argc, char * argv[])
 	if(server != SOCKET_ERROR)
 	{
 		// Notify User
-		INFO_LOG(SCENET, "AdhocServer: Listening for Connections on TCP Port %u", port); //SERVER_PORT
+		INFO_LOG(Log::sceNet, "AdhocServer: Listening for Connections on TCP Port %u", port); //SERVER_PORT
 
 		// Port forward
 		UPnP_Add(IP_PROTOCOL_TCP, port); // g_PortManager.Add(IP_PROTOCOL_TCP, port);
@@ -1707,13 +1687,13 @@ int proAdhocServerThread(int port) // (int argc, char * argv[])
 		UPnP_Remove(IP_PROTOCOL_TCP, port); // g_PortManager.Remove(IP_PROTOCOL_TCP, port);
 
 		// Notify User
-		INFO_LOG(SCENET, "AdhocServer: Shutdown complete");
+		INFO_LOG(Log::sceNet, "AdhocServer: Shutdown complete");
 	}
 
 	//_status = 0;
 	adhocServerRunning = false;
 
-	INFO_LOG(SCENET, "AdhocServer: End of AdhocServer Thread");
+	INFO_LOG(Log::sceNet, "AdhocServer: End of AdhocServer Thread");
 
 	// Return Result
 	return result;
@@ -1726,7 +1706,7 @@ int proAdhocServerThread(int port) // (int argc, char * argv[])
 void interrupt(int sig)
 {
 	// Notify User
-	INFO_LOG(SCENET, "AdhocServer: Shutting down... please wait");
+	INFO_LOG(Log::sceNet, "AdhocServer: Shutting down... please wait");
 
 	// Trigger Shutdown
 	//_status = 0;
@@ -1856,18 +1836,18 @@ int create_listen_socket(uint16_t port)
 
 		// Notify User
 		else {
-			ERROR_LOG(SCENET, "AdhocServer: Bind returned %i (Socket error %d)", bindresult, errno);
+			ERROR_LOG(Log::sceNet, "AdhocServer: Bind returned %i (Socket error %d)", bindresult, socket_errno);
 			auto n = GetI18NCategory(I18NCat::NETWORKING);
-			g_OSD.Show(OSDType::MESSAGE_ERROR, std::string(n->T("AdhocServer Failed to Bind Port")) + " " + std::to_string(port));
+			g_OSD.Show(OSDType::MESSAGE_ERROR, std::string(n->T("AdhocServer Failed to Bind Port")) + " " + std::to_string(port), "portbindfail");
 		}
 
 		// Close Socket
 		closesocket(fd);
+	} else {
+		// Notify User
+		ERROR_LOG(Log::sceNet, "AdhocServer: Socket returned %i (Socket error %d)", fd, socket_errno);
 	}
 
-	// Notify User
-	else ERROR_LOG(SCENET, "AdhocServer: Socket returned %i (Socket error %d)", fd, errno);
-
 	// Return Error
 	return -1;
 }
@@ -1923,7 +1903,7 @@ int server_loop(int server)
 						u8 *pip = (u8*)&sip;
 						if (gethostbyname(str)->h_addrtype == AF_INET && gethostbyname(str)->h_addr_list[0] != NULL) pip = (u8*)gethostbyname(str)->h_addr_list[0];
 						sip = *(u32_le*)pip;
-						WARN_LOG(SCENET, "AdhocServer: Replacing IP %s with %s", inet_ntoa(addr.sin_addr), inet_ntoa(*(in_addr*)&pip));
+						WARN_LOG(Log::sceNet, "AdhocServer: Replacing IP %s with %s", inet_ntoa(addr.sin_addr), inet_ntoa(*(in_addr*)&pip));
 					}
 					*/
 					login_user_stream(loginresult, sip);
@@ -1939,10 +1919,10 @@ int server_loop(int server)
 			SceNetAdhocctlUserNode * next = user->next;
 
 			// Receive Data from User
-			int recvresult = recv(user->stream, (char*)user->rx + user->rxpos, sizeof(user->rx) - user->rxpos, MSG_NOSIGNAL);
+			int recvresult = (int)recv(user->stream, (char*)user->rx + user->rxpos, sizeof(user->rx) - user->rxpos, MSG_NOSIGNAL);
 
 			// Connection Closed or Timed Out
-			if(recvresult == 0 || (recvresult == -1 && errno != EAGAIN && errno != EWOULDBLOCK) || get_user_state(user) == USER_STATE_TIMED_OUT)
+			if(recvresult == 0 || (recvresult == -1 && socket_errno != EAGAIN && socket_errno != EWOULDBLOCK) || get_user_state(user) == USER_STATE_TIMED_OUT)
 			{
 				// Logout User
 				logout_user(user);
@@ -1985,7 +1965,7 @@ int server_loop(int server)
 					else
 					{
 						// Notify User
-						WARN_LOG(SCENET, "AdhocServer: Invalid Opcode 0x%02X in Waiting State from %s", user->rx[0], ip2str(*(in_addr*)&user->resolver.ip).c_str());
+						WARN_LOG(Log::sceNet, "AdhocServer: Invalid Opcode 0x%02X in Waiting State from %s", user->rx[0], ip2str(*(in_addr*)&user->resolver.ip).c_str());
 
 						// Logout User
 						logout_user(user);
@@ -2068,7 +2048,7 @@ int server_loop(int server)
 					else
 					{
 						// Notify User
-						WARN_LOG(SCENET, "AdhocServer: Invalid Opcode 0x%02X in Logged-In State from %s (MAC: %s - IP: %s)", user->rx[0], (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str());
+						WARN_LOG(Log::sceNet, "AdhocServer: Invalid Opcode 0x%02X in Logged-In State from %s (MAC: %s - IP: %s)", user->rx[0], (char *)user->resolver.name.data, mac2str(&user->resolver.mac).c_str(), ip2str(*(in_addr*)&user->resolver.ip).c_str());
 
 						// Logout User
 						logout_user(user);
@@ -2081,10 +2061,12 @@ int server_loop(int server)
 		}
 
 		// Prevent needless CPU Overload (1ms Sleep)
-		sleep_ms(10);
+		// !!!! COMMENT NOT REFLECTING REALITY
+		sleep_ms(10, "pro-adhoc-poll");
 
 		// Don't do anything if it's paused, otherwise the log will be flooded
-		while (adhocServerRunning && Core_IsStepping() && coreState != CORE_POWERDOWN) sleep_ms(10);
+		while (adhocServerRunning && Core_IsStepping() && coreState != CORE_POWERDOWN)
+			sleep_ms(10, "pro-adhot-paused-poll");
 	}
 
 	// Free User Database Memory
