@@ -21,12 +21,14 @@
 #include "Common/Serialize/SerializeFuncs.h"
 #include "Core/HLE/HLE.h"
 #include "Core/HLE/FunctionWrappers.h"
-#include "Core/Core.h"
+#include "Core/HLE/ErrorCodes.h"
 #include "Core/CoreTiming.h"
 #include "Core/MemMap.h"
 #include "Core/Reporting.h"
 #include "Core/Config.h"
+#include "Core/System.h"
 #include "Core/Compatibility.h"
+#include "Core/Debugger/MemBlockInfo.h"
 
 #include "Core/HLE/scePower.h"
 #include "Core/HLE/sceKernelThread.h"
@@ -119,7 +121,11 @@ int PowerBusMhzToHz(int mhz) {
 
 void __PowerInit() {
 	memset(powerCbSlots, 0, sizeof(powerCbSlots));
+
 	volatileMemLocked = false;
+	NotifyMemInfo(MemBlockFlags::ALLOC, 0x08400000, 0x400000, "Volatile memory (not locked)");
+	// The tags are functional, because scePower is initialized after sceKernelMemory
+
 	volatileWaitingThreads.clear();
 
 	if (GetLockedCPUSpeedMhz() > 0) {
@@ -165,59 +171,49 @@ void __PowerDoState(PointerWrap &p) {
 }
 
 static int scePowerGetBatteryLifePercent() {
-	DEBUG_LOG(HLE, "100=scePowerGetBatteryLifePercent");
-	return 100;
+	return hleLogDebug(Log::HLE, 100);
 }
 
 static int scePowerGetBatteryLifeTime() {
-	DEBUG_LOG(HLE, "0=scePowerGetBatteryLifeTime()");
 	// 0 means we're on AC power.
-	return 0;
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int scePowerGetBatteryTemp() {
-	DEBUG_LOG(HLE, "0=scePowerGetBatteryTemp()");
 	// 0 means celsius temperature of the battery
-	return 0;
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int scePowerIsPowerOnline() {
-	DEBUG_LOG(HLE, "1=scePowerIsPowerOnline");
-	return 1;
+	return hleLogDebug(Log::HLE, 1);
 }
 
 static int scePowerIsBatteryExist() {
-	DEBUG_LOG(HLE, "1=scePowerIsBatteryExist");
-	return 1;
+	return hleLogDebug(Log::HLE, 1);
 }
 
 static int scePowerIsBatteryCharging() {
-	DEBUG_LOG(HLE, "0=scePowerIsBatteryCharging");
-	return 0;
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int scePowerGetBatteryChargingStatus() {
-	DEBUG_LOG(HLE, "0=scePowerGetBatteryChargingStatus");
-	return 0;
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int scePowerIsLowBattery() {
-	DEBUG_LOG(HLE, "0=scePowerIsLowBattery");
-	return 0;
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int scePowerRegisterCallback(int slot, int cbId) {
-	DEBUG_LOG(HLE, "0=scePowerRegisterCallback(%i, %i)", slot, cbId);
-
 	if (slot < -1 || slot >= numberOfCBPowerSlotsPrivate) {
-		return PSP_POWER_ERROR_INVALID_SLOT;
+		return hleLogError(Log::HLE, PSP_POWER_ERROR_INVALID_SLOT);
 	}
 	if (slot >= numberOfCBPowerSlots) {
-		return SCE_KERNEL_ERROR_PRIV_REQUIRED;
+		return hleLogError(Log::HLE, SCE_KERNEL_ERROR_PRIV_REQUIRED);
 	}
 	// TODO: If cbId is invalid return PSP_POWER_ERROR_INVALID_CB.
 	if (cbId == 0) {
-		return PSP_POWER_ERROR_INVALID_CB;
+		return hleLogError(Log::HLE, PSP_POWER_ERROR_INVALID_CB);
 	}
 
 	int retval = -1;
@@ -230,67 +226,63 @@ static int scePowerRegisterCallback(int slot, int cbId) {
 			}
 		}
 		if (retval == -1) {
-			return PSP_POWER_ERROR_SLOTS_FULL;
+			return hleLogError(Log::HLE, PSP_POWER_ERROR_SLOTS_FULL);
 		}
 	} else {
 		if (powerCbSlots[slot] == 0) {
 			powerCbSlots[slot] = cbId;
 			retval = 0;
 		} else {
-			return PSP_POWER_ERROR_TAKEN_SLOT;
+			return hleLogError(Log::HLE, PSP_POWER_ERROR_TAKEN_SLOT);
 		}
 	}
 	if (retval >= 0) {
 		int arg = PSP_POWER_CB_AC_POWER | PSP_POWER_CB_BATTERY_EXIST | PSP_POWER_CB_BATTERY_FULL;
 		__KernelNotifyCallback(cbId, arg);
 	}
-	return retval;
+	return hleLogDebugOrError(Log::HLE, retval);
 }
 
 static int scePowerUnregisterCallback(int slotId) {
-	DEBUG_LOG(HLE, "0=scePowerUnregisterCallback(%i)", slotId);
+	DEBUG_LOG(Log::HLE, "0=scePowerUnregisterCallback(%i)", slotId);
 
 	if (slotId < 0 || slotId >= numberOfCBPowerSlotsPrivate) {
-		return PSP_POWER_ERROR_INVALID_SLOT;
+		return hleLogError(Log::HLE, PSP_POWER_ERROR_INVALID_SLOT);
 	}
 	if (slotId >= numberOfCBPowerSlots) {
-		return SCE_KERNEL_ERROR_PRIV_REQUIRED;
+		return hleLogError(Log::HLE, SCE_KERNEL_ERROR_PRIV_REQUIRED);
 	}
 
 	if (powerCbSlots[slotId] != 0) {
 		int cbId = powerCbSlots[slotId];
-		DEBUG_LOG(HLE, "0=scePowerUnregisterCallback(%i) (cbid = %i)", slotId, cbId);
 		powerCbSlots[slotId] = 0;
+		return hleLogDebug(Log::HLE, 0, "(cbid = %i)", cbId);
 	} else {
-		return PSP_POWER_ERROR_EMPTY_SLOT;
+		return hleLogError(Log::HLE, PSP_POWER_ERROR_EMPTY_SLOT);
 	}
-
-	return 0;
 }
 
 static int sceKernelPowerLock(int lockType) {
-	DEBUG_LOG(HLE, "0=sceKernelPowerLock(%i)", lockType);
 	if (lockType == 0) {
-		return 0;
+		return hleLogDebug(Log::HLE, 0);
 	} else {
-		return SCE_KERNEL_ERROR_INVALID_MODE;
+		return hleLogError(Log::HLE, SCE_KERNEL_ERROR_INVALID_MODE);
 	}
 }
 
 static int sceKernelPowerUnlock(int lockType) {
-	DEBUG_LOG(HLE, "0=sceKernelPowerUnlock(%i)", lockType);
 	if (lockType == 0) {
-		return 0;
+		return hleLogDebug(Log::HLE, 0);
 	} else {
-		return SCE_KERNEL_ERROR_INVALID_MODE;
+		return hleLogError(Log::HLE, SCE_KERNEL_ERROR_INVALID_MODE);
 	}
 }
 
 static int sceKernelPowerTick(int flag) {
-	DEBUG_LOG(HLE, "UNIMPL 0=sceKernelPowerTick(%i)", flag);
-	return 0;
+	return hleLogDebug(Log::HLE, 0, "UNIMPL");
 }
 
+// not a syscall
 int KernelVolatileMemLock(int type, u32 paddr, u32 psize) {
 	if (type != 0) {
 		return SCE_KERNEL_ERROR_INVALID_MODE;
@@ -303,13 +295,13 @@ int KernelVolatileMemLock(int type, u32 paddr, u32 psize) {
 	// It's always available in the emu.
 	// TODO: Should really reserve this properly!
 	if (Memory::IsValidAddress(paddr)) {
-		Memory::Write_U32(0x08400000, paddr);
+		Memory::WriteUnchecked_U32(0x08400000, paddr);
 	}
 	if (Memory::IsValidAddress(psize)) {
-		Memory::Write_U32(0x00400000, psize);
+		Memory::WriteUnchecked_U32(0x00400000, psize);
 	}
 	volatileMemLocked = true;
-
+	NotifyMemInfo(MemBlockFlags::ALLOC, 0x08400000, 0x400000, "Volatile memory (locked)");
 	return 0;
 }
 
@@ -323,19 +315,19 @@ static int sceKernelVolatileMemTryLock(int type, u32 paddr, u32 psize) {
 		// and with that it's still broken. So it's not this, unfortunately.
 		// Leaving it in for the 0.9.8 release anyway.
 		hleEatCycles(500000);
-		DEBUG_LOG(HLE, "sceKernelVolatileMemTryLock(%i, %08x, %08x) - success", type, paddr, psize);
+		DEBUG_LOG(Log::HLE, "sceKernelVolatileMemTryLock(%i, %08x, %08x) - success", type, paddr, psize);
 		break;
 
 	case SCE_KERNEL_ERROR_POWER_VMEM_IN_USE:
-		ERROR_LOG(HLE, "sceKernelVolatileMemTryLock(%i, %08x, %08x) - already locked!", type, paddr, psize);
-		break;
+		// This is OK, let's not ERROR_LOG.
+		return hleLogDebug(Log::HLE, error, "(%i, %08x, %08x) - already locked!", type, paddr, psize);
 
 	default:
-		ERROR_LOG_REPORT(HLE, "%08x=sceKernelVolatileMemTryLock(%i, %08x, %08x) - error", type, paddr, psize, error);
+		ERROR_LOG_REPORT(Log::HLE, "%08x=sceKernelVolatileMemTryLock(%i, %08x, %08x) - error", type, paddr, psize, error);
 		break;
 	}
 
-	return error;
+	return hleLogDebugOrError(Log::HLE, error);
 }
 
 int KernelVolatileMemUnlock(int type) {
@@ -348,6 +340,7 @@ int KernelVolatileMemUnlock(int type) {
 	}
 
 	volatileMemLocked = false;
+	NotifyMemInfo(MemBlockFlags::ALLOC, 0x08400000, 0x400000, "Volatile memory (not locked)");
 
 	// Wake someone, always fifo.
 	bool wokeThreads = false;
@@ -365,7 +358,7 @@ int KernelVolatileMemUnlock(int type) {
 	}
 
 	if (wokeThreads) {
-		INFO_LOG(HLE, "KernelVolatileMemUnlock(%i) handed over to another thread", type);
+		INFO_LOG(Log::HLE, "KernelVolatileMemUnlock(%i) handed over to another thread", type);
 		hleReSchedule("volatile mem unlocked");
 	}
 	return 0;
@@ -374,14 +367,14 @@ int KernelVolatileMemUnlock(int type) {
 static int sceKernelVolatileMemUnlock(int type) {
 	int error = KernelVolatileMemUnlock(type);
 	if (error == SCE_KERNEL_ERROR_INVALID_MODE) {
-		ERROR_LOG_REPORT(HLE, "sceKernelVolatileMemUnlock(%i) - invalid mode", type);
+		ERROR_LOG_REPORT(Log::HLE, "sceKernelVolatileMemUnlock(%i) - invalid mode", type);
 		return error;
 	} else if (error == SCE_KERNEL_ERROR_SEMA_OVF) {
-		ERROR_LOG_REPORT(HLE, "sceKernelVolatileMemUnlock(%i) FAILED - not locked", type);
+		ERROR_LOG_REPORT(Log::HLE, "sceKernelVolatileMemUnlock(%i) FAILED - not locked", type);
 		return error;
 	}
 
-	return hleLogSuccessI(HLE, 0);
+	return hleLogDebug(Log::HLE, 0);
 }
 
 static int sceKernelVolatileMemLock(int type, u32 paddr, u32 psize) {
@@ -401,12 +394,12 @@ static int sceKernelVolatileMemLock(int type, u32 paddr, u32 psize) {
 	case 0:
 		// Should only wait 1200 cycles though according to Unknown's testing,
 		hleEatCycles(1200);
-		DEBUG_LOG(HLE, "sceKernelVolatileMemLock(%i, %08x, %08x) - success", type, paddr, psize);
+		DEBUG_LOG(Log::HLE, "sceKernelVolatileMemLock(%i, %08x, %08x) - success", type, paddr, psize);
 		break;
 
 	case SCE_KERNEL_ERROR_POWER_VMEM_IN_USE:
 		{
-			WARN_LOG(HLE, "sceKernelVolatileMemLock(%i, %08x, %08x) - already locked, waiting", type, paddr, psize);
+			WARN_LOG(Log::HLE, "sceKernelVolatileMemLock(%i, %08x, %08x) - already locked, waiting", type, paddr, psize);
 			const VolatileWaitingThread waitInfo = { __KernelGetCurThread(), paddr, psize };
 			volatileWaitingThreads.push_back(waitInfo);
 			__KernelWaitCurThread(WAITTYPE_VMEM, 1, 0, 0, false, "volatile mem waited");
@@ -415,7 +408,7 @@ static int sceKernelVolatileMemLock(int type, u32 paddr, u32 psize) {
 
 	case SCE_KERNEL_ERROR_CAN_NOT_WAIT:
 		{
-			WARN_LOG(HLE, "sceKernelVolatileMemLock(%i, %08x, %08x): dispatch disabled", type, paddr, psize);
+			WARN_LOG(Log::HLE, "sceKernelVolatileMemLock(%i, %08x, %08x): dispatch disabled", type, paddr, psize);
 			Memory::Write_U32(0x08400000, paddr);
 			Memory::Write_U32(0x00400000, psize);
 		}
@@ -423,37 +416,37 @@ static int sceKernelVolatileMemLock(int type, u32 paddr, u32 psize) {
 
 	case SCE_KERNEL_ERROR_ILLEGAL_CONTEXT:
 		{
-			WARN_LOG(HLE, "sceKernelVolatileMemLock(%i, %08x, %08x): in interrupt", type, paddr, psize);
+			WARN_LOG(Log::HLE, "sceKernelVolatileMemLock(%i, %08x, %08x): in interrupt", type, paddr, psize);
 			Memory::Write_U32(0x08400000, paddr);
 			Memory::Write_U32(0x00400000, psize);
 		}
 		break;
 
 	default:
-		ERROR_LOG_REPORT(HLE, "%08x=sceKernelVolatileMemLock(%i, %08x, %08x) - error", type, paddr, psize, error);
+		ERROR_LOG_REPORT(Log::HLE, "%08x=sceKernelVolatileMemLock(%i, %08x, %08x) - error", type, paddr, psize, error);
 		break;
 	}
 
-	return error;
+	return hleNoLog(error);
 }
 
 
 static u32 scePowerSetClockFrequency(u32 pllfreq, u32 cpufreq, u32 busfreq) {
 	// 190 might (probably) be a typo for 19, but it's what the actual PSP validates against.
 	if (pllfreq < 19 || pllfreq < cpufreq || pllfreq > 333) {
-		return hleLogWarning(SCEMISC, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid pll frequency");
+		return hleLogWarning(Log::sceMisc, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid pll frequency");
 	}
 	if (cpufreq == 0 || cpufreq > 333) {
-		return hleLogWarning(SCEMISC, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid cpu frequency");
+		return hleLogWarning(Log::sceMisc, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid cpu frequency");
 	}
 	if (busfreq == 0 || busfreq > 166) {
-		return hleLogWarning(SCEMISC, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid bus frequency");
+		return hleLogWarning(Log::sceMisc, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid bus frequency");
 	}
 	// TODO: More restrictions.
 	if (GetLockedCPUSpeedMhz() > 0) {
-		INFO_LOG(HLE, "scePowerSetClockFrequency(%i,%i,%i): locked by user config at %i, %i, %i", pllfreq, cpufreq, busfreq, GetLockedCPUSpeedMhz(), GetLockedCPUSpeedMhz(), busFreq);
+		INFO_LOG(Log::HLE, "scePowerSetClockFrequency(%i,%i,%i): locked by user config at %i, %i, %i", pllfreq, cpufreq, busfreq, GetLockedCPUSpeedMhz(), GetLockedCPUSpeedMhz(), busFreq);
 	} else {
-		INFO_LOG(HLE, "scePowerSetClockFrequency(%i,%i,%i)", pllfreq, cpufreq, busfreq);
+		INFO_LOG(Log::HLE, "scePowerSetClockFrequency(%i,%i,%i)", pllfreq, cpufreq, busfreq);
 	}
 	// Only reschedules when the stepped PLL frequency changes.
 	// It seems like the busfreq parameter has no effect (but can cause errors.)
@@ -476,30 +469,30 @@ static u32 scePowerSetClockFrequency(u32 pllfreq, u32 cpufreq, u32 busfreq) {
 		else if ((newPll == 266 && oldPll == 333) || (newPll == 333 && oldPll == 266))
 			usec = 16600;
 
-		return hleDelayResult(0, "scepower set clockFrequency", usec);
+		return hleDelayResult(hleNoLog(0), "scepower set clockFrequency", usec);
 	}
 	if (GetLockedCPUSpeedMhz() <= 0)
 		CoreTiming::SetClockFrequencyHz(PowerCpuMhzToHz(cpufreq, pllFreq));
-	return 0;
+	return hleNoLog(0);
 }
 
 static u32 scePowerSetCpuClockFrequency(u32 cpufreq) {
 	if (cpufreq == 0 || cpufreq > 333) {
-		return hleLogWarning(SCEMISC, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid frequency");
+		return hleLogWarning(Log::sceMisc, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid frequency");
 	}
 	if (GetLockedCPUSpeedMhz() > 0) {
-		return hleLogDebug(SCEMISC, 0, "locked by user config at %i", GetLockedCPUSpeedMhz());
+		return hleLogDebug(Log::sceMisc, 0, "locked by user config at %i", GetLockedCPUSpeedMhz());
 	}
 	CoreTiming::SetClockFrequencyHz(PowerCpuMhzToHz(cpufreq, pllFreq));
-	return hleLogSuccessI(SCEMISC, 0);
+	return hleLogDebug(Log::sceMisc, 0);
 }
 
 static u32 scePowerSetBusClockFrequency(u32 busfreq) {
 	if (busfreq == 0 || busfreq > 111) {
-		return hleLogWarning(SCEMISC, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid frequency");
+		return hleLogWarning(Log::sceMisc, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid frequency");
 	}
 	if (GetLockedCPUSpeedMhz() > 0) {
-		return hleLogDebug(SCEMISC, 0, "locked by user config at %i", GetLockedCPUSpeedMhz() / 2);
+		return hleLogDebug(Log::sceMisc, 0, "locked by user config at %i", GetLockedCPUSpeedMhz() / 2);
 	}
 
 	// The value passed is validated, but then doesn't seem to matter for the result.
@@ -516,49 +509,45 @@ static u32 scePowerSetBusClockFrequency(u32 busfreq) {
 	else
 		busFreq = pllFreq / 2;
 
-	return hleLogSuccessI(SCEMISC, 0);
+	return hleLogDebug(Log::sceMisc, 0);
 }
 
 static u32 scePowerGetCpuClockFrequencyInt() {
 	int cpuFreq = CoreTiming::GetClockFrequencyHz() / 1000000;
-	return hleLogSuccessI(SCEMISC, cpuFreq);
+	return hleLogDebug(Log::sceMisc, cpuFreq);
 }
 
 static u32 scePowerGetPllClockFrequencyInt() {
-	return hleLogSuccessInfoI(SCEMISC, pllFreq / 1000000);
+	return hleLogDebug(Log::sceMisc, pllFreq / 1000000);
 }
 
 static u32 scePowerGetBusClockFrequencyInt() {
-	return hleLogSuccessInfoI(SCEMISC, busFreq / 1000000);
+	return hleLogDebug(Log::sceMisc, busFreq / 1000000);
 }
 
 static float scePowerGetCpuClockFrequencyFloat() {
 	float cpuFreq = CoreTiming::GetClockFrequencyHz() / 1000000.0f;
-	DEBUG_LOG(SCEMISC, "%f=scePowerGetCpuClockFrequencyFloat()", (float)cpuFreq);
+	DEBUG_LOG(Log::sceMisc, "%f=scePowerGetCpuClockFrequencyFloat()", (float)cpuFreq);
 	return cpuFreq;
 }
 
 static float scePowerGetPllClockFrequencyFloat() {
-	INFO_LOG(SCEMISC, "%f=scePowerGetPllClockFrequencyFloat()", (float)pllFreq / 1000000.0f);
+	INFO_LOG(Log::sceMisc, "%f=scePowerGetPllClockFrequencyFloat()", (float)pllFreq / 1000000.0f);
 	return (float) pllFreq / 1000000.0f;
 }
 
 static float scePowerGetBusClockFrequencyFloat() {
-	INFO_LOG(SCEMISC, "%f=scePowerGetBusClockFrequencyFloat()", (float)busFreq / 1000000.0f);
+	INFO_LOG(Log::sceMisc, "%f=scePowerGetBusClockFrequencyFloat()", (float)busFreq / 1000000.0f);
 	return (float) busFreq / 1000000.0f;
 }
 
 static int scePowerTick() {
-	DEBUG_LOG(SCEMISC, "scePowerTick()");
 	// Don't think we need to do anything.
-	return 0;
+	return hleLogDebug(Log::sceMisc, 0);
 }
 
-
 static u32 IsPSPNonFat() {
-	DEBUG_LOG(SCEMISC, "%d=scePower_a85880d0_IsPSPNonFat()", g_Config.iPSPModel);
-
-	return g_Config.iPSPModel;  
+	return hleLogDebug(Log::sceMisc, g_Config.iPSPModel);
 }
 
 static const HLEFunction scePower[] = {
@@ -641,9 +630,9 @@ const HLEFunction sceSuspendForUser[] = {
 
 
 void Register_scePower() {
-	RegisterModule("scePower",ARRAY_SIZE(scePower),scePower);
+	RegisterHLEModule("scePower",ARRAY_SIZE(scePower),scePower);
 }
 
 void Register_sceSuspendForUser() {
-	RegisterModule("sceSuspendForUser", ARRAY_SIZE(sceSuspendForUser), sceSuspendForUser);
+	RegisterHLEModule("sceSuspendForUser", ARRAY_SIZE(sceSuspendForUser), sceSuspendForUser);
 }
