@@ -87,12 +87,12 @@ JitBlockCache::~JitBlockCache() {
 bool JitBlock::ContainsAddress(u32 em_address) const {
 	// WARNING - THIS DOES NOT WORK WITH JIT INLINING ENABLED.
 	// However, that doesn't exist yet so meh.
-	return (em_address >= originalAddress && em_address < originalAddress + 4 * originalSize);
+	return em_address >= originalAddress && em_address < originalAddress + 4 * originalSize;
 }
 
 bool JitBlockCache::IsFull() const {
-	// -10 to safely leave space for some proxy blocks, which we don't check before we allocate (not ideal, but should work).
-	return num_blocks_ >= MAX_NUM_BLOCKS - 10;
+	// Subtract some amount to safely leave space for some proxy blocks, which we don't check before we allocate (not ideal, but should be enough).
+	return num_blocks_ >= MAX_NUM_BLOCKS - 512;
 }
 
 void JitBlockCache::Init() {
@@ -120,10 +120,12 @@ void JitBlockCache::Shutdown() {
 // This clears the JIT cache. It's called from JitCache.cpp when the JIT cache
 // is full and when saving and loading states.
 void JitBlockCache::Clear() {
+	// Note: We intentionally clear the block_map_ first to avoid O(N^2) behavior in RemoveBlockMap
 	block_map_.clear();
-	proxyBlockMap_.clear();
-	for (int i = 0; i < num_blocks_; i++)
+	for (int i = 0; i < num_blocks_; i++) {
 		DestroyBlock(i, DestroyType::CLEAR);
+	}
+	proxyBlockMap_.clear();
 	links_to_.clear();
 	num_blocks_ = 0;
 
@@ -146,6 +148,8 @@ const JitBlock *JitBlockCache::GetBlock(int no) const {
 }
 
 int JitBlockCache::AllocateBlock(u32 startAddress) {
+	_assert_(num_blocks_ < MAX_NUM_BLOCKS);
+
 	JitBlock &b = blocks_[num_blocks_];
 
 	b.proxyFor = 0;
@@ -177,11 +181,13 @@ int JitBlockCache::AllocateBlock(u32 startAddress) {
 }
 
 void JitBlockCache::ProxyBlock(u32 rootAddress, u32 startAddress, u32 size, const u8 *codePtr) {
+	_assert_(num_blocks_ < MAX_NUM_BLOCKS);
+
 	// If there's an existing block at the startAddress, add rootAddress as a proxy root of that block
 	// instead of creating a new block.
 	int num = GetBlockNumberFromStartAddress(startAddress, false);
 	if (num != -1) {
-		DEBUG_LOG(HLE, "Adding proxy root %08x to block at %08x", rootAddress, startAddress);
+		DEBUG_LOG(Log::HLE, "Adding proxy root %08x to block at %08x", rootAddress, startAddress);
 		if (!blocks_[num].proxyFor) {
 			blocks_[num].proxyFor = new std::vector<u32>();
 		}
@@ -214,7 +220,6 @@ void JitBlockCache::ProxyBlock(u32 rootAddress, u32 startAddress, u32 size, cons
 void JitBlockCache::AddBlockMap(int block_num) {
 	const JitBlock &b = blocks_[block_num];
 	// Convert the logical address to a physical address for the block map
-	// Yeah, this'll work fine for PSP too I think.
 	u32 pAddr = b.originalAddress & 0x1FFFFFFF;
 	block_map_[std::make_pair(pAddr + 4 * b.originalSize, pAddr)] = block_num;
 }
@@ -231,8 +236,10 @@ void JitBlockCache::RemoveBlockMap(int block_num) {
 		block_map_.erase(it);
 	} else {
 		// It wasn't in there, or it has the wrong key.  Let's search...
+		// TODO: This is O(n), so O(n^2) when called for every block.
 		for (auto it = block_map_.begin(); it != block_map_.end(); ++it) {
 			if (it->second == (u32)block_num) {
+				_dbg_assert_(false);
 				block_map_.erase(it);
 				break;
 			}
@@ -314,7 +321,7 @@ bool JitBlockCache::RangeMayHaveEmuHacks(u32 start, u32 end) const {
 
 static int binary_search(const JitBlock blocks_[], const u8 *baseoff, int imin, int imax) {
 	while (imin < imax) {
-		int imid = (imin + imax) / 2;
+		int imid = (imin + imax) >> 1;
 		if (blocks_[imid].normalEntry < baseoff)
 			imin = imid + 1;
 		else
@@ -334,7 +341,7 @@ int JitBlockCache::GetBlockNumberFromEmuHackOp(MIPSOpcode inst, bool ignoreBad)
 	const u8 *baseoff = codeBlock_->GetBasePtr() + off;
 	if (baseoff < codeBlock_->GetBasePtr() || baseoff >= codeBlock_->GetCodePtr()) {
 		if (!ignoreBad) {
-			ERROR_LOG(JIT, "JitBlockCache: Invalid Emuhack Op %08x", inst.encoding);
+			ERROR_LOG(Log::JIT, "JitBlockCache: Invalid Emuhack Op %08x", inst.encoding);
 		}
 		return -1;
 	}
@@ -451,7 +458,7 @@ void JitBlockCache::LinkBlock(int i) {
 	if (ppp.first == ppp.second)
 		return;
 	for (auto iter = ppp.first; iter != ppp.second; ++iter) {
-		// INFO_LOG(JIT, "Linking block %i to block %i", iter->second, i);
+		// INFO_LOG(Log::JIT, "Linking block %i to block %i", iter->second, i);
 		LinkBlockExits(iter->second);
 	}
 }
@@ -464,7 +471,7 @@ void JitBlockCache::UnlinkBlock(int i) {
 	for (auto iter = ppp.first; iter != ppp.second; ++iter) {
 		if ((size_t)iter->second >= num_blocks_) {
 			// Something probably went very wrong. Try to stumble along nevertheless.
-			ERROR_LOG(JIT, "UnlinkBlock: Invalid block number %d", iter->second);
+			ERROR_LOG(Log::JIT, "UnlinkBlock: Invalid block number %d", iter->second);
 			continue;
 		}
 		JitBlock &sourceBlock = blocks_[iter->second];
@@ -497,9 +504,9 @@ std::vector<u32> JitBlockCache::SaveAndClearEmuHackOps() {
 	return result;
 }
 
-void JitBlockCache::RestoreSavedEmuHackOps(std::vector<u32> saved) {
+void JitBlockCache::RestoreSavedEmuHackOps(const std::vector<u32> &saved) {
 	if (num_blocks_ != (int)saved.size()) {
-		ERROR_LOG(JIT, "RestoreSavedEmuHackOps: Wrong saved block size.");
+		ERROR_LOG(Log::JIT, "RestoreSavedEmuHackOps: Wrong saved block size.");
 		return;
 	}
 
@@ -516,7 +523,7 @@ void JitBlockCache::RestoreSavedEmuHackOps(std::vector<u32> saved) {
 
 void JitBlockCache::DestroyBlock(int block_num, DestroyType type) {
 	if (block_num < 0 || block_num >= num_blocks_) {
-		ERROR_LOG_REPORT(JIT, "DestroyBlock: Invalid block number %d", block_num);
+		ERROR_LOG_REPORT(Log::JIT, "DestroyBlock: Invalid block number %d", block_num);
 		return;
 	}
 	JitBlock *b = &blocks_[block_num];
@@ -554,7 +561,7 @@ void JitBlockCache::DestroyBlock(int block_num, DestroyType type) {
 
 	if (b->invalid) {
 		if (type == DestroyType::INVALIDATE)
-			ERROR_LOG(JIT, "Invalidating invalid block %d", block_num);
+			ERROR_LOG(Log::JIT, "Invalidating invalid block %d", block_num);
 		return;
 	}
 
@@ -581,7 +588,7 @@ void JitBlockCache::DestroyBlock(int block_num, DestroyType type) {
 			MIPSComp::jit->UnlinkBlock(writableEntry, b->originalAddress);
 		}
 	} else {
-		ERROR_LOG(JIT, "Unlinking block with no entry: %08x (%d)", b->originalAddress, block_num);
+		ERROR_LOG(Log::JIT, "Unlinking block with no entry: %08x (%d)", b->originalAddress, block_num);
 	}
 }
 
@@ -591,7 +598,7 @@ void JitBlockCache::InvalidateICache(u32 address, const u32 length) {
 	const u32 pEnd = pAddr + length;
 
 	if (pEnd < pAddr) {
-		ERROR_LOG(JIT, "Bad InvalidateICache: %08x with len=%d", address, length);
+		ERROR_LOG(Log::JIT, "Bad InvalidateICache: %08x with len=%d", address, length);
 		return;
 	}
 
@@ -637,7 +644,7 @@ void JitBlockCache::InvalidateChangedBlocks() {
 		}
 
 		if (changed) {
-			DEBUG_LOG(JIT, "Invalidating changed block at %08x", b.originalAddress);
+			DEBUG_LOG(Log::JIT, "Invalidating changed block at %08x", b.originalAddress);
 			DestroyBlock(block_num, DestroyType::INVALIDATE);
 		}
 	}
@@ -655,6 +662,9 @@ int JitBlockCache::GetBlockExitSize() {
 #elif PPSSPP_ARCH(RISCV64)
 	// Will depend on the sequence found to encode the destination address.
 	return 0;
+#elif PPSSPP_ARCH(LOONGARCH64)
+	// Will depend on the sequence found to encode the destination address.
+	return 0;
 #else
 #warning GetBlockExitSize unimplemented
 	return 0;
@@ -681,7 +691,6 @@ void JitBlockCache::ComputeStats(BlockCacheStats &bcStats) const {
 			bcStats.maxBloatBlock = b->originalAddress;
 		}
 		totalBloat += bloat;
-		bcStats.bloatMap[(float)bloat] = b->originalAddress;
 	}
 	bcStats.numBlocks = num_blocks_;
 	bcStats.minBloat = (float)minBloat;
@@ -709,7 +718,8 @@ JitBlockDebugInfo JitBlockCache::GetBlockDebugInfo(int blockNum) const {
 	debugInfo.targetDisasm = DisassembleX86(block->normalEntry, block->codeSize);
 #elif PPSSPP_ARCH(RISCV64)
 	debugInfo.targetDisasm = DisassembleRV64(block->normalEntry, block->codeSize);
+#elif PPSSPP_ARCH(LOONGARCH64)
+	debugInfo.targetDisasm = DisassembleLA64(block->normalEntry, block->codeSize);
 #endif
-
 	return debugInfo;
 }
