@@ -17,8 +17,6 @@
 
 #include "ppsspp_config.h"
 #include <algorithm>
-#include <map>
-#include <set>
 #include <unordered_map>
 #include <unordered_set>
 #include <mutex>
@@ -514,6 +512,16 @@ static const HardHashTableEntry hardcodedHashes[] = {
 	{ 0x795d940ad0a605f8, 40, "gow_fps_hack", }, // God of War (all)
 	{ 0x4c75043b7b0c643b, 512, "gow_vortex_hack", }, // God of War: Ghost of Sparta vortex timer hack, avoids softlock #8299
 	{ 0x7624dde603717640, 288, "ZZT3_select_hack", }, // Zettai Zetsumei Toshi 3 - bypasses softlock on character select screen #4901
+	{ 0x0dc5ca84f707863c, 452, "blitz_fps_hack", }, // Blitz: Overtime
+	{ 0xf93d3cd093595a6c, 856, "brian_lara_fps_hack", }, // Brian Lara 2007: Pressure Play
+	{ 0xc1d4af42a4c8860f, 964, "persona1_download_frame", },  // Persona 1 (issue #13079)
+	{ 0xde4286b2e7f6d3c1, 304, "persona2_download_frame", },  // Persona 2 (issue #13079)
+	{ 0x478c9c93d02011af, 1068, "steinsgate_download_frame", }, // Steins;Gate (issue #18262)
+	{ 0xb85dbe639f7acf13, 312, "infinity_download_frame", },  // Never7, Ever17, Remember11
+	{ 0x649d3305344f7afd, 464, "takuyo_1_download_frame", },
+	{ 0x505ad2073e4381d7, 428, "takuyo_2_download_frame", },
+	{ 0x0f4dbf28320798d7, 1160, "takuyo_3_download_frame", },
+	{ 0xf2bbff4807c5dab2, 556, "kingdomhearts_download_frame", },
 };
 
 namespace MIPSAnalyst {
@@ -735,7 +743,7 @@ namespace MIPSAnalyst {
 		}
 		totalUsedRegs += numUsedRegs;
 		numAnalyzings++;
-		VERBOSE_LOG(CPU, "[ %08x ] Used regs: %i Average: %f", address, numUsedRegs, (float)totalUsedRegs / (float)numAnalyzings);
+		VERBOSE_LOG(Log::CPU, "[ %08x ] Used regs: %i Average: %f", address, numUsedRegs, (float)totalUsedRegs / (float)numAnalyzings);
 
 		return results;
 	}
@@ -904,33 +912,6 @@ skip:
 		}
 	}
 
-	void PrecompileFunction(u32 startAddr, u32 length) {
-		// Direct calls to this ignore the bPreloadFunctions flag, since it's just for stubs.
-		std::lock_guard<std::recursive_mutex> guard(MIPSComp::jitLock);
-		if (MIPSComp::jit) {
-			MIPSComp::jit->CompileFunction(startAddr, length);
-		}
-	}
-
-	void PrecompileFunctions() {
-		if (!g_Config.bPreloadFunctions) {
-			return;
-		}
-		std::lock_guard<std::recursive_mutex> guard(functions_lock);
-
-		// TODO: Load from cache file if available instead.
-
-		double st = time_now_d();
-		for (auto iter = functions.begin(), end = functions.end(); iter != end; iter++) {
-			const AnalyzedFunction &f = *iter;
-
-			PrecompileFunction(f.start, f.end - f.start + 4);
-		}
-		double et = time_now_d();
-
-		NOTICE_LOG(JIT, "Precompiled %d MIPS functions in %0.2f milliseconds", (int)functions.size(), (et - st) * 1000.0);
-	}
-
 	static const char *DefaultFunctionName(char buffer[256], u32 startAddr) {
 		snprintf(buffer, 256, "z_un_%08x", startAddr);
 		return buffer;
@@ -1021,6 +1002,8 @@ skip:
 	}
 
 	bool ScanForFunctions(u32 startAddr, u32 endAddr, bool insertSymbols) {
+		_assert_((startAddr & 3) == 0);
+
 		std::lock_guard<std::recursive_mutex> guard(functions_lock);
 
 		FunctionsVector new_functions;
@@ -1202,6 +1185,8 @@ skip:
 	}
 
 	void RegisterFunction(u32 startAddr, u32 size, const char *name) {
+		_assert_((startAddr & 3) == 0);
+
 		std::lock_guard<std::recursive_mutex> guard(functions_lock);
 
 		// Check if we have this already
@@ -1217,7 +1202,7 @@ skip:
 					hashMap.insert(hfun);
 					return;
 				} else if (!iter->hasHash || size == 0) {
-					ERROR_LOG(HLE, "%s: %08x %08x : match but no hash (%i) or no size", name, startAddr, size, iter->hasHash);
+					ERROR_LOG(Log::HLE, "%s: %08x %08x : match but no hash (%i) or no size", name, startAddr, size, iter->hasHash);
 				}
 			}
 		}
@@ -1235,6 +1220,8 @@ skip:
 	}
 
 	void ForgetFunctions(u32 startAddr, u32 endAddr) {
+		_assert_(((startAddr | endAddr) & 3) == 0);
+
 		std::lock_guard<std::recursive_mutex> guard(functions_lock);
 
 		// It makes sense to forget functions as modules are unloaded but it breaks
@@ -1327,7 +1314,7 @@ skip:
 
 		FILE *file = File::OpenCFile(filename, "wt");
 		if (!file) {
-			WARN_LOG(LOADER, "Could not store hash map: %s", filename.c_str());
+			WARN_LOG(Log::Loader, "Could not store hash map: %s", filename.c_str());
 			return;
 		}
 
@@ -1335,7 +1322,7 @@ skip:
 			const HashMapFunc &mf = *it;
 			if (!mf.hardcoded) {
 				if (fprintf(file, "%016llx:%d = %s\n", mf.hash, mf.size, mf.name) <= 0) {
-					WARN_LOG(LOADER, "Could not store hash map: %s", filename.c_str());
+					WARN_LOG(Log::Loader, "Could not store hash map: %s", filename.c_str());
 					break;
 				}
 			}
@@ -1384,7 +1371,7 @@ skip:
 	void LoadHashMap(const Path &filename) {
 		FILE *file = File::OpenCFile(filename, "rt");
 		if (!file) {
-			WARN_LOG(LOADER, "Could not load hash map: %s", filename.c_str());
+			WARN_LOG(Log::Loader, "Could not load hash map: %s", filename.c_str());
 			return;
 		}
 		hashmapFileName = filename;
@@ -1395,7 +1382,7 @@ skip:
 			if (fscanf(file, "%llx:%d = %63s\n", &mf.hash, &mf.size, mf.name) < 3) {
 				char temp[1024];
 				if (!fgets(temp, 1024, file)) {
-					WARN_LOG(LOADER, "Could not read from hash map: %s", filename.c_str());
+					WARN_LOG(Log::Loader, "Could not read from hash map: %s", filename.c_str());
 				}
 				continue;
 			}
@@ -1422,7 +1409,7 @@ skip:
 		return vec;
 	}
 
-	MipsOpcodeInfo GetOpcodeInfo(DebugInterface* cpu, u32 address) {
+	MipsOpcodeInfo GetOpcodeInfo(DebugInterface *cpu, u32 address) {
 		MipsOpcodeInfo info;
 		memset(&info, 0, sizeof(info));
 
