@@ -21,22 +21,16 @@
 #include "Common/Profiler/Profiler.h"
 
 #include "Common/Log.h"
+#include "Common/TimeUtil.h"
 #include "Common/File/FileUtil.h"
 #include "Common/GraphicsContext.h"
-#include "Common/Serialize/Serializer.h"
-#include "Common/TimeUtil.h"
-#include "Common/Thread/ThreadUtil.h"
 
 #include "Core/Config.h"
-#include "Core/Debugger/Breakpoints.h"
-#include "Core/MemMapHelpers.h"
 #include "Core/Reporting.h"
 #include "Core/System.h"
 #include "Core/ELF/ParamSFO.h"
 
 #include "GPU/GPUState.h"
-#include "GPU/ge_constants.h"
-#include "GPU/GeDisasm.h"
 #include "GPU/Common/FramebufferManagerCommon.h"
 #include "GPU/Vulkan/ShaderManagerVulkan.h"
 #include "GPU/Vulkan/GPU_Vulkan.h"
@@ -48,8 +42,9 @@
 
 GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 	: GPUCommonHW(gfxCtx, draw), drawEngine_(draw) {
+	_assert_(draw);
+
 	gstate_c.SetUseFlags(CheckGPUFeatures());
-	drawEngine_.InitDeviceObjects();
 
 	VulkanContext *vulkan = (VulkanContext *)gfxCtx->GetAPIContext();
 
@@ -64,6 +59,7 @@ GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 	drawEngineCommon_ = &drawEngine_;
 	shaderManager_ = shaderManagerVulkan_;
 
+	drawEngine_.SetGPUCommon(this);
 	drawEngine_.SetTextureCache(textureCacheVulkan_);
 	drawEngine_.SetFramebufferManager(framebufferManagerVulkan_);
 	drawEngine_.SetShaderManager(shaderManagerVulkan_);
@@ -72,22 +68,21 @@ GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 	framebufferManagerVulkan_->SetTextureCache(textureCacheVulkan_);
 	framebufferManagerVulkan_->SetDrawEngine(&drawEngine_);
 	framebufferManagerVulkan_->SetShaderManager(shaderManagerVulkan_);
-	framebufferManagerVulkan_->Init(msaaLevel_);
 	textureCacheVulkan_->SetFramebufferManager(framebufferManagerVulkan_);
 	textureCacheVulkan_->SetShaderManager(shaderManagerVulkan_);
 	textureCacheVulkan_->SetDrawEngine(&drawEngine_);
 
-	InitDeviceObjects();
-
 	// Sanity check gstate
 	if ((int *)&gstate.transferstart - (int *)&gstate != 0xEA) {
-		ERROR_LOG(G3D, "gstate has drifted out of sync!");
+		ERROR_LOG(Log::G3D, "gstate has drifted out of sync!");
 	}
 
 	BuildReportingInfo();
 
 	textureCache_->NotifyConfigChanged();
 
+	drawEngine_.InitDeviceObjects();  // Creates important things like the pipeline layout. Required for loading the disk cache.
+
 	// Load shader cache.
 	std::string discID = g_paramSFO.GetDiscID();
 	if (discID.size()) {
@@ -95,15 +90,21 @@ GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 		shaderCachePath_ = GetSysDirectory(DIRECTORY_APP_CACHE) / (discID + ".vkshadercache");
 		LoadCache(shaderCachePath_);
 	}
+
+	InitDeviceObjects();
+}
+
+void GPU_Vulkan::FinishInitOnMainThread() {
+	// This can end up stopping/starting the vulkan render manager.
+	framebufferManagerVulkan_->Init(msaaLevel_);
 }
 
 void GPU_Vulkan::LoadCache(const Path &filename) {
 	if (!g_Config.bShaderCache) {
-		WARN_LOG(G3D, "Shader cache disabled. Not loading.");
+		WARN_LOG(Log::G3D, "Shader cache disabled. Not loading.");
 		return;
 	}
 
-	PSP_SetLoading("Loading shader cache...");
 	// Actually precompiled by IsReady() since we're single-threaded.
 	FILE *f = File::OpenCFile(filename, "rb");
 	if (!f)
@@ -112,9 +113,9 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 	// First compile shaders to SPIR-V, then load the pipeline cache and recreate the pipelines.
 	// It's when recreating the pipelines that the pipeline cache is useful - in the ideal case,
 	// it can just memcpy the finished shader binaries out of the pipeline cache file.
-	bool result = shaderManagerVulkan_->LoadCacheFlags(f, &drawEngine_);
+	bool result = ShaderManagerVulkan::LoadCacheFlags(f, &drawEngine_);
 	if (!result) {
-		WARN_LOG(G3D, "ShaderManagerVulkan failed to load cache header.");
+		WARN_LOG(Log::G3D, "ShaderManagerVulkan failed to load cache header.");
 	}
 	if (result) {
 		// Reload use flags in case LoadCacheFlags() changed them.
@@ -124,7 +125,7 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 		gstate_c.SetUseFlags(CheckGPUFeatures());
 		result = shaderManagerVulkan_->LoadCache(f);
 		if (!result) {
-			WARN_LOG(G3D, "ShaderManagerVulkan failed to load cache.");
+			WARN_LOG(Log::G3D, "ShaderManagerVulkan failed to load cache.");
 		}
 	}
 	if (result) {
@@ -133,24 +134,32 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 	}
 	fclose(f);
 
+	// Now, since we're on the loader thread, we can just block here until all pipelines are actually created.
+	// This makes it so that the on-screen spinner keeps spinning until we are done.
+	double start = time_now_d();
+	VulkanRenderManager *rm = (VulkanRenderManager *)draw_->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
+	int maxTasksSeen = rm->WaitForPipelines();
+	double seconds = time_now_d() - start;
+	INFO_LOG(Log::G3D, "Waited %0.1fms for at least %d pipeline tasks to finish compiling.", seconds * 1000.0, maxTasksSeen);
+
 	if (!result) {
-		WARN_LOG(G3D, "Incompatible Vulkan pipeline cache - rebuilding.");
+		WARN_LOG(Log::G3D, "Incompatible Vulkan pipeline cache - rebuilding.");
 		// Bad cache file for this GPU/Driver/etc. Delete it.
 		File::Delete(filename);
 	} else {
-		INFO_LOG(G3D, "Loaded Vulkan pipeline cache.");
+		INFO_LOG(Log::G3D, "Loaded Vulkan pipeline cache.");
 	}
 }
 
 void GPU_Vulkan::SaveCache(const Path &filename) {
 	if (!g_Config.bShaderCache) {
-		INFO_LOG(G3D, "Shader cache disabled. Not saving.");
+		INFO_LOG(Log::G3D, "Shader cache disabled. Not saving.");
 		return;
 	}
 
 	if (!draw_) {
 		// Already got the lost message, we're in shutdown.
-		WARN_LOG(G3D, "Not saving shaders - shutting down from in-game.");
+		WARN_LOG(Log::G3D, "Not saving shaders - shutting down from in-game.");
 		return;
 	}
 
@@ -160,7 +169,7 @@ void GPU_Vulkan::SaveCache(const Path &filename) {
 	shaderManagerVulkan_->SaveCache(f, &drawEngine_);
 	// WARNING: See comment in LoadCache if you are tempted to flip the second parameter to true.
 	pipelineManager_->SavePipelineCache(f, false, shaderManagerVulkan_, draw_);
-	INFO_LOG(G3D, "Saved Vulkan pipeline cache");
+	INFO_LOG(Log::G3D, "Saved Vulkan pipeline cache");
 	fclose(f);
 }
 
@@ -265,7 +274,7 @@ u32 GPU_Vulkan::CheckGPUFeatures() const {
 		if ((fmt4444 & Draw::FMT_TEXTURE) && (fmt565 & Draw::FMT_TEXTURE) && (fmt1555 & Draw::FMT_TEXTURE)) {
 			features |= GPU_USE_16BIT_FORMATS;
 		} else {
-			INFO_LOG(G3D, "Deficient texture format support: 4444: %d  1555: %d  565: %d", fmt4444, fmt1555, fmt565);
+			INFO_LOG(Log::G3D, "Deficient texture format support: 4444: %d  1555: %d  565: %d", fmt4444, fmt1555, fmt565);
 		}
 	}
 
@@ -306,7 +315,7 @@ void GPU_Vulkan::BeginHostFrame() {
 	if (gstate_c.useFlagsChanged) {
 		// TODO: It'd be better to recompile them in the background, probably?
 		// This most likely means that saw equal depth changed.
-		WARN_LOG(G3D, "Shader use flags changed, clearing all shaders and depth buffers");
+		WARN_LOG(Log::G3D, "Shader use flags changed, clearing all shaders and depth buffers");
 		// TODO: Not all shaders need to be recompiled. In fact, quite few? Of course, depends on
 		// the use flag change.. This is a major frame rate hitch in the start of a race in Outrun.
 		shaderManager_->ClearShaders();
@@ -316,7 +325,7 @@ void GPU_Vulkan::BeginHostFrame() {
 	}
 
 	if (dumpNextFrame_) {
-		NOTICE_LOG(G3D, "DUMPING THIS FRAME");
+		NOTICE_LOG(Log::G3D, "DUMPING THIS FRAME");
 		dumpThisFrame_ = true;
 		dumpNextFrame_ = false;
 	} else if (dumpThisFrame_) {
@@ -388,7 +397,7 @@ void GPU_Vulkan::FinishDeferred() {
 }
 
 void GPU_Vulkan::InitDeviceObjects() {
-	INFO_LOG(G3D, "GPU_Vulkan::InitDeviceObjects");
+	INFO_LOG(Log::G3D, "GPU_Vulkan::InitDeviceObjects");
 
 	uint32_t hacks = 0;
 	if (PSP_CoreParameter().compat.flags().MGS2AcidHack)
@@ -406,7 +415,7 @@ void GPU_Vulkan::InitDeviceObjects() {
 }
 
 void GPU_Vulkan::DestroyDeviceObjects() {
-	INFO_LOG(G3D, "GPU_Vulkan::DestroyDeviceObjects");
+	INFO_LOG(Log::G3D, "GPU_Vulkan::DestroyDeviceObjects");
 	// Need to turn off hacks when shutting down the GPU. Don't want them running in the menu.
 	if (draw_) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw_->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
@@ -447,7 +456,7 @@ void GPU_Vulkan::DeviceLost() {
 }
 
 void GPU_Vulkan::DeviceRestore(Draw::DrawContext *draw) {
-	GPUCommonHW::DeviceRestore(draw);
+	GPUCommonHW::DeviceRestore(draw);  // this updates draw_.
 
 	VulkanContext *vulkan = (VulkanContext *)draw_->GetNativeObject(Draw::NativeObject::CONTEXT);
 	pipelineManager_->DeviceRestore(vulkan);
