@@ -33,8 +33,7 @@
 #include "SHA1.h"
 
 /* ------------------------- KEY VAULT ------------------------- */
-unsigned char keyvault[0x80][0x10] =
-{
+static const unsigned char keyvault[0x80][0x10] = {
     {0x2C, 0x92, 0xE5, 0x90, 0x2B, 0x86, 0xC1, 0x06, 0xB7, 0x2E, 0xEA, 0x6C, 0xD4, 0xEC, 0x72, 0x48},
     {0x05, 0x8D, 0xC8, 0x0B, 0x33, 0xA5, 0xBF, 0x9D, 0x56, 0x98, 0xFA, 0xE0, 0xD3, 0x71, 0x5E, 0x1F},
     {0xB8, 0x13, 0xC3, 0x5E, 0xC6, 0x44, 0x41, 0xE3, 0xDC, 0x3C, 0x16, 0xF5, 0xB4, 0x5E, 0x64, 0x84},
@@ -165,62 +164,56 @@ unsigned char keyvault[0x80][0x10] =
     {0x5F, 0x8C, 0x17, 0x9F, 0xC1, 0xB2, 0x1D, 0xF1, 0xF6, 0x36, 0x7A, 0x9C, 0xF7, 0xD3, 0xD4, 0x7C},
 };
 
-u8 kirk1_key[]  = {0x98, 0xC9, 0x40, 0x97, 0x5C, 0x1D, 0x10, 0xE8, 0x7F, 0xE6, 0x0E, 0xA3, 0xFD, 0x03, 0xA8, 0xBA};
-u8 kirk16_key[] = {0x47, 0x5E, 0x09, 0xF4, 0xA2, 0x37, 0xDA, 0x9B, 0xEF, 0xFF, 0x3B, 0xC0, 0x77, 0x14, 0x3D, 0x8A};
+static const u8 kirk1_key[]  = {0x98, 0xC9, 0x40, 0x97, 0x5C, 0x1D, 0x10, 0xE8, 0x7F, 0xE6, 0x0E, 0xA3, 0xFD, 0x03, 0xA8, 0xBA};
+static const u8 kirk16_key[] = {0x47, 0x5E, 0x09, 0xF4, 0xA2, 0x37, 0xDA, 0x9B, 0xEF, 0xFF, 0x3B, 0xC0, 0x77, 0x14, 0x3D, 0x8A};
 
 /* ECC Curves for Kirk 1 and Kirk 0x11 */
 // Common Curve paramters p and a
-static u8 ec_p[20] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
-static u8 ec_a[20] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC}; // mon
+static const u8 ec_p[20] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
+static const u8 ec_a[20] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC}; // mon
 
 // Kirk 0xC,0xD,0x10,0x11,(likely 0x12)- Unique curve parameters for b, N, and base point G for Kirk 0xC,0xD,0x10,0x11,(likely 0x12) service
 // Since public key is variable, it is not specified here
-static u8 ec_b2[20] = {0xA6, 0x8B, 0xED, 0xC3, 0x34, 0x18, 0x02, 0x9C, 0x1D, 0x3C, 0xE3, 0x3B, 0x9A, 0x32, 0x1F, 0xCC, 0xBB, 0x9E, 0x0F, 0x0B};// mon
-static u8 ec_N2[21] = {0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFF, 0xFF, 0xB5, 0xAE, 0x3C, 0x52, 0x3E, 0x63, 0x94, 0x4F, 0x21, 0x27};
-static u8 Gx2[20] = {0x12, 0x8E, 0xC4, 0x25, 0x64, 0x87, 0xFD, 0x8F, 0xDF, 0x64, 0xE2, 0x43, 0x7B, 0xC0, 0xA1, 0xF6, 0xD5, 0xAF, 0xDE, 0x2C };
-static u8 Gy2[20] = {0x59, 0x58, 0x55, 0x7E, 0xB1, 0xDB, 0x00, 0x12, 0x60, 0x42, 0x55, 0x24, 0xDB, 0xC3, 0x79, 0xD5, 0xAC, 0x5F, 0x4A, 0xDF };
+static const u8 ec_b2[20] = {0xA6, 0x8B, 0xED, 0xC3, 0x34, 0x18, 0x02, 0x9C, 0x1D, 0x3C, 0xE3, 0x3B, 0x9A, 0x32, 0x1F, 0xCC, 0xBB, 0x9E, 0x0F, 0x0B};// mon
+static const u8 ec_N2[21] = {0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFF, 0xFF, 0xB5, 0xAE, 0x3C, 0x52, 0x3E, 0x63, 0x94, 0x4F, 0x21, 0x27};
+static const u8 Gx2[20] = {0x12, 0x8E, 0xC4, 0x25, 0x64, 0x87, 0xFD, 0x8F, 0xDF, 0x64, 0xE2, 0x43, 0x7B, 0xC0, 0xA1, 0xF6, 0xD5, 0xAF, 0xDE, 0x2C };
+static const u8 Gy2[20] = {0x59, 0x58, 0x55, 0x7E, 0xB1, 0xDB, 0x00, 0x12, 0x60, 0x42, 0x55, 0x24, 0xDB, 0xC3, 0x79, 0xD5, 0xAC, 0x5F, 0x4A, 0xDF };
 
 // KIRK 1 - Unique curve parameters for b, N, and base point G
 // Since public key is hard coded, it is also included
 
-static u8 ec_b1[20] = {0x65, 0xD1, 0x48, 0x8C, 0x03, 0x59, 0xE2, 0x34, 0xAD, 0xC9, 0x5B, 0xD3, 0x90, 0x80, 0x14, 0xBD, 0x91, 0xA5, 0x25, 0xF9};
-static u8 ec_N1[21] = {0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x01, 0xB5, 0xC6, 0x17, 0xF2, 0x90, 0xEA, 0xE1, 0xDB, 0xAD, 0x8F};
-static u8 Gx1[20] = {0x22, 0x59, 0xAC, 0xEE, 0x15, 0x48, 0x9C, 0xB0, 0x96, 0xA8, 0x82, 0xF0, 0xAE, 0x1C, 0xF9, 0xFD, 0x8E, 0xE5, 0xF8, 0xFA };
-static u8 Gy1[20] = {0x60, 0x43, 0x58, 0x45, 0x6D, 0x0A, 0x1C, 0xB2, 0x90, 0x8D, 0xE9, 0x0F, 0x27, 0xD7, 0x5C, 0x82, 0xBE, 0xC1, 0x08, 0xC0 };
+static const u8 ec_b1[20] = {0x65, 0xD1, 0x48, 0x8C, 0x03, 0x59, 0xE2, 0x34, 0xAD, 0xC9, 0x5B, 0xD3, 0x90, 0x80, 0x14, 0xBD, 0x91, 0xA5, 0x25, 0xF9};
+static const u8 ec_N1[21] = {0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x01, 0xB5, 0xC6, 0x17, 0xF2, 0x90, 0xEA, 0xE1, 0xDB, 0xAD, 0x8F};
+static const u8 Gx1[20] = {0x22, 0x59, 0xAC, 0xEE, 0x15, 0x48, 0x9C, 0xB0, 0x96, 0xA8, 0x82, 0xF0, 0xAE, 0x1C, 0xF9, 0xFD, 0x8E, 0xE5, 0xF8, 0xFA };
+static const u8 Gy1[20] = {0x60, 0x43, 0x58, 0x45, 0x6D, 0x0A, 0x1C, 0xB2, 0x90, 0x8D, 0xE9, 0x0F, 0x27, 0xD7, 0x5C, 0x82, 0xBE, 0xC1, 0x08, 0xC0 };
 
-static u8 Px1[20] = {0xED, 0x9C, 0xE5, 0x82, 0x34, 0xE6, 0x1A, 0x53, 0xC6, 0x85, 0xD6, 0x4D, 0x51, 0xD0, 0x23, 0x6B, 0xC3, 0xB5, 0xD4, 0xB9 };
-static u8 Py1[20] = {0x04, 0x9D, 0xF1, 0xA0, 0x75, 0xC0, 0xE0, 0x4F, 0xB3, 0x44, 0x85, 0x8B, 0x61, 0xB7, 0x9B, 0x69, 0xA6, 0x3D, 0x2C, 0x39 };
+static const u8 Px1[20] = {0xED, 0x9C, 0xE5, 0x82, 0x34, 0xE6, 0x1A, 0x53, 0xC6, 0x85, 0xD6, 0x4D, 0x51, 0xD0, 0x23, 0x6B, 0xC3, 0xB5, 0xD4, 0xB9 };
+static const u8 Py1[20] = {0x04, 0x9D, 0xF1, 0xA0, 0x75, 0xC0, 0xE0, 0x4F, 0xB3, 0x44, 0x85, 0x8B, 0x61, 0xB7, 0x9B, 0x69, 0xA6, 0x3D, 0x2C, 0x39 };
 
 /* ------------------------- KEY VAULT END ------------------------- */
 
+  // Some randomly selected data for a "key" to add to each randomization
+static const u8 random_data[0x10] = { 0xA7, 0x2E, 0x4C, 0xB6, 0xC3, 0x34, 0xDF, 0x85, 0x70, 0x01, 0x49, 0xFC, 0xC0, 0x87, 0xC4, 0x77 };
+// Another randomly selected data for a "key" to add to each randomization
+static const u8 random_key[0x10] = { 0x07, 0xAB, 0xEF, 0xF8, 0x96, 0x8C, 0xF3, 0xD6, 0x14, 0xE0, 0xEB, 0xB2, 0x9D, 0x8B, 0x4E, 0x74 };
+
 /* ------------------------- INTERNAL STUFF ------------------------- */
-typedef struct blah
-{
+typedef struct blah {
   u8 fuseid[8]; //0
   u8 mesh[0x40];  //0x8
 } kirk16_data; //0x48
  
-typedef struct header_keys
-{
+typedef struct header_keys {
   u8 AES[16];
   u8 CMAC[16];
-}header_keys;  //small struct for temporary keeping AES & CMAC key from CMD1 header
-
-
-u32 g_fuse90;  // This is to match FuseID HW at BC100090 and BC100094
-u32 g_fuse94;
-
-AES_ctx aes_kirk1; //global
-u8 PRNG_DATA[0x14];
-
-char is_kirk_initialized; //"init" emulation
+} header_keys;  //small struct for temporary keeping AES & CMAC key from CMD1 header
 
 /* ------------------------- INTERNAL STUFF END ------------------------- */
 
 
 /* ------------------------- IMPLEMENTATION ------------------------- */
 
-int kirk_CMD0(u8* outbuff, u8* inbuff, int size, int generate_trash)
+int kirk_CMD0(KirkState *kirk, u8* outbuff, const u8* inbuff, int size, int generate_trash)
 {
   KIRK_CMD1_HEADER* header = (KIRK_CMD1_HEADER*)outbuff;
   header_keys *keys = (header_keys *)outbuff; //0-15 AES key, 16-31 CMAC key
@@ -230,14 +223,14 @@ int kirk_CMD0(u8* outbuff, u8* inbuff, int size, int generate_trash)
   u8 cmac_header_hash[16];
   u8 cmac_data_hash[16];
     
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
 
   memcpy(outbuff, inbuff, size);
    
   if(header->mode != KIRK_MODE_CMD1) return KIRK_INVALID_MODE;
   
   //FILL PREDATA WITH RANDOM DATA
-  if(generate_trash) kirk_CMD14(outbuff+sizeof(KIRK_CMD1_HEADER), header->data_offset);
+  if(generate_trash) kirk_CMD14(kirk, outbuff+sizeof(KIRK_CMD1_HEADER), header->data_offset);
   
   //Make sure data is 16 aligned
   chk_size = header->data_size;
@@ -256,28 +249,30 @@ int kirk_CMD0(u8* outbuff, u8* inbuff, int size, int generate_trash)
   memcpy(header->CMAC_data_hash, cmac_data_hash, 16);
   
   //ENCRYPT KEYS
-  AES_cbc_encrypt(&aes_kirk1, inbuff, outbuff, 16*2);
+  AES_cbc_encrypt(&kirk->aes_kirk1, inbuff, outbuff, 16*2);
   return KIRK_OPERATION_SUCCESS;
 }
 
-int kirk_CMD1(u8* outbuff, u8* inbuff, int size)
+// This one writes to inbuff.
+int kirk_CMD1(KirkState *kirk, u8* outbuff, u8* inbuff, int size)
 {
-  KIRK_CMD1_HEADER* header = (KIRK_CMD1_HEADER*)inbuff;
+  const KIRK_CMD1_HEADER* header = (const KIRK_CMD1_HEADER*)inbuff;
   header_keys keys; //0-15 AES key, 16-31 CMAC key
   AES_ctx k1;
 	
 	if(size < 0x90) return KIRK_INVALID_SIZE;
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
   if(header->mode != KIRK_MODE_CMD1) return KIRK_INVALID_MODE;
   
-  AES_cbc_decrypt(&aes_kirk1, inbuff, (u8*)&keys, 16*2); //decrypt AES & CMAC key to temp buffer
+  AES_cbc_decrypt(&kirk->aes_kirk1, inbuff, (u8*)&keys, 16*2); //decrypt AES & CMAC key to temp buffer
   
   if(header->ecdsa_hash == 1)
   {
   	SHA_CTX sha;
   	KIRK_CMD1_ECDSA_HEADER* eheader = (KIRK_CMD1_ECDSA_HEADER*) inbuff;
   	u8 kirk1_pub[40];
-  	u8 header_hash[20];u8 data_hash[20];
+  	u8 header_hash[20];
+	u8 data_hash[20];
   	ecdsa_set_curve(ec_p,ec_a,ec_b1,ec_N1,Gx1,Gy1);
   	memcpy(kirk1_pub,Px1,20);
   	memcpy(kirk1_pub+20,Py1,20);
@@ -299,7 +294,7 @@ int kirk_CMD1(u8* outbuff, u8* inbuff, int size)
 	  }
 
   } else  {
-    int ret = kirk_CMD10(inbuff, size);
+    int ret = kirk_CMD10(kirk, inbuff, size);
     if(ret != KIRK_OPERATION_SUCCESS) return ret;
   }
   
@@ -309,13 +304,13 @@ int kirk_CMD1(u8* outbuff, u8* inbuff, int size)
   return KIRK_OPERATION_SUCCESS;
 }
 
-int kirk_CMD4(u8* outbuff, u8* inbuff, int size)
+int kirk_CMD4(KirkState *kirk, u8* outbuff, const u8* inbuff, int size)
 {
-  KIRK_AES128CBC_HEADER *header = (KIRK_AES128CBC_HEADER*)inbuff;
-  u8* key;
+  const KIRK_AES128CBC_HEADER *header = (const KIRK_AES128CBC_HEADER*)inbuff;
+  const u8* key;
   AES_ctx aesKey;
   
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
   if(header->mode != KIRK_MODE_ENCRYPT_CBC) return KIRK_INVALID_MODE;
   if(header->data_size == 0) return KIRK_DATA_SIZE_ZERO;
   
@@ -332,18 +327,18 @@ int kirk_CMD4(u8* outbuff, u8* inbuff, int size)
 void kirk4(u8* outbuff, const u8* inbuff, size_t size, int keyId)
 {
   AES_ctx aesKey;
-  u8* key = kirk_4_7_get_key(keyId);
+  const u8* key = kirk_4_7_get_key(keyId);
   AES_set_key(&aesKey, key, 128);
   AES_cbc_encrypt(&aesKey, inbuff, outbuff, (int)size);
 }
 
-int kirk_CMD7(u8* outbuff, u8* inbuff, int size)
+int kirk_CMD7(KirkState *kirk, u8* outbuff, const u8* inbuff, int size)
 {
   KIRK_AES128CBC_HEADER *header = (KIRK_AES128CBC_HEADER*)inbuff;
-  u8* key;
+  const u8* key;
   AES_ctx aesKey;
   
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
   if(header->mode != KIRK_MODE_DECRYPT_CBC) return KIRK_INVALID_MODE;
   if(header->data_size == 0) return KIRK_DATA_SIZE_ZERO;
   
@@ -360,12 +355,13 @@ int kirk_CMD7(u8* outbuff, u8* inbuff, int size)
 void kirk7(u8* outbuff, const u8* inbuff, size_t size, int keyId)
 {
   AES_ctx aesKey;
-  u8* key = kirk_4_7_get_key(keyId);
+  const u8* key = kirk_4_7_get_key(keyId);
   AES_set_key(&aesKey, key, 128);
   AES_cbc_decrypt(&aesKey, inbuff, outbuff, (int)size);
 }
 
-int kirk_CMD10(u8* inbuff, int insize)
+// This one works in-place.
+int kirk_CMD10(KirkState *kirk, u8* inbuff, int insize)
 {
   KIRK_CMD1_HEADER* header = (KIRK_CMD1_HEADER*)inbuff;
   header_keys keys; //0-15 AES key, 16-31 CMAC key
@@ -374,13 +370,13 @@ int kirk_CMD10(u8* inbuff, int insize)
   AES_ctx cmac_key;
   int chk_size;
   
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
   if(!(header->mode == KIRK_MODE_CMD1 || header->mode == KIRK_MODE_CMD2 || header->mode == KIRK_MODE_CMD3)) return KIRK_INVALID_MODE;
   if(header->data_size == 0) return KIRK_DATA_SIZE_ZERO;
   
   if(header->mode == KIRK_MODE_CMD1)
   {
-    AES_cbc_decrypt(&aes_kirk1, inbuff, (u8*)&keys, 32); //decrypt AES & CMAC key to temp buffer
+    AES_cbc_decrypt(&kirk->aes_kirk1, inbuff, (u8*)&keys, 32); //decrypt AES & CMAC key to temp buffer
     AES_set_key(&cmac_key, keys.CMAC, 128);
     AES_CMAC(&cmac_key, inbuff+0x60, 0x30, cmac_header_hash);
   
@@ -397,11 +393,11 @@ int kirk_CMD10(u8* inbuff, int insize)
   return KIRK_SIG_CHECK_INVALID; //Checks for cmd 2 & 3 not included right now
 }
 
-int kirk_CMD11(u8* outbuff, u8* inbuff, int size)
+int kirk_CMD11(KirkState *kirk, u8* outbuff, const u8* inbuff, int size)
 {
   KIRK_SHA1_HEADER *header = (KIRK_SHA1_HEADER *)inbuff;
   SHA_CTX sha;
-  if(is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
+  if(kirk->is_kirk_initialized == 0) return KIRK_NOT_INITIALIZED;
   if(header->data_size == 0 || size == 0) return KIRK_DATA_SIZE_ZERO;
   
 	SHAInit(&sha);
@@ -413,23 +409,24 @@ int kirk_CMD11(u8* outbuff, u8* inbuff, int size)
 // Generate an ECDSA Key pair
 // offset 0 = private key (0x14 len)
 // offset 0x14 = public key point (0x28 len)
-int kirk_CMD12(u8 * outbuff, int outsize) {
+int kirk_CMD12(KirkState *kirk, u8 *outbuff, int outsize) {
   u8 k[0x15];
   KIRK_CMD12_BUFFER * keypair = (KIRK_CMD12_BUFFER *) outbuff;
 
   if(outsize != 0x3C) return KIRK_INVALID_SIZE;
   ecdsa_set_curve(ec_p,ec_a,ec_b2,ec_N2,Gx2,Gy2);
   k[0] = 0;
-  kirk_CMD14(k+1,0x14);
+  kirk_CMD14(kirk, k+1,0x14);
   ec_priv_to_pub(k, (u8*)keypair->public_key.x);
   memcpy(keypair->private_key,k+1,0x14);
   
   return KIRK_OPERATION_SUCCESS;
 }
+
 // Point multiplication
 // offset 0 = mulitplication value (0x14 len)
 // offset 0x14 = point to multiply (0x28 len)
-int kirk_CMD13(u8 * outbuff, int outsize,u8 * inbuff, int insize) {
+int kirk_CMD13(KirkState *kirk, u8 * outbuff, int outsize,u8 * inbuff, int insize) {
   u8 k[0x15];
   KIRK_CMD13_BUFFER * pointmult = (KIRK_CMD13_BUFFER *) inbuff;
   k[0]=0;
@@ -442,28 +439,31 @@ int kirk_CMD13(u8 * outbuff, int outsize,u8 * inbuff, int insize) {
   return KIRK_OPERATION_SUCCESS;
 }
 
-int kirk_CMD14(u8 * outbuff, int outsize) {
+int kirk_CMD14(KirkState *kirk, u8 *outbuff, int outsize) {
   u8 temp[0x104];
+  // This was added to mollify valgrind.
+  memset(temp, 0xAA, sizeof(temp));
+
   KIRK_SHA1_HEADER *header = (KIRK_SHA1_HEADER *) temp;
   
-  // Some randomly selected data for a "key" to add to each randomization
-  u8 key[0x10] = { 0xA7, 0x2E, 0x4C, 0xB6, 0xC3, 0x34, 0xDF, 0x85, 0x70, 0x01, 0x49, 0xFC, 0xC0, 0x87, 0xC4, 0x77 };
   u32 curtime;
   //if(outsize != 0x14) return KIRK_INVALID_SIZE; // Need real error code
   if(outsize <=0) return KIRK_OPERATION_SUCCESS;
     
-  memcpy(temp+4, PRNG_DATA,0x14);
+  memcpy(temp+4, kirk->PRNG_DATA,0x14);
   // This uses the standard C time function for portability.
-  curtime=(u32)time(0);
+  curtime = (u32)time(0);
   temp[0x18] = curtime &0xFF;
   temp[0x19] = (curtime>>8) &0xFF;
   temp[0x1A] = (curtime>>16) &0xFF;
   temp[0x1B] = (curtime>>24) &0xFF;
-  memcpy(&temp[0x1C], key, 0x10);
-  //This leaves the remainder of the 0x100 bytes in temp to whatever remains on the stack 
+  memcpy(&temp[0x1C], random_data, 0x10);
+  
+  // WARNING: These next two lines of comments are no longer accurate since I added the memset above.
+  // This leaves the remainder of the 0x100 bytes in temp to whatever remains on the stack 
   // in an uninitialized state. This should add unpredicableness to the results as well
   header->data_size=0x100;
-  kirk_CMD11(PRNG_DATA, temp, 0x104);
+  kirk_CMD11(kirk, kirk->PRNG_DATA, temp, 0x104);
   while(outsize)
   {
     int blockrem= outsize %0x14;
@@ -471,14 +471,14 @@ int kirk_CMD14(u8 * outbuff, int outsize) {
     
     if(block)
     {
-      memcpy(outbuff, PRNG_DATA, 0x14);
+      memcpy(outbuff, kirk->PRNG_DATA, 0x14);
       outbuff+=0x14;
       outsize -= 0x14;
-      kirk_CMD14(outbuff, outsize);
+      kirk_CMD14(kirk, outbuff, outsize);
     } else {
       if(blockrem)
       {
-        memcpy(outbuff, PRNG_DATA, blockrem);
+        memcpy(outbuff, kirk->PRNG_DATA, blockrem);
         outsize -= blockrem;
       }
     }
@@ -487,21 +487,21 @@ int kirk_CMD14(u8 * outbuff, int outsize) {
   return KIRK_OPERATION_SUCCESS;
 }
 
-void decrypt_kirk16_private(u8 *dA_out, u8 *dA_enc)
+void decrypt_kirk16_private(KirkState *kirk, u8 *dA_out, u8 *dA_enc)
 {
   int i, k;
   kirk16_data keydata;
   u8 subkey_1[0x10], subkey_2[0x10];
   rijndael_ctx aes_ctx;
 
-  keydata.fuseid[7] = g_fuse90 &0xFF;
-  keydata.fuseid[6] = (g_fuse90>>8) &0xFF;
-  keydata.fuseid[5] = (g_fuse90>>16) &0xFF;
-  keydata.fuseid[4] = (g_fuse90>>24) &0xFF; 
-  keydata.fuseid[3] = g_fuse94 &0xFF;
-  keydata.fuseid[2] = (g_fuse94>>8) &0xFF;
-  keydata.fuseid[1] = (g_fuse94>>16) &0xFF;
-  keydata.fuseid[0] = (g_fuse94>>24) &0xFF;
+  keydata.fuseid[7] = kirk->g_fuse90 &0xFF;
+  keydata.fuseid[6] = (kirk->g_fuse90>>8) &0xFF;
+  keydata.fuseid[5] = (kirk->g_fuse90>>16) &0xFF;
+  keydata.fuseid[4] = (kirk->g_fuse90>>24) &0xFF; 
+  keydata.fuseid[3] = kirk->g_fuse94 &0xFF;
+  keydata.fuseid[2] = (kirk->g_fuse94>>8) &0xFF;
+  keydata.fuseid[1] = (kirk->g_fuse94>>16) &0xFF;
+  keydata.fuseid[0] = (kirk->g_fuse94>>24) &0xFF;
  
   /* set encryption key */
   rijndael_set_key(&aes_ctx, kirk16_key, 128);
@@ -555,7 +555,7 @@ void decrypt_kirk16_private(u8 *dA_out, u8 *dA_enc)
   AES_cbc_decrypt((AES_ctx *)&aes_ctx, dA_enc, dA_out, 0x20);
 }
  
-void encrypt_kirk16_private(u8 *dA_out, u8 *dA_dec)
+void encrypt_kirk16_private(KirkState *kirk, u8 *dA_out, u8 *dA_dec)
 {
   int i, k;
   kirk16_data keydata;
@@ -563,14 +563,14 @@ void encrypt_kirk16_private(u8 *dA_out, u8 *dA_dec)
   rijndael_ctx aes_ctx;
  
 
-  keydata.fuseid[7] = g_fuse90 &0xFF;
-  keydata.fuseid[6] = (g_fuse90>>8) &0xFF;
-  keydata.fuseid[5] = (g_fuse90>>16) &0xFF;
-  keydata.fuseid[4] = (g_fuse90>>24) &0xFF; 
-  keydata.fuseid[3] = g_fuse94 &0xFF;
-  keydata.fuseid[2] = (g_fuse94>>8) &0xFF;
-  keydata.fuseid[1] = (g_fuse94>>16) &0xFF;
-  keydata.fuseid[0] = (g_fuse94>>24) &0xFF;
+  keydata.fuseid[7] = kirk->g_fuse90 &0xFF;
+  keydata.fuseid[6] = (kirk->g_fuse90>>8) &0xFF;
+  keydata.fuseid[5] = (kirk->g_fuse90>>16) &0xFF;
+  keydata.fuseid[4] = (kirk->g_fuse90>>24) &0xFF; 
+  keydata.fuseid[3] = kirk->g_fuse94 &0xFF;
+  keydata.fuseid[2] = (kirk->g_fuse94>>8) &0xFF;
+  keydata.fuseid[1] = (kirk->g_fuse94>>16) &0xFF;
+  keydata.fuseid[0] = (kirk->g_fuse94>>24) &0xFF;
   /* set encryption key */
   rijndael_set_key(&aes_ctx, kirk16_key, 128);
  
@@ -623,18 +623,18 @@ void encrypt_kirk16_private(u8 *dA_out, u8 *dA_dec)
   AES_cbc_encrypt((AES_ctx *)&aes_ctx, dA_dec, dA_out, 0x20);
 }
 
-int kirk_CMD16(u8 * outbuff, int outsize, u8 * inbuff, int insize) {
+int kirk_CMD16(KirkState *kirk, u8 * outbuff, int outsize, u8 * inbuff, int insize) {
 	u8 dec_private[0x20];
 	KIRK_CMD16_BUFFER * signbuf = (KIRK_CMD16_BUFFER *) inbuff;
 	ECDSA_SIG * sig = (ECDSA_SIG *) outbuff;
 	if(insize != 0x34) return KIRK_INVALID_SIZE;
 	if(outsize != 0x28) return KIRK_INVALID_SIZE;
-	decrypt_kirk16_private(dec_private,signbuf->enc_private);
+	decrypt_kirk16_private(kirk, dec_private,signbuf->enc_private);
 	// Clear out the padding for safety
 	memset(&dec_private[0x14], 0, 0xC);
 	ecdsa_set_curve(ec_p,ec_a,ec_b2,ec_N2,Gx2,Gy2);
 	ecdsa_set_priv(dec_private);
-	ecdsa_sign(signbuf->message_hash,sig->r, sig->s);
+	ecdsa_sign(kirk, signbuf->message_hash,sig->r, sig->s);
   return KIRK_OPERATION_SUCCESS;
 }
 
@@ -644,7 +644,7 @@ int kirk_CMD16(u8 * outbuff, int outsize, u8 * inbuff, int insize) {
 // 28 = message hash (0x14 length)
 // 3C = signature R (0x14 length)
 // 50 = signature S (0x14 length)
-int kirk_CMD17(u8 * inbuff, int insize) {
+int kirk_CMD17(KirkState *kirk, const u8 * inbuff, int insize) {
 	KIRK_CMD17_BUFFER * sig = (KIRK_CMD17_BUFFER *) inbuff;
   if(insize != 0x64) return KIRK_INVALID_SIZE;
   ecdsa_set_curve(ec_p,ec_a,ec_b2,ec_N2,Gx2,Gy2);
@@ -657,19 +657,16 @@ int kirk_CMD17(u8 * inbuff, int insize) {
   }
 }
 
-
-
-int kirk_init()
+int kirk_init(KirkState *kirk)
 {
-  return kirk_init2((u8*)"Lazy Dev should have initialized!",33,0xBABEF00D, 0xDEADBEEF );
+  return kirk_init2(kirk, (u8*)"Lazy Dev should have initialized!",33,0xBABEF00D, 0xDEADBEEF );
 }
 
-int kirk_init2(u8 * rnd_seed, u32 seed_size, u32 fuseid_90, u32 fuseid_94) {
+int kirk_init2(KirkState *kirk, u8 * rnd_seed, u32 seed_size, u32 fuseid_90, u32 fuseid_94) {
   u8 temp[0x104];
+  memset(temp, 0xAA, sizeof(temp));
   
   KIRK_SHA1_HEADER *header = (KIRK_SHA1_HEADER *) temp;
-  // Another randomly selected data for a "key" to add to each randomization
-  u8 key[0x10] = {0x07, 0xAB, 0xEF, 0xF8, 0x96, 0x8C, 0xF3, 0xD6, 0x14, 0xE0, 0xEB, 0xB2, 0x9D, 0x8B, 0x4E, 0x74};
   u32 curtime;
 
   //Set PRNG_DATA initially, otherwise use what ever uninitialized data is in the buffer
@@ -677,42 +674,42 @@ int kirk_init2(u8 * rnd_seed, u32 seed_size, u32 fuseid_90, u32 fuseid_94) {
     u8 * seedbuf;
     KIRK_SHA1_HEADER *seedheader;
     seedbuf=(u8*)malloc(seed_size+4);
-    seedheader= (KIRK_SHA1_HEADER *) seedbuf;
+    memset(seedbuf, 0, seed_size+4);
+    seedheader = (KIRK_SHA1_HEADER *) seedbuf;
     seedheader->data_size = seed_size;
-    kirk_CMD11(PRNG_DATA, seedbuf, seed_size+4);    
+    kirk_CMD11(kirk, kirk->PRNG_DATA, seedbuf, seed_size+4);    
     free(seedbuf);
   }
-  memcpy(temp+4, PRNG_DATA,0x14);
+  memcpy(temp+4, kirk->PRNG_DATA,0x14);
   // This uses the standard C time function for portability.
   curtime=(u32)time(0);
   temp[0x18] = curtime &0xFF;
   temp[0x19] = (curtime>>8) &0xFF;
   temp[0x1A] = (curtime>>16) &0xFF;
   temp[0x1B] = (curtime>>24) &0xFF;
-  memcpy(&temp[0x1C], key, 0x10);
+  memcpy(&temp[0x1C], random_key, 0x10);
   //This leaves the remainder of the 0x100 bytes in temp to whatever remains on the stack 
   // in an uninitialized state. This should add unpredicableness to the results as well
   header->data_size=0x100;
-  kirk_CMD11(PRNG_DATA, temp, 0x104); 
+  kirk_CMD11(kirk, kirk->PRNG_DATA, temp, 0x104);
   
   //Set Fuse ID
-  g_fuse90=fuseid_90;
-  g_fuse94=fuseid_94;
+  kirk->g_fuse90=fuseid_90;
+  kirk->g_fuse94=fuseid_94;
   
   //Set KIRK1 main key
-  AES_set_key(&aes_kirk1, kirk1_key, 128);
-  
+  AES_set_key(&kirk->aes_kirk1, kirk1_key, 128);
 
-  is_kirk_initialized = 1;
+  kirk->is_kirk_initialized = 1;
   return 0;
 }
-u8* kirk_4_7_get_key(int key_type)
-{
-	if((key_type < 0) || (key_type >=0x80)) return (u8*)KIRK_INVALID_SIZE;
+
+const u8* kirk_4_7_get_key(int key_type){
+	if((key_type < 0) || (key_type >=0x80)) return (const u8*)KIRK_INVALID_SIZE;
 	return keyvault[key_type];
 }
 
-int kirk_CMD1_ex(u8* outbuff, u8* inbuff, int size, KIRK_CMD1_HEADER* header)
+int kirk_CMD1_ex(KirkState *kirk, u8* outbuff, u8* inbuff, int size, KIRK_CMD1_HEADER* header)
 {
   u8* buffer = (u8*)malloc(size);
   int ret;
@@ -720,26 +717,26 @@ int kirk_CMD1_ex(u8* outbuff, u8* inbuff, int size, KIRK_CMD1_HEADER* header)
   memcpy(buffer, header, sizeof(KIRK_CMD1_HEADER));
   memcpy(buffer+sizeof(KIRK_CMD1_HEADER), inbuff, header->data_size);
   
-  ret = kirk_CMD1(outbuff, buffer, size);
+  ret = kirk_CMD1(kirk, outbuff, buffer, size);
   free(buffer);
   return ret;
 }
 
-int kirk_sceUtilsBufferCopyWithRange(u8* outbuff, int outsize, const u8* inbuff, int insize, int cmd)
+int kirk_sceUtilsBufferCopyWithRange(KirkState *kirk, u8* outbuff, int outsize, u8* inbuff, int insize, int cmd)
 {
   // TODO: propagate const-correctness into all these functions.
   switch(cmd)
   {
-    case KIRK_CMD_DECRYPT_PRIVATE: return kirk_CMD1(outbuff, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_ENCRYPT_IV_0: return kirk_CMD4(outbuff, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_DECRYPT_IV_0: return kirk_CMD7(outbuff, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_PRIV_SIGN_CHECK: return kirk_CMD10((u8 *)inbuff, insize); break;
-    case KIRK_CMD_SHA1_HASH: return kirk_CMD11(outbuff, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_ECDSA_GEN_KEYS: return kirk_CMD12(outbuff, outsize); break;
-    case KIRK_CMD_ECDSA_MULTIPLY_POINT: return kirk_CMD13(outbuff, outsize, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_PRNG: return kirk_CMD14(outbuff, outsize); break;
-    case KIRK_CMD_ECDSA_SIGN: return kirk_CMD16(outbuff, outsize, (u8 *)inbuff, insize); break;
-    case KIRK_CMD_ECDSA_VERIFY: return kirk_CMD17((u8 *)inbuff, insize); break;
+    case KIRK_CMD_DECRYPT_PRIVATE: return kirk_CMD1(kirk, outbuff, inbuff, insize); break;  // NOTE: I think this actually trashes inbuff
+    case KIRK_CMD_ENCRYPT_IV_0: return kirk_CMD4(kirk, outbuff, inbuff, insize); break;
+    case KIRK_CMD_DECRYPT_IV_0: return kirk_CMD7(kirk, outbuff, inbuff, insize); break;
+    case KIRK_CMD_PRIV_SIGN_CHECK: return kirk_CMD10(kirk, inbuff, insize); break;
+    case KIRK_CMD_SHA1_HASH: return kirk_CMD11(kirk, outbuff, inbuff, insize); break;
+    case KIRK_CMD_ECDSA_GEN_KEYS: return kirk_CMD12(kirk, outbuff, outsize); break;
+    case KIRK_CMD_ECDSA_MULTIPLY_POINT: return kirk_CMD13(kirk, outbuff, outsize, inbuff, insize); break;
+    case KIRK_CMD_PRNG: return kirk_CMD14(kirk, outbuff, outsize); break;
+    case KIRK_CMD_ECDSA_SIGN: return kirk_CMD16(kirk, outbuff, outsize, inbuff, insize); break;
+    case KIRK_CMD_ECDSA_VERIFY: return kirk_CMD17(kirk, inbuff, insize); break;
   }
   return -1;
 }
