@@ -121,7 +121,7 @@ RC_EXPORT void* RC_CCONV rc_client_get_userdata(const rc_client_t* client);
 /**
  * Sets the name of the server to use.
  */
-RC_EXPORT void RC_CCONV rc_client_set_host(const rc_client_t* client, const char* hostname);
+RC_EXPORT void RC_CCONV rc_client_set_host(rc_client_t* client, const char* hostname);
 
 typedef uint64_t rc_clock_t;
 typedef rc_clock_t (RC_CCONV *rc_get_time_millisecs_func_t)(const rc_client_t* client);
@@ -188,6 +188,8 @@ typedef struct rc_client_user_t {
   uint32_t score;
   uint32_t score_softcore;
   uint32_t num_unread_messages;
+  /* minimum version: 12.0 */
+  const char* avatar_url;
 } rc_client_user_t;
 
 /**
@@ -217,10 +219,44 @@ typedef struct rc_client_user_game_summary_t {
  */
 RC_EXPORT void RC_CCONV rc_client_get_user_game_summary(const rc_client_t* client, rc_client_user_game_summary_t* summary);
 
+typedef struct rc_client_all_user_progress_entry_t {
+  uint32_t game_id;
+  uint32_t num_achievements;
+  uint32_t num_unlocked_achievements;
+  uint32_t num_unlocked_achievements_hardcore;
+} rc_client_all_user_progress_entry_t;
+
+typedef struct rc_client_all_user_progress_t {
+  rc_client_all_user_progress_entry_t* entries;
+  uint32_t num_entries;
+} rc_client_all_user_progress_t;
+
+/**
+ * Callback that is fired when an all progress query completes. list may be null if the query failed.
+ */
+typedef void(RC_CCONV* rc_client_fetch_all_user_progress_callback_t)(int result, const char* error_message,
+                                                                     rc_client_all_user_progress_t* list,
+                                                                     rc_client_t* client, void* callback_userdata);
+
+/**
+ * Starts an asynchronous request for all progress for the given console.
+ * This query returns the total number of achievements for all games tracked by this console, as well as
+ * the user's achievement unlock count for both softcore and hardcore modes.
+ */
+RC_EXPORT rc_client_async_handle_t* RC_CCONV
+rc_client_begin_fetch_all_user_progress(rc_client_t* client, uint32_t console_id,
+                                        rc_client_fetch_all_user_progress_callback_t callback, void* callback_userdata);
+
+/**
+ * Destroys a previously-allocated result from the rc_client_begin_fetch_all_progress_list() callback.
+ */
+RC_EXPORT void RC_CCONV rc_client_destroy_all_user_progress(rc_client_all_user_progress_t* list);
+
 /*****************************************************************************\
 | Game                                                                        |
 \*****************************************************************************/
 
+#ifdef RC_CLIENT_SUPPORTS_HASH
 /**
  * Start loading an unidentified game.
  */
@@ -228,6 +264,7 @@ RC_EXPORT rc_client_async_handle_t* RC_CCONV rc_client_begin_identify_and_load_g
     uint32_t console_id, const char* file_path,
     const uint8_t* data, size_t data_size,
     rc_client_callback_t callback, void* callback_userdata);
+#endif
 
 /**
  * Start loading a game.
@@ -249,6 +286,11 @@ enum {
   RC_CLIENT_LOAD_GAME_STATE_ABORTED
 };
 
+/**
+ * Determines if a game was successfully identified and loaded.
+ */
+RC_EXPORT int RC_CCONV rc_client_is_game_loaded(const rc_client_t* client);
+
 /**
  * Unloads the current game.
  */
@@ -260,10 +302,13 @@ typedef struct rc_client_game_t {
   const char* title;
   const char* hash;
   const char* badge_name;
+  /* minimum version: 12.0 */
+  const char* badge_url;
 } rc_client_game_t;
 
 /**
  * Get information about the current game. Returns NULL if no game is loaded.
+ * NOTE: returns a dummy game record if an unidentified game is loaded.
  */
 RC_EXPORT const rc_client_game_t* RC_CCONV rc_client_get_game_info(const rc_client_t* client);
 
@@ -273,11 +318,19 @@ RC_EXPORT const rc_client_game_t* RC_CCONV rc_client_get_game_info(const rc_clie
  */
 RC_EXPORT int RC_CCONV rc_client_game_get_image_url(const rc_client_game_t* game, char buffer[], size_t buffer_size);
 
+#ifdef RC_CLIENT_SUPPORTS_HASH
 /**
  * Changes the active disc in a multi-disc game.
  */
 RC_EXPORT rc_client_async_handle_t* RC_CCONV rc_client_begin_change_media(rc_client_t* client, const char* file_path,
     const uint8_t* data, size_t data_size, rc_client_callback_t callback, void* callback_userdata);
+#endif
+
+/**
+ * Changes the active disc in a multi-disc game.
+ */
+RC_EXPORT rc_client_async_handle_t* RC_CCONV rc_client_begin_change_media_from_hash(rc_client_t* client, const char* hash,
+    rc_client_callback_t callback, void* callback_userdata);
 
 /*****************************************************************************\
 | Subsets                                                                     |
@@ -290,10 +343,47 @@ typedef struct rc_client_subset_t {
 
   uint32_t num_achievements;
   uint32_t num_leaderboards;
+
+  /* minimum version: 12.0 */
+  const char* badge_url;
 } rc_client_subset_t;
 
 RC_EXPORT const rc_client_subset_t* RC_CCONV rc_client_get_subset_info(rc_client_t* client, uint32_t subset_id);
 
+/*****************************************************************************\
+| Fetch Game Hashes                                                           |
+\*****************************************************************************/
+
+typedef struct rc_client_hash_library_entry_t {
+  char hash[33];
+  uint32_t game_id;
+} rc_client_hash_library_entry_t;
+
+typedef struct rc_client_hash_library_t {
+  rc_client_hash_library_entry_t* entries;
+  uint32_t num_entries;
+} rc_client_hash_library_t;
+
+/**
+ * Callback that is fired when a hash library request completes. list may be null if the query failed.
+ */
+typedef void(RC_CCONV* rc_client_fetch_hash_library_callback_t)(int result, const char* error_message,
+                                                                rc_client_hash_library_t* list, rc_client_t* client,
+                                                                void* callback_userdata);
+
+/**
+ * Starts an asynchronous request for all hashes for the given console.
+ * This request returns a mapping from hashes to the game's unique identifier. A single game may have multiple
+ * hashes in the case of multi-disc games, or variants that are still compatible with the same achievement set.
+ */
+RC_EXPORT rc_client_async_handle_t* RC_CCONV rc_client_begin_fetch_hash_library(
+  rc_client_t* client, uint32_t console_id, rc_client_fetch_hash_library_callback_t callback, void* callback_userdata);
+
+/**
+ * Destroys a previously-allocated result from the rc_client_destroy_hash_library() callback.
+ */
+RC_EXPORT void RC_CCONV rc_client_destroy_hash_library(rc_client_hash_library_t* list);
+
 /*****************************************************************************\
 | Achievements                                                                |
 \*****************************************************************************/
@@ -357,6 +447,9 @@ typedef struct rc_client_achievement_t {
   float rarity;
   float rarity_hardcore;
   uint8_t type;
+  /* minimum version: 12.0 */
+  const char* badge_url;
+  const char* badge_locked_url;
 } rc_client_achievement_t;
 
 /**
@@ -371,7 +464,7 @@ RC_EXPORT const rc_client_achievement_t* RC_CCONV rc_client_get_achievement_info
 RC_EXPORT int RC_CCONV rc_client_achievement_get_image_url(const rc_client_achievement_t* achievement, int state, char buffer[], size_t buffer_size);
 
 typedef struct rc_client_achievement_bucket_t {
-  rc_client_achievement_t** achievements;
+  const rc_client_achievement_t** achievements;
   uint32_t num_achievements;
 
   const char* label;
@@ -380,7 +473,7 @@ typedef struct rc_client_achievement_bucket_t {
 } rc_client_achievement_bucket_t;
 
 typedef struct rc_client_achievement_list_t {
-  rc_client_achievement_bucket_t* buckets;
+  const rc_client_achievement_bucket_t* buckets;
   uint32_t num_buckets;
 } rc_client_achievement_list_t;
 
@@ -447,7 +540,7 @@ typedef struct rc_client_leaderboard_tracker_t {
 } rc_client_leaderboard_tracker_t;
 
 typedef struct rc_client_leaderboard_bucket_t {
-  rc_client_leaderboard_t** leaderboards;
+  const rc_client_leaderboard_t** leaderboards;
   uint32_t num_leaderboards;
 
   const char* label;
@@ -456,7 +549,7 @@ typedef struct rc_client_leaderboard_bucket_t {
 } rc_client_leaderboard_bucket_t;
 
 typedef struct rc_client_leaderboard_list_t {
-  rc_client_leaderboard_bucket_t* buckets;
+  const rc_client_leaderboard_bucket_t* buckets;
   uint32_t num_buckets;
 } rc_client_leaderboard_list_t;
 
@@ -501,6 +594,7 @@ typedef struct rc_client_leaderboard_entry_t {
 typedef struct rc_client_leaderboard_entry_list_t {
   rc_client_leaderboard_entry_t* entries;
   uint32_t num_entries;
+  uint32_t total_entries;
   int32_t user_index;
 } rc_client_leaderboard_entry_list_t;
 
@@ -602,7 +696,8 @@ enum {
   RC_CLIENT_EVENT_GAME_COMPLETED = 15, /* all achievements for the game have been earned */
   RC_CLIENT_EVENT_SERVER_ERROR = 16, /* an API response returned a [server_error] and will not be retried */
   RC_CLIENT_EVENT_DISCONNECTED = 17, /* an unlock request could not be completed and is pending */
-  RC_CLIENT_EVENT_RECONNECTED = 18 /* all pending unlocks have been completed */
+  RC_CLIENT_EVENT_RECONNECTED = 18, /* all pending unlocks have been completed */
+  RC_CLIENT_EVENT_SUBSET_COMPLETED = 19 /* all achievements for the subset have been earned */
 };
 
 typedef struct rc_client_server_error_t {
@@ -620,6 +715,7 @@ typedef struct rc_client_event_t {
   rc_client_leaderboard_tracker_t* leaderboard_tracker;
   rc_client_leaderboard_scoreboard_t* leaderboard_scoreboard;
   rc_client_server_error_t* server_error;
+  rc_client_subset_t* subset;
 
 } rc_client_event_t;
 
@@ -677,15 +773,29 @@ RC_EXPORT size_t RC_CCONV rc_client_progress_size(rc_client_t* client);
 /**
  * Serializes the runtime state into a buffer.
  * Returns RC_OK on success, or an error indicator.
+ * [deprecated] use rc_client_serialize_progress_sized instead
  */
 RC_EXPORT int RC_CCONV rc_client_serialize_progress(rc_client_t* client, uint8_t* buffer);
 
 /**
- * Deserializes the runtime state from a buffer.
+ * Serializes the runtime state into a buffer.
  * Returns RC_OK on success, or an error indicator.
  */
+RC_EXPORT int RC_CCONV rc_client_serialize_progress_sized(rc_client_t* client, uint8_t* buffer, size_t buffer_size);
+
+/**
+ * Deserializes the runtime state from a buffer.
+ * Returns RC_OK on success, or an error indicator.
+ * [deprecated] use rc_client_deserialize_progress_sized instead
+ */
 RC_EXPORT int RC_CCONV rc_client_deserialize_progress(rc_client_t* client, const uint8_t* serialized);
 
+/**
+ * Serializes the runtime state into a buffer.
+ * Returns RC_OK on success, or an error indicator.
+ */
+RC_EXPORT int RC_CCONV rc_client_deserialize_progress_sized(rc_client_t* client, const uint8_t* serialized, size_t serialized_size);
+
 RC_END_C_DECLS
 
 #endif /* RC_RUNTIME_H */
